#!/bin/bash
#Test aide

# DEFENITION
###############################################################################
date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
short_date_stamp=$(date "+%y%m%d%H%M%S" 2>/dev/null)

progname=$(basename $0 2>/dev/null)
working_dir="/tmp/audit_test_${date_stamp}"

cd $(dirname $0)

cpwd=$(pwd)
#log="${cpwd}/trace.log"
log=

tmp_dir="${cpwd}/tmp"
tmp_file_output="${tmp_dir}/tmp_output"
tmp_file_res="${tmp_dir}/tmp_res"

#test_dir="/tmp/test_aide_${date_stamp}"
test_dir="/tmp/test_aide"
test_dir_fs="${test_dir}/test_fs"
test_dir_change="${test_dir}/test_change"
test_dir_et="${test_dir}/test_etalon"

fs_tar="test_fs.tgz"
ch_tar="test_change.tgz"
et_tar="test_etalon.tgz"

aide_db=${test_dir}/aide.db
aide_db_new=${test_dir}/aide.db.new
aide_conf=${test_dir}/aide.conf

test_file_res="res.test.aide"
test_file_res_tmp="tmp.res.test.aide"
test_file_et="etalon.aide"

arch=
user=
utils=aide

aide_conf_file="
@@define TOPDIR ${test_dir}\n
database=file:@@{TOPDIR}/aide.db\n
database_out=file:@@{TOPDIR}/aide.db.new\n
verbose=20\n
${test_dir_fs} R\n
"


verbose=0
debug=0
###############################################################################


# Usage
###############################################################################
usage()
{
        echo " usage: $progname [-vdh]"
        echo
        exit 1
}
################################################################################

# Echo
###############################################################################
DoEcho()
{
	if [ $verbose != 0 ]; then 
		echo -e "${progname}: === $*" 
	fi
	echo -e "${progname}: === $*" >> $log
}
###############################################################################

# Result checking
###############################################################################
DoCheckResult()
{
	local return_code_to_check="$1"

	if [ -d ${tmp_dir} ]; then
		DoInputRes $return_code_to_check
	fi

	if [ "$return_code_to_check" -ne 0 ]; then
		exit "$return_code_to_check"
        fi
}
###############################################################################


# Input text formating
###############################################################################
DoInputRes()
{
	DoEcho "   | Output command:"
	while read LINE; do DoEcho "   | $LINE"; done < ${tmp_file_output}
	DoEcho "   | Result command:"
	if [ "$1" -eq 0 ]; then
		DoEcho "   | PASS"
	else
		DoEcho "   | FAIL"
	fi
}
###############################################################################

# Operation
###############################################################################
DoCmd()
{
	unset do_cmd_out
	unset do_cmd_out_add
	unset do_cmd_ret_code

	local cmd="$@"

	if [ -d ${tmp_dir} ]; then
		echo "" > ${tmp_file_output}
	fi

	let "oper = oper + 1"
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | ${cmd}"
	do_cmd_out=$(eval $cmd 2>>$log)
	do_cmd_ret_code=$?
	do_cmd_ret_code=$?
	if [ -d ${tmp_dir} ]; then
		echo "$do_cmd_out" > ${tmp_file_output}
	fi
	return "$do_cmd_ret_code"
}
###############################################################################


# Init
###############################################################################
DoInit()
{
#	oper=0
	DoEcho ""
	DoEcho "***************************************************************************"
	DoEcho " Test init"
	DoEcho "***************************************************************************"

	DoEcho "Making tmp dir"
	DoCmd "mkdir -p $tmp_dir"
	DoCheckResult "$?"

	DoEcho "Making test dir"
	DoCmd "mkdir -p $test_dir"
	DoCheckResult "$?"

	DoEcho "Checking utils"
	DoCmd "which aide"
	DoCheckResult "$?"

	DoEcho "Arch settings"
	DoCmd "uname -m"
	DoCheckResult "$?"
	arch=$do_cmd_out
	
	DoEcho "Checking user"
	DoCmd "whoami"
	DoCheckResult "$?"
	arch=$do_cmd_out
}
###############################################################################


# Tests FS creating
###############################################################################
DoCreateTestFS()
{


	DoEcho "Extracting test files, creating test filesystems"

	DoCmd tar xzf ${cpwd}/${fs_tar} -C ${test_dir}
	DoCheckResult "$?"

	DoCmd tar xzf ${cpwd}/${ch_tar} -C ${test_dir}
	DoCheckResult "$?"

	DoCmd tar xzf ${cpwd}/${et_tar} -C ${test_dir}
	DoCheckResult "$?"
	
	DoEcho "Creating ${aide_conf}"
	DoCmd "echo -e '${aide_conf_file}' 2>&1 | tee ${aide_conf}"
	DoCheckResult "$?"
}

#Initial aide DB
###############################################################################
DoInitial()
{
	DoEcho "Initializing AIDE DB"
	DoCmd "aide --init --config ${aide_conf}"
	DoCheckResult "$?"
	DoEcho "Copying test AIDE DB ${aide_db_new} in ${aide_db}"
	DoCmd "cp ${aide_db_new} ${aide_db}"
	DoCheckResult "$?"
}

#Action
###############################################################################
DoAction()
{
	
	DoEcho ""
	DoEcho "Changing test file"
	DoCmd "cp ${test_dir_change}/cp ${test_dir_fs}/bin/cp"
	DoCheckResult "$?"
 
	DoEcho "Changing test file owner"
	DoCmd "chown root:root ${test_dir_fs}/home/virus"
	DoCheckResult "$?"

	DoEcho "Changing test file mode bits"
	DoCmd "chmod a+x ${test_dir_fs}/home/virus"
	DoCheckResult "$?"

	DoEcho "Removing test file"
	DoCmd "rm ${test_dir_fs}/etc/passwd"
	DoCheckResult "$?"
	
	DoEcho "Changing test file"
	DoCmd "echo -e '\n142.168.77.33' >> ${test_dir_fs}/etc/hosts"
	DoCheckResult "$?"
	
}

#Check change
###############################################################################
DoCheck()
{

	DoEcho "Checking AIDE test filesystems "
	DoCmd "aide --check -V10 --config ${aide_conf} | tee ${test_dir}/${test_file_res_tmp}"
	DoCheckResult "$?"

	DoEcho "Checking the difference "
	DoCmd "grep differences ${test_dir}/${test_file_res_tmp}"
	DoCheckResult "$?"

	DoCmd "sed -n '/Summ/,\$p' ${test_dir}/${test_file_res_tmp} | sed '/time/d' | tee ${test_dir}/${test_file_res}"
	DoCheckResult "$?"
}


#Diff input
###############################################################################
DoResult()
{
	DoEcho "Diff result"
	DoCmd "diff ${test_dir}/${test_file_res} ${test_dir_et}/${test_file_et}"
	DoCheckResult "$?"
}

# User creating
###############################################################################
AddUser()
{
	DoEcho "Adding test_aide user"
	DoCmd "useradd -u 777 $1"
	DoCheckResult "$?"
}

# User deleting
###############################################################################
DelUser()
{
	DoEcho "Deleting test_aide user"
	DoCmd "userdel $1"
	DoCheckResult "$?"
}



# Clean
###############################################################################
DoCleanUp()
{
#	oper=0
	DoEcho "***************************************************************************"
	DoEcho " Cleanup test"
	DoEcho "***************************************************************************"

	DelUser test_aide
	
	if [ -d ${test_dir} ]; then
		DoCmd rm -rf ${test_dir}
		DoCheckResult "$?"
	fi
}
###############################################################################



trap "DoExit" EXIT
trap "exit 127" HUP INT QUIT PIPE TERM

###############################################################################
DoExit()
{
	local exit_code="$?"
	DoCleanUp
	date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
	DoEcho "Stopping AIDE TEST at $date_stamp"
	if [ "$exit_code" -eq 0 ]; then
		DoEcho "PASS"
	else
		DoEcho "FAIL"
	fi
}
###############################################################################





# MAIN
###############################################################################

OPTIONS="vdh"
while getopts $OPTIONS option
do
        case $option in
                v) verbose=1;;
                d) debug=1;;
		h) echo -e "\v - verbose"
		   echo -e "\d - debug mode (set -x)"
		   echo -e "\h - help" 
		   exit 1;;
                *) echo "$prog: illegal option -- $option"; usage; exit 1;;
        esac
done


log="${cpwd}/trace.${utils}.log"
if [ -f ${log} ]; then
	DoEcho "Removing old log"
	DoCmd rm  ${log}
	DoCheckResult "$?"
fi


DoEcho "Starting AIDE TEST at $date_stamp"

DoInit ${utils}

DoEcho ""
DoEcho "***************************************************************************"
DoEcho "1) Checking ${utils} utils"
DoEcho "***************************************************************************"

#oper=0


AddUser test_aide	&&
DoCreateTestFS		&&
DoInitial		&&
DoAction		&&
DoCheck			&&
DoResult

