#!/bin/bash
set +x

# DEFENITION
###############################################################################
date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
short_date_stamp=$(date "+%y%m%d%H%M%S" 2>/dev/null)

progname=$(basename $0 2>/dev/null)
working_dir="/tmp/audit_test_${date_stamp}"

cd $(dirname $0)

cpwd=$(pwd)
log="${cpwd}/trace.log"
tmp_dir="${cpwd}/tmp"
tmp_file="${tmp_dir}/tmp_res"

test_file_for_execve=`ls -l  /bin/ls 2>/dev/null | awk '{print $NF}'`

arch=
user=
file_search=
verbose=0
debug=0
###############################################################################


# Usage
###############################################################################
usage()
{
        echo " usage: $progname [-vdsh]"
        echo
        exit 1
}
################################################################################

# Echo
###############################################################################
DoEcho()
{
	if [ $verbose != 0 ]; then
		echo -e "${progname}: === $*"
	fi
	echo -e "${progname}: === $*" >> $log
}
###############################################################################

# Input text formating
###############################################################################
DoInputRes()
{
	DoEcho "   | Output command:"
	while read LINE; do DoEcho "   | $LINE"; done < ${tmp_file}
	DoEcho "   | Result command:"
	if [ "$1" -eq 0 ]; then
		DoEcho "   | PASS"
	else
		DoEcho "   | FAIL"
	fi
}
###############################################################################

# Clean
###############################################################################
DoCleanUp()
{
#	oper=0
DoEcho "***************************************************************************"
	DoEcho " Cleanup test"
DoEcho "***************************************************************************"

	DoCleanRules

	if [ -d ${working_dir} ]; then
		DoCmd rm -rf ${working_dir}
		DoCheckResult "$?"
	fi
	if [ -d ${tmp_dir} ]; then
		DoCmd rm -rf ${tmp_dir}
		DoCheckResult "$?"
	fi

	if [ -n "$stop" ] && [ "$stop" -eq 1 ]; then
		DoCmd1 /etc/init.d/auditd restart
		ret="$?"
		DoCheckResult "$ret"
	fi

}
###############################################################################

# Set test rules
###############################################################################
DoSetRules()
{
	DoCmd auditctl -a exit,always -F dir=$working_dir -S mkdir \
                   -S mkdirat -k create_object \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir -S mkdir -k create_object
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -F dir=$working_dir -S unlink \
                   -S unlinkat -k delete_object \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir -S unlink -k delete_object
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -F dir=$working_dir -S open \
                   -S openat -k object_open \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir -S open -k object_open
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -F dir=$working_dir \
                   -S rename -S renameat -S renameat2 \
        || DoCmd auditctl -a exit,always -F dir=$working_dir \
                   -S rename -S renameat \
        || DoCmd auditctl -a exit,always -F dir=$working_dir -S rename
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -F dir=$working_dir -S chown -S fchown \
                   -S fchownat -k change_object_owner \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir -S chown \
                    -S fchown -k change_object_owner \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir \
                    -S chown -k change_object_owner
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -F dir=$working_dir -S chmod -S fchmod \
                   -S fchmodat -k change_object_mod \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir -S chmod \
                    -S fchmod -k change_object_mod \
	    || DoCmd auditctl -a exit,always -F dir=$working_dir -S chmod \
                    -k change_object_mod
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -F path=$test_file_for_execve -S execve \
                   -S execveat -k object_exec_${date_stamp} \
	    || DoCmd auditctl -a exit,always -F path=$test_file_for_execve \
                          -S execve -k object_exec_${date_stamp}
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -S exit_group -k object_exec_exit_${date_stamp}
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -S mount -k mount_${date_stamp}
	DoCheckResult "$?"
	DoCmd auditctl -a exit,always -S umount -S umount2 -k umount_${date_stamp} \
	    || DoCmd auditctl -a exit,always -S umount -k umount_${date_stamp} \
	    || DoCmd auditctl -a exit,always -S umount2 -k umount_${date_stamp}
	DoCheckResult "$?"
}
###############################################################################

# Reset test rules
###############################################################################
DoCleanRules()
{
	DoCmd auditctl -d exit,always -F dir=$working_dir -S mkdir \
                   -S mkdirat -k create_object \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S mkdir -k create_object
	DoCmd auditctl -d exit,always -F dir=$working_dir -S unlink \
                   -S unlinkat -k delete_object \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S unlink -k delete_object
	DoCmd auditctl -d exit,always -F dir=$working_dir -S open \
                   -S openat -k object_open \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S open -k object_open
	DoCmd auditctl -d exit,always -F dir=$working_dir -S rename \
                   -S renameat -S renameat2 \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S rename \
                    -S renameat \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S rename
	DoCmd auditctl -d exit,always -F dir=$working_dir -S chown -S fchown \
                   -S fchownat -k change_object_owner \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S chown \
                    -S fchown -k change_object_owner \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S chown \
                    -k change_object_owner
	DoCmd auditctl -d exit,always -F dir=$working_dir -S chmod -S fchmod \
                   -S fchmodat -k change_object_mod \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S chmod \
                     -S fchmod -k change_object_mod \
	    || DoCmd auditctl -d exit,always -F dir=$working_dir -S chmod \
                     -S fchmod -S fchmodat -k change_object_mod
	DoCmd auditctl -d exit,always -F path=$test_file_for_execve -S execve \
                   -S execveat -k object_exec_${date_stamp} \
	    || DoCmd auditctl -d exit,always -F path=$test_file_for_execve -S execve \
                          -k object_exec_${date_stamp}
	DoCmd auditctl -d exit,always -S exit_group -k object_exec_exit_${date_stamp}
	DoCmd auditctl -d exit,always -S mount -k mount_${date_stamp}
	DoCmd auditctl -d exit,always -S umount -S umount2 -k umount_${date_stamp} \
	    || DoCmd auditctl -d exit,always -S umount -k umount_${date_stamp} \
	    || DoCmd auditctl -d exit,always -S umount2 -k umount_${date_stamp}
}
###############################################################################


trap "DoExit" EXIT HUP INT QUIT PIPE TERM 

# Exit
###############################################################################
DoExit()
{
	local exit_code="$?"
	DoCleanUp
	date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
	DoEcho "Stopping audit test at $date_stamp"
	if [ "$exit_code" -eq 0 ]; then
		DoEcho "PASS"
        exit 0
	else
		DoEcho "FAIL"
        exit 1
	fi
}
###############################################################################

# Operation
###############################################################################
DoCmd()
{
	unset do_cmd_out
	unset do_cmd_ret_code

	sleep 0.5

	local cmd="$*"
	let "oper = oper + 1"
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | $cmd"
	do_cmd_out=$($cmd 2>/dev/null)
	do_cmd_ret_code="$?"
	if [ -d ${tmp_dir} ]; then
		echo "" > ${tmp_file}
		echo "$do_cmd_out" > ${tmp_file}
		DoInputRes $do_cmd_ret_code
	fi
	return "$do_cmd_ret_code"
}

DoCmd1()
{
	unset do_cmd_out
	unset do_cmd_ret_code

	local cmd="$*"
	let "oper = oper + 1"
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | $cmd"
	do_cmd_out=$($cmd > /dev/null 2>&1)
	do_cmd_ret_code="$?"
	if [ -d ${tmp_dir} ]; then
	        echo "" > ${tmp_file}
		echo "$do_cmd_out" > ${tmp_file}
	    DoInputRes $do_cmd_ret_code
	fi
	return "$do_cmd_ret_code"
}
###############################################################################

# Result checking
###############################################################################
DoCheckResult()
{
	local return_code_to_check="$1"

	if [ "$return_code_to_check" -ne 0 ]; then
		exit "$return_code_to_check"
    fi
}
###############################################################################

# Init
###############################################################################
DoInit()
{
	oper=0
    rm $log 2>/dev/null
	DoEcho ""
	DoEcho "***************************************************************************"
	DoEcho " Test init"
	DoEcho "***************************************************************************"

	arch=$(uname -m 2>/dev/null)
	#DoCmd arch=`uname -m`
	DoCheckResult "$?"

	user=$(whoami 2>/dev/null)
	#DoCmd user=`whoami`
	if [ "$user" = "root" ]; then
	    ret=0
	else
	    ret=1
	fi
	DoCheckResult "$ret"

	DoCmd mkdir -p ${working_dir}
	DoCheckResult "$?"

	if [ -n "$stop" ] && [ "$stop" -eq 1 ]; then
		DoCmd1 /etc/init.d/auditd restart
		DoCheckResult "$?"
		DoCmd auditctl -D
		DoCheckResult "$?"
	fi

	DoCmd mkdir -p ${tmp_dir}
	DoCheckResult "$?"

	DoSetRules
	sleep 10
}
###############################################################################


# MAIN
###############################################################################

OPTIONS="vdhsf:"
while getopts $OPTIONS option
do
        case $option in
                v) verbose=1 ;;
                d) debug=1 ;;
		s) stop=1 ;;
		f) file_search="-if ${OPTARG}";;
		h) echo -e "\v - verbose"
	           echo -e "\d - debug mode (set -x)"
		   echo -s "\s - stop audit after test"
		   echo -s "\f - file from search"
		   echo -e "\h - help"
		   exit 1;;
                *) echo "$prog: illegal option -- $option"; usage; exit 1;;
        esac
done

DoEcho "Starting audit test at $date_stamp"

DoInit

#-S mkdir/mkdirat
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "1) Checking create object event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd mkdir $working_dir/trust_object
	DoCmd ausearch -i -f $working_dir/trust_object -sc mkdir ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object -sc mkdirat ${file_search}
	DoCheckResult "$?"

#-S unlink/unlinkat
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "2) Checking delete object event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd touch $working_dir/trust_object_delete
	DoCmd rm -f $working_dir/trust_object_delete
	DoCmd ausearch -i -f $working_dir/trust_object_delete \
                   -sc unlink ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_delete \
                    -sc unlinkat ${file_search}
	DoCheckResult "$?"

#-S open/openat
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "3) Checking open object event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd touch $working_dir/trust_object_open
	DoCmd ausearch -i -f $working_dir/trust_object_open \
                   -sc open ${file_search} \
       || DoCmd ausearch -i -f $working_dir/trust_object_open \
                   -sc openat ${file_search}
	DoCheckResult "$?"

#-S rename/renameat/renameat2
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "4) Checking rename object event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd touch $working_dir/trust_object_name1
	DoCmd mv $working_dir/trust_object_name1 $working_dir/trust_object_name2
	DoCmd ausearch -i -f $working_dir/trust_object_name1 \
                   -sc renameat2 ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_name1 \
                   -sc renameat ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_name1 -sc rename \
                    -sc rename ${file_search}
	DoCheckResult "$?"

#-S chown/fchown/fchownat
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "5) Checking change object owner event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd touch $working_dir/trust_object_chown
	DoCmd chown nobody $working_dir/trust_object_chown
	DoCmd ausearch -i -f $working_dir/trust_object_chown \
                   -sc chown ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_chown \
                    -sc fchown ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_chown \
                    -sc fchownat ${file_search}
	DoCheckResult "$?"

#-S chmod/fchmod/fchmodat
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "6) Checking change object mod event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd touch $working_dir/trust_object_chmod
	DoCmd chmod 777 $working_dir/trust_object_chmod
	DoCmd ausearch -i -f $working_dir/trust_object_chmod \
                   -sc chmod ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_chmod \
                    -sc fchmod ${file_search} \
	    || DoCmd ausearch -i -f $working_dir/trust_object_chmod \
                    -sc fchmodat ${file_search}
	DoCheckResult "$?"

#-S execve/execveat/exit_group
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "7) Checking exec and exit object event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd $test_file_for_execve .
	DoCmd ausearch -i -f $test_file_for_execve -sc execve ${file_search} \
                   -k object_exec_${date_stamp} \
	    || DoCmd ausearch -i -f $test_file_for_execve -sc execveat ${file_search} \
                          -k object_exec_${date_stamp}
	DoCheckResult "$?"
	DoCmd ausearch -i -c `basename $test_file_for_execve` -sc exit_group ${file_search} \
                   -k object_exec_exit_${date_stamp}
	DoCheckResult "$?"

#-S mount/umount/umount2
DoEcho ""
DoEcho "***************************************************************************"
DoEcho "8) Checking mount and umount object event"
DoEcho "***************************************************************************"
#	oper=0
	DoCmd mkdir $working_dir/mount_in
	DoCmd mkdir $working_dir/mount_this
	DoCmd mount --bind $working_dir/mount_this $working_dir/mount_in
	DoCmd ausearch -i -f $working_dir/mount_in -sc mount ${file_search} \
                   -k mount_${date_stamp}
	DoCheckResult "$?"
	DoCmd umount $working_dir/mount_in
	DoCmd ausearch -i -f $working_dir/mount_in -sc umount ${file_search} \
                   -k umount_${date_stamp} \
	    || DoCmd ausearch -i -f $working_dir/mount_in -sc umount2 ${file_search} \
                          -k umount_${date_stamp}
	DoCheckResult "$?"

