#!/bin/bash
set +x
#Test DAC

# DEFENITION
###############################################################################
date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
short_date_stamp=$(date "+%y%m%d%H%M%S" 2>/dev/null)

progname=$(basename $0 2>/dev/null)

cd $(dirname $0)

cpwd=$(pwd)
#log="${cpwd}/trace.log"
log=
utils=dac

tmp_dir="${cpwd}/tmp"
tmp_file_output="${tmp_dir}/tmp_output"
tmp_file_res="${tmp_dir}/tmp_res"

export TMPFILE="/tmp/rwx_${date_stamp}"
export RESFILE="/tmp/rwx_res_${date_stamp}"

arch=
user=


verbose=0
debug=0
###############################################################################


# Usage
###############################################################################
usage()
{
        echo " usage: $progname [-vdh]"
        echo
        exit 1
}
################################################################################

# Echo
###############################################################################
DoEcho()
{
	if [ $verbose != 0 ]; then 
		echo -e "${progname}: === $*" 
	fi
	echo -e "${progname}: === $*" >> $log
}
###############################################################################

# Result checking
###############################################################################
DoCheckResult()
{
	local return_code_to_check="$1"

	if [ -d ${tmp_dir} ]; then
		DoInputRes $return_code_to_check
	fi

	if [ "$return_code_to_check" -ne 0 ]; then
		exit "$return_code_to_check"
        fi
}
###############################################################################


# Input text formating
###############################################################################
DoInputRes()
{
	DoEcho "   | Output command:"
	while read LINE; do DoEcho "   | $LINE"; done < ${tmp_file_output}
	DoEcho "   | Result command:"
	if [ "$1" -eq 0 ]; then
		DoEcho "   | PASS"
	else
		DoEcho "   | FAIL"
	fi
}
###############################################################################

# Operation
###############################################################################
DoCmd()
{
	unset do_cmd_out
	unset do_cmd_out_add
	unset do_cmd_ret_code

	local cmd="$@"

	if [ -d ${tmp_dir} ]; then
		echo "" > ${tmp_file_output}
	fi

	let "oper = oper + 1"
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | ${cmd}"
	do_cmd_out=$(eval $cmd 2>>$log)
	do_cmd_ret_code=$?

	do_cmd_ret_code=$?
	if [ -d ${tmp_dir} ]; then
		echo "$do_cmd_out" > ${tmp_file_output}
	fi
	return "$do_cmd_ret_code"
}
###############################################################################

# Init
###############################################################################
DoInit()
{

#	oper=0
	DoEcho ""
	DoEcho "***************************************************************************"
	DoEcho " Test init"
	DoEcho "***************************************************************************"

	DoEcho "Make tmp dir"
	DoCmd "mkdir -p $tmp_dir"
	DoCheckResult "$?"

	DoEcho "Arch settings"
	DoCmd "uname -m"
	DoCheckResult "$?"
	arch=$do_cmd_out
	
	DoEcho "User checking"
	DoCmd "whoami"
	DoCheckResult "$?"
	arch=$do_cmd_out

	DoEcho "Creating test users"
	DoCmd "groupadd rwxtest_users"
	DoCheckResult "$?"

	DoCmd "useradd rwxtest_owner"
	DoCheckResult "$?"

	DoCmd "useradd -G rwxtest_users rwxtest_user1"
	DoCheckResult "$?"

	DoCmd "useradd rwxtest_other"
	DoCheckResult "$?"



}
###############################################################################


# User creating
###############################################################################
AddUser()
{
	DoEcho "Adding test_aide user"
	DoCmd "useradd -u 777 $1"
	DoCheckResult "$?"
}

# User deleting
###############################################################################
DelUser()
{
	DoEcho "Deleting user: $1"
	DoCmd "userdel $1"
	DoCheckResult "0"
}

# Group deleting
###############################################################################
DelGroup()
{
	DoEcho "Deleting group: $1"
	DoCmd "groupdel $1"
	DoCheckResult "0"
}

# Clean
###############################################################################
DoCleanUp()
{
#	oper=0
	DoEcho "***************************************************************************"
	DoEcho " Cleanup test"
	DoEcho "***************************************************************************"

	DoEcho "Deleting test files: $TMPFILE $RESFILE"
	DoCmd rm -rf $TMPFILE $RESFILE
	
	DelUser rwxtest_owner
	DelUser rwxtest_user1
	DelUser rwxtest_other
	DelGroup rwxtest_users
	
}
###############################################################################


trap "DoExit" EXIT
trap "exit 127" HUP INT QUIT PIPE TERM

###############################################################################
DoExit()
{
	local exit_code="$?"
	DoCleanUp
	date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
	DoEcho "Stopping DAC TEST at $date_stamp"
	if [ "$exit_code" -eq 0 ]; then
		DoEcho "PASS"
	else
		DoEcho "FAIL"
	fi
}
###############################################################################





# MAIN
###############################################################################

OPTIONS="vdh"
while getopts $OPTIONS option
do
        case $option in
                v) verbose=1;;
                d) debug=1;;
		h) echo -e "\v - verbose"
	           echo -e "\d - debug mode (set -x)"
		   echo -e "\h - help" 
		   exit 1;;
                *) echo "$prog: illegal option -- $option"; usage; exit 1;;
        esac
done


log="${cpwd}/trace.${utils}.log"
if [ -f ${log} ]; then
	DoEcho "Removing old log"
	DoCmd rm  ${log}
	DoCheckResult "$?"
fi


DoEcho "Starting DAC TEST at $date_stamp"

DoInit 

DoEcho ""
DoEcho "***************************************************************************"
DoEcho "1) Checking DAC "
DoEcho "***************************************************************************"

oper=0



DoEcho "Creating test files"
DoCmd "echo 'DAC test'> $TMPFILE"
DoCheckResult "$?"

DoCmd "chown rwxtest_owner:rwxtest_users $TMPFILE"
DoCheckResult "$?"

DoCmd "chmod 0640 $TMPFILE"
DoCheckResult "$?"

DoCmd "echo > $RESFILE"
DoCheckResult "$?"

DoCmd "chmod 0666 $RESFILE"
DoCheckResult "$?"

DoEcho "\"Reading owner\" check"
DoCmd "su -c 'cat $TMPFILE; echo $? > $RESFILE' rwxtest_owner"
DoCheckResult "$?"

DoEcho "\"Writing owner\" check"
DoCmd "su -c 'echo Write owner test >> $TMPFILE; echo $? > $RESFILE' rwxtest_owner"
DoCheckResult "$?"

DoEcho "\"Reading group\" check"
DoCmd "su -c 'cat $TMPFILE  2>&1; echo $? > $RESFILE' rwxtest_user1"
DoCheckResult "$?"

DoEcho "\"Writing group\" check"
DoCmd "su -c 'echo Write group test 2>&1 >> $TMPFILE; echo $? > $RESFILE' rwxtest_user1"
DoCheckResult "$?"

DoEcho "\"Reading check\" other"
DoCmd "su -c 'cat $TMPFILE 2>&1; echo $? > $RESFILE' rwxtest_other"
DoCheckResult "$?"

DoEcho "\"Writing check\" other"
DoCmd "su -c 'echo Write other test 2>&1 >> $TMPFILE; echo $? > $RESFILE' rwxtest_other"
DoCheckResult "$?"
