#!/bin/bash
set +x
#Test cryptsetup

# DEFENITION
###############################################################################
date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
short_date_stamp=$(date "+%y%m%d%H%M%S" 2>/dev/null)

progname=$(basename $0 2>/dev/null)

cd $(dirname $0)

cpwd=$(pwd)
#log="${cpwd}/trace.log"
log=
utils=dm-crypt

tmp_dir="${cpwd}/tmp"
tmp_file_output="${tmp_dir}/tmp_output"
tmp_file_res="${tmp_dir}/tmp_res"

diskimage=./diskimage
mount_dir=./mnt_disk
loop_free=
arch=


verbose=0
debug=0
###############################################################################


# Usage
###############################################################################
usage()
{
        echo " usage: $progname [-vdh]"
        echo
        exit 1
}
################################################################################

# Echo
###############################################################################
DoEcho()
{
	if [ $verbose != 0 ]; then 
		echo -e "${progname}: === $*" 
	fi
	echo -e "${progname}: === $*" >> $log
}
###############################################################################

# Result checking
###############################################################################
DoCheckResult()
{
	local return_code_to_check="$1"

	if [ -d ${tmp_dir} ]; then
		DoInputRes $return_code_to_check
	fi

	if [ "$return_code_to_check" -ne 0 ]; then
		exit "$return_code_to_check"
        fi
}
###############################################################################


# Input text formating
###############################################################################
DoInputRes()
{
	DoEcho "   | Output command:"
	while read LINE; do DoEcho "   | $LINE"; done < ${tmp_file_output}
	DoEcho "   | Result command:"
	if [ "$1" -eq 0 ]; then
		DoEcho "   | PASS"
	else
		DoEcho "   | FAIL"
	fi
}
###############################################################################

# Operation
###############################################################################
DoCmd()
{
	unset do_cmd_out
	unset do_cmd_out_add
	unset do_cmd_ret_code

	local cmd="$@"

	if [ -d ${tmp_dir} ]; then
		echo "" > ${tmp_file_output}
	fi

	let "oper = oper + 1"
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | ${cmd}"
	do_cmd_out=$(eval $cmd 2>>$log)
	do_cmd_ret_code=$?

	do_cmd_ret_code=$?
	if [ -d ${tmp_dir} ]; then
		echo "$do_cmd_out" > ${tmp_file_output}
	fi
	return "$do_cmd_ret_code"
}
###############################################################################

# Init
###############################################################################
DoInit()
{

#	oper=0
	DoEcho ""
	DoEcho "***************************************************************************"
	DoEcho " Test init"
	DoEcho "***************************************************************************"

	DoEcho "Make tmp dir"
	DoCmd "mkdir -p $tmp_dir"
	DoCheckResult "$?"

	DoEcho "Arch settings"
	DoCmd "uname -m"
	DoCheckResult "$?"
	arch=$do_cmd_out
	
}
###############################################################################


# Clean
###############################################################################
DoCleanUp()
{
#	oper=0
	DoEcho "***************************************************************************"
	DoEcho " Cleanup test"
	DoEcho "***************************************************************************"

	DoEcho "Removing test files: $mount_dir $diskimage"
	DoCmd rm -rf $mount_dir $diskimage
	
}
###############################################################################



# Help functions
###############################################################################

umount_dev() {
	DoEcho "Unmounting crypto loop device"
	DoCmd "umount $mount_dir"
	DoCmd "cryptsetup luksClose user_disk"
	DoCmd "losetup -d $loop_free"
}

mount_dev() {
	DoEcho "Mounting loop device"
	DoCmd "losetup $loop_free $diskimage"
	DoCheckResult "$?"
	DoCmd modprobe dm-crypt
	DoCmd modprobe xts
	
	if ((fist == 1 ))
	then
		DoEcho "Formatting crypto device"
		DoCmd "cryptsetup luksFormat  $loop_free -d ${cpwd}/key.txt < ${cpwd}/YES"
	else
		DoCmd "cryptsetup luksDump $loop_free"
	fi
	DoCheckResult "$?"

	DoEcho "Opening crypto device"
	DoCmd "cryptsetup luksOpen $loop_free user_disk -d ./key.txt"
	DoCheckResult "$?"

	if ((fist == 1 ))
	then
		DoEcho "Formatting disk (mkfs)"
		DoCmd "mkfs.ext4 /dev/mapper/user_disk"
		DoCheckResult "$?"
		fist=0
	fi
	
	DoEcho "Mounting disk"
	DoCmd "mount /dev/mapper/user_disk $mount_dir"
	DoCheckResult "$?"

}
###############################################################################

trap "DoExit" EXIT
trap "exit 127" HUP INT QUIT PIPE TERM

###############################################################################
DoExit()
{
	local exit_code="$?"
	umount_dev
	DoCleanUp
	date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
	DoEcho "Stopping dm-crypt TEST at $date_stamp"
	if [ "$exit_code" -eq 0 ]; then
		DoEcho "PASS"
	else
		DoEcho "FAIL"
	fi
}
###############################################################################



# MAIN
###############################################################################

OPTIONS="vdh"
while getopts $OPTIONS option
do
        case $option in
                v) verbose=1;;
                d) debug=1;;
		h) echo -e "\v - verbose"
	           echo -e "\d - debug mode (set -x)"
		   echo -e "\h - help" 
		   exit 1;;
                *) echo "$prog: illegal option -- $option"; usage; exit 1;;
        esac
done


log="${cpwd}/trace.${utils}.log"
if [ -f ${log} ]; then
	DoEcho "Removing old log"
	DoCmd rm  ${log}
	DoCheckResult "$?"
fi

DoInit

DoEcho "Starting dm-crypt TEST at $date_stamp"

DoEcho ""
DoEcho "***************************************************************************"
DoEcho "1) Checking dm-crypt on loop device "
DoEcho "***************************************************************************"

oper=0

fist=1

DoEcho "Checking free loop device"
DoCmd "losetup -f"
DoCheckResult "$?"
loop_free=$do_cmd_out

DoEcho "Creating diskimage file"
DoCmd "dd if=/dev/zero of=$diskimage bs=1M count=20"
DoCheckResult "$?"

DoEcho "Creating test dir"
DoCmd "mkdir -p $mount_dir"
DoCheckResult "$?"

mount_dev

DoEcho "Copying file to test disk"
DoCmd "cp -r /lib/libatomic.so.1 $mount_dir"
DoCheckResult "$?"

DoCmd "sync"
DoCheckResult "$?"

DoCmd "sync"
DoCheckResult "$?"

umount_dev

mount_dev

DoEcho "Check test file on diff"
DoCmd "diff /lib/libatomic.so.1 $mount_dir/libatomic.so.1"
DoCheckResult "$?"
