#!/bin/bash
set +x

timeout=10

OPTIONS="ht:"
while getopts $OPTIONS option
do
	case $option in
	t) timeout="${OPTARG}";;
	h) echo -e "\tt - curl timeout connection (sec)"; exit 1;;
	*) exit 1;;
	esac
done

TEST_PATH=`cd  $(dirname $0); pwd`
CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)
BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TMP_DIR=${TC_DIR}/res
CFG_DIR=${TC_DIR}/cfg

APACHE2=`which apache2`
CURL=`which curl`
ELMACEXEC=`which elmacexec`
GETFELMAC=`which getfelmac`
SETFELMAC=`which setfelmac`
USERADD=`which useradd`
USERDEL=`which userdel`

TEST_PORT="8082"
QUEST="${CURL} --connect-timeout ${timeout} -I http://localhost:${TEST_PORT}"

MAC_CONF_FILE=${TEST_PATH}/apache_mac.conf

TEST_DIR="/elmac-test-dir"
TEST_LEVEL="0 1 2 3"
TEST_CAT="0x0000 0x0001 0x0002 0x0003 0x0ba0 0xa000 0xfff0 0xfff1 0xfff2 0xfff3"

TEMPLATE="<html><body><h1>Apache MAC test!</h1></body></html>"

if [ -d /sys/fs/elmacfs ]; then
ELMACFS="/sys/fs/elmacfs"
else
ELMACFS="/elmac"
fi

progname=$(basename $0)


#Formatting echo
DoEcho()
{
    #echo "$progname: === $*"
    echo "=== $*"
}

#Verbose echo
DoEchoVer()
{
    [[ $verbose == 1 ]] && DoEcho $*
}

#Formatting exit
DoExit()
{
    Cleanup
    [[ $1 == 0 ]] && DoEcho "PASS"
    [[ $1 != 0 ]] && DoEcho "FAIL"
    exit $1
}

#Formating execute command
DoCmd()
{
    local cmd ret
    cmd="$*"
    DoEchoVer "$cmd"
    $cmd > /dev/null 2>/dev/null
    ret=$?
    if [[ $ret != 0 ]]; then
        DoEcho "$cmd - returned $ret"
        DoExit 1
    fi

    return 0
}

#Create user apache
CreateUser()
{
    id elmac_apache > /dev/null 2>/dev/null
    ret=$?
    if [[ $ret != 0 ]]; then
		DoCmd ${USERADD} elmac_apache
    fi
}

#Remove user apache
RemoveUser()
{
    ${USERDEL} elmac_apache > /dev/null 2>/dev/null
    ret=$?
    if [[ $ret != 0 ]]; then
		DoEcho "${USERDEL} elmac_apache - returned $ret"
    fi
}

#Create testing site
CreateTestObj()
{
	if [ -d ${TEST_DIR} ]; then
		DoCmd rm -rf ${TEST_DIR}
	fi
	DoCmd mkdir -p ${TEST_DIR}
	containers=`cat ${ELMACFS}/containers 2>/dev/null`
	ret=$?
	if [[ $ret == 0 ]]; then
		if [[ $containers == 1 ]]; then
            DoCmd ${GETFELMAC} -s / 2>/dev/null
            label=`${GETFELMAC} -s / 2>/dev/null | awk '{print $1}'`
            DoCmd ${SETFELMAC} -s $label ${TEST_DIR}
		fi
	fi

	for l in ${TEST_LEVEL}; do
		for c in ${TEST_CAT}; do
			echo ${TEMPLATE} >  ${TEST_DIR}/level_${l}_cat_${c}.html
			DoCmd ${SETFELMAC} -l ${l} -c ${c} ${TEST_DIR}/level_${l}_cat_${c}.html
		done
	done
}

#Remove testing site
RemoveTestObj()
{
	if [ -d ${TEST_DIR} ]; then
	    rm -rf ${TEST_DIR} > /dev/null 2>/dev/null
	    ret=$?
        if [[ $ret != 0 ]]; then
		    DoEcho "rm -rf ${TEST_DIR} - returned $ret"
        fi
	fi
}

#Check MAC,prefork apache
CheckPrefork()
{
	prefork=$(apachectl -M -f ${MAC_CONF_FILE}  2>/dev/null | grep mpm_prefork | head -n 1 | cut -d ' ' -f 2)
	if [ "${prefork}" != "mpm_prefork_module" ]; then
		DoEcho "Apache mode no prefork"
		DoEcho "See ${MAC_CONF_FILE}"
		DoExit 1
	fi
}

#Start apache
ApacheStart()
{
	DoCmd ${APACHE2} -f ${MAC_CONF_FILE} -k start
}

#Stop apache
ApacheStop()
{
	${APACHE2} -f ${MAC_CONF_FILE} -k stop > /dev/null 2>/dev/null
    ret=$?
    if [[ $ret != 0 ]]; then
        DoEcho "${APACHE2} -f ${MAC_CONF_FILE} -k stop - returned $ret"
    fi
}

#Testing
TestProc()
{
	if [ -d ${TMP_DIR} ]; then
		DoCmd rm -rf ${TMP_DIR}
	fi
	DoCmd mkdir -p ${TMP_DIR}

	for l in ${TEST_LEVEL}; do
		for c in ${TEST_CAT}; do
			for ll in ${TEST_LEVEL}; do
				for cc in ${TEST_CAT}; do
					status=$(${ELMACEXEC} -l ${l} -c ${c} -- ${QUEST}/level_${ll}_cat_${cc}.html  2> /dev/null | head -n 1 | cut -d ' ' -f '3 4')
                    if [ -z "$status" ]; then
                        status="Not connected"
                    fi
					echo "Proc ( ${l}:${c} ) Object ( http://localhost:${TEST_PORT}/level_${ll}_cat_${cc}.html ${ll}:${cc})  $status" >> ${TMP_DIR}/elmac_ret
                    DoEchoVer "${ELMACEXEC} -l ${l} -c ${c} -- ${QUEST}/level_${ll}_cat_${cc}.html res = $status"
				done
			done
		done
	done
}

#Check matrix
CheckMatrix()
{
	hfiles=""
	hf=`cat ${ELMACFS}/hfiles 2>/dev/null`
	ret=$?
	if [[ $ret == 0 ]]; then
		if [[ $hf != 0 ]]; then
			hfiles="_hf_present"
		fi
	fi

	 DoCmd diff ${TMP_DIR}/elmac_ret ${TEST_PATH}/elmac_ret_etalon${hfiles}
}

#Cleanup
Cleanup()
{
    ApacheStop
    sleep 10
    RemoveTestObj
    RemoveUser
}

#Signal handler
SigHandler()
{
	DoExit 1
}


trap "SigHandler" SIGINT

#Usage
usage()
{
        echo " usage: $progname [-v]"
        echo "        -v - verbose enable"
        echo " "
        exit 1
}

#Main
verbose=0

OPTIONS="v:"
while getopts $OPTIONS option
do
        case $option in
                v) verbose=1;;
                *) usage; exit 1;;
        esac
done

if [ -z ${APACHE2} ]; then
	DoEcho "apache2 not found"
    DoEcho "apt-get install apache2-bin"
	DoExit 1
fi

if [ -z ${CURL} ]; then
	DoEcho "curl not found"
    DoEcho "apt-get install curl"
	DoExit 1
fi

if [ -z ${ELMACEXEC} ]; then
	DoEcho "elmacexec not found"
    DoEcho "apt-get install elmac-utils"
	DoExit 1
fi

if [ -z ${GETFELMAC} ]; then
	DoEcho "getfelmac not found"
    DoEcho "apt-get install elmac-utils"
	DoExit 1
fi

if [ -z ${SETFELMAC} ]; then
	DoEcho "setfelmac not found"
    DoEcho "apt-get install elmac-utils"
	DoExit 1
fi

if [ -z ${USERADD} ]; then
	DoEcho "useradd not found"
    DoEcho "apt-get install passwd"
	DoExit 1
fi

if [ -z ${USERDEL} ]; then
	DoEcho "userdel not found"
    DoEcho "apt-get install passwd"
	DoExit 1
fi

CreateTestObj
CreateUser
ApacheStart
CheckPrefork
TestProc
CheckMatrix

DoExit 0
