#!/bin/bash

CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)
CUR_DIR=$(pwd)

if [ -d /sys/fs/elmacfs ]; then
ELMACFS="/sys/fs/elmacfs"
else
ELMACFS="/elmac"
fi

PORT=8081
APACHE2=`which apache2`
GETFELMAC=`which getfelmac`
SETFELMAC=`which setfelmac`
USERADD=`which useradd`
USERDEL=`which userdel`
HOSTNAME=`hostname`

CONF_FILE=${CUR_DIR}/apache_comp.conf

TEST_DIR="/elmac-test-dir"
SITE_DIR=${TEST_DIR}/test_ipso_compatible
SITE_FILES_TAR=${CUR_DIR}/test_site.tar

SITE_FILES_NOCONFID=${SITE_DIR}/noconfid.html
SITE_FILES_DSP=${SITE_DIR}/dsp.html
SITE_FILES_SECRETNO=${SITE_DIR}/secretno.html

#Formatting echo
DoEcho()
{
    #echo "$progname: === $*"
    echo "=== $*"
}

#Verbose echo
DoEchoVer()
{
    [[ $verbose == 1 ]] && DoEcho $*
}

#Formatting exit
DoExit()
{
    Cleanup
    [[ $1 == 0 ]] && DoEcho "PASS"
    [[ $1 != 0 ]] && DoEcho "FAIL"
    exit $1
}

#Formating execute command
DoCmd()
{
    local cmd ret
    cmd="$*"
    DoEchoVer "$cmd"
    $cmd > /dev/null 2>/dev/null
    ret=$?
    if [[ $ret != 0 ]]; then
        DoEcho "$cmd - returned $ret"
        DoExit 1
    fi

    return 0
}

#Create user apache
CreateUser()
{
        id elmac_apache > /dev/null 2>/dev/null
        ret=$?
        if [[ $ret != 0 ]]; then
		    DoCmd ${USERADD} elmac_apache > /dev/null 2>/dev/null
        fi
}

#Remove user apache
RemoveUser()
{
    ${USERDEL} elmac_apache > /dev/null 2>/dev/null
    ret=$?
    if [[ $ret != 0 ]]; then
		DoEcho "${USERDEL} elmac_apache - returned $ret"
		DoExit 1
    fi
}

#Create test site
CreateTestSite()
{
    if [ -d ${TEST_DIR} ]; then
		DoCmd rm -rf ${TEST_DIR}
	fi
    DoCmd mkdir -p ${SITE_DIR}
	containers=`cat ${ELMACFS}/containers 2>/dev/null`
	ret=$?
	if [[ $ret == 0 ]]; then
		if [[ $containers == 1 ]]; then
            DoCmd ${GETFELMAC} -s / 2>/dev/null
            label=`${GETFELMAC} -s / 2>/dev/null | awk '{print $1}'`
            DoCmd ${SETFELMAC} -R -s $label ${TEST_DIR}
		fi
	fi
    DoCmd tar -xvf ${SITE_FILES_TAR} -C ${SITE_DIR} > /dev/null 2>&1
    DoCmd chmod a+r ${SITE_FILES_NOCONFID}
    DoCmd ${SETFELMAC} -l 1 ${SITE_FILES_DSP}
    DoCmd chmod a+r ${SITE_FILES_DSP}
    DoCmd ${SETFELMAC} -l 2 ${SITE_FILES_SECRETNO}
    DoCmd chmod a+r ${SITE_FILES_SECRETNO}
}

#Remove test site
RemoveTestSite()
{
	if [ -d ${TEST_DIR} ]; then
	    rm -rf ${TEST_DIR} > /dev/null 2>/dev/null
	    ret=$?
        if [[ $ret != 0 ]]; then
		    DoEcho "rm -rf ${TEST_DIR} - returned $ret"
        fi
	fi
}

#Check MAC,prefork apache
CheckPrefork()
{
	prefork=$(apachectl -M -f ${MAC_CONF_FILE}  2>/dev/null | grep mpm_prefork | head -n 1 | cut -d ' ' -f 2)
	if [ "${prefork}" != "mpm_prefork_module" ]; then
		DoEcho "Apache mode no prefork"
		DoEcho "See ${MAC_CONF_FILE}"
        DoExit 1
	fi
}

#Start server
StartServer()
{
	DoCmd ${APACHE2} -f ${CONF_FILE} -k start
}

#Stop server
StopServer()
{
    ${APACHE2} -f ${CONF_FILE} -k stop > /dev/null 2>/dev/null
    if [[ $ret != 0 ]]; then
        DoEcho "${APACHE2} -f ${CONF_FILE} -k stop - returned $ret"
    fi
}

Cleanup()
{
    StopServer
    sleep 10
    RemoveTestSite
    RemoveUser
}

#Signals handler
SigHandler()
{
    Cleanup
    exit 1
}

#Main

trap "SigHandler" "SIGINT"

if [ -z ${APACHE2} ]; then
	DoEcho "apache2 not found"
    DoEcho "apt-get install apache2-bin"
	DoExit 1
fi

if [ -z ${GETFELMAC} ]; then
	DoEcho "getfelmac not found"
    DoEcho "apt-get install elmac-utils"
	DoExit 1
fi

if [ -z ${SETFELMAC} ]; then
	DoEcho "setfelmac not found"
    DoEcho "apt-get install elmac-utils"
	DoExit 1
fi

if [ -z ${USERADD} ]; then
	DoEcho "useradd not found"
    DoEcho "apt-get install passwd"
	DoExit 1
fi

if [ -z ${USERDEL} ]; then
	DoEcho "userdel not found"
    DoEcho "apt-get install passwd"
	DoExit 1
fi



CreateUser
CreateTestSite
StartServer
echo "
1) Запущен сервер ГОД (\"${APACHE2} -f ${CONF_FILE} -k start\")

2) Выполните тестовую процедуру elmac_apache_tc
   в соответсвии с ТВГИ.<...> обратившись с тестовой машины к тестовым
   страницам

   http://${HOSTNAME}:8081/noconfid.html
   http://${HOSTNAME}:8081/dsp.html
   http://${HOSTNAME}:8081/secretno.html

   Сравните полученный результат с матрицей доступа, приведенной ниже

   --------------------------------------------------------------------------
   | Уровень  пользователя | noconfid.html |    dsp.html    | secretno.html |
   --------------------------------------------------------------------------
   |           0           |       +       |        -       |       -       |
   --------------------------------------------------------------------------
   |           1           |       +       |        +       |       -       |
   --------------------------------------------------------------------------
   |           2           |       +       |        +       |       +       |
   --------------------------------------------------------------------------
   где <<+>> - доступ к странице разрешен
       <<->> - доступ к странице запрещен

   Примечание.
   1) При запрете доступа к странице выдается  «Forbidden» или «Not Found».
   Forbidden — формируется при отказе в доступе к объекту при выключенном
   режиме «видимость имен файлов в соответствии с мандатным разграничением
   доступа».
   Not Found — формируется при отказе в доступе к объекту при
   включенном режиме «видимость имен файлов в соответствии с мандатным
   разграничением доступа».
   Режим «видимость имен файлов в соответствии с мандатным разграничением доступа»
   определяется значением в файле /elmac/hfiles (0 — режим выключен; 1 — режим
   включен).
   2) В зависимости от настроек сети в запросах вместо сетевого имени сервера
   может использоваться ip адрес.

   Тестовая процедура считается завершившейся успешно, если результат обращения
   к тестовым страницам совпадает с приведенной матрицей доступа.
"

echo "
   Если тестовая процедура завершена успешно введите \"yes\", иначе введите \"no\"
"

read CONTINUE_ANSWER
if [ "${CONTINUE_ANSWER}" = "yes" ]; then
    RemoveTestSite()
    DoExit 0
fi

RemoveTestSite()
DoExit 1
