#!/bin/bash

set +xv

CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TEST_NAME=elmac_net_tc
TMP_DIR=${TC_DIR}/tmp
CFG_DIR=${TC_DIR}/cfg
ENV_FILE=${CFG_DIR}/env
if [ -d /sys/fs/elmacfs ]; then
ELMACFS="/sys/fs/elmacfs"
else
ELMACFS="/elmac"
fi
ELMACFS_DIR=/elmac
SYSWRITEUP_FILE=${ELMACFS}/mac_write_up
SOCKETWRITEUP_FILE=${ELMACFS}/mac_sock_write_up

tmpf=/tmp/_elmac_tmpf

port=5777
mode=0
debug=0
timeout=0

CWD=$(pwd)

verbose=0
################################################################################
usage()
{
	echo " usage: $progname [-v [-p PORT_NUM] [-m TEST_MODE] [-t TIMEOUT]]"
	echo
	exit 1
}
OPTIONS="vp:m:t:"
while getopts $OPTIONS option
do
	case $option in
		v) verbose=1;;
		p) port=${OPTARG};;
		m) mode=${OPTARG};;
		t) timeout=${OPTARG};;
		*) echo "$prog: illegal option -- $option"; usage; exit 1;;
	esac
done
################################################################################
DoEcho()
{
	echo "$progname: === $*"
}
DoEchoVer()
{
	[[ $verbose == 1 ]] && DoEcho $*
}
SigHandler()
{
	DoExit 1
}
DoExit()
{
	[[ ! -z $2 ]] && DoEcho $2
	[[ $1 == 0 ]] && DoEcho "PASS"
	[[ $1 != 0 ]] && DoEcho "FAIL"
	if [ -d ${TC_DIR}/tmpfs ]; then
		umount  ${TC_DIR}/tmpfs 2>/dev/null
		rm -rf  ${TC_DIR}/tmpfs 2>/dev/null
		rm tmpf 2>/dev/null
	fi
	cd $CPWD
	exit $1
}
DoCmd()
{
	local cmd ret
	cmd="$*"
	DoEchoVer "$cmd"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		DoEcho "$cmd - returned $ret"
		cat $tmpf
		DoExit 1
	fi
	[[ $verbose == 1 ]] && cat $tmpf
	return 0
}

trap "SigHandler" "SIGINT"

export ROOT_E90=$CWD
[[ $verbose == 1 ]] && DoEcho ROOT_E90=$ROOT_E90

test_elmac_enable=`zcat /proc/config.gz | grep _ELMAC=y`
ret=$?
if [[ $ret != 0 ]]; then
	DoExit 1 "The kernel does not support ELMAC"
fi


test_elmac_enable=`cat ${SYSWRITEUP_FILE}`
ret=$?
if [[ $ret != 0 ]]; then
	DoExit 1 "ELMAC is not activated or does not function correctly"
fi


DoCmd rm -rf $TMP_DIR
DoCmd mkdir -p -m 0777 $TMP_DIR

if [ -d ${TC_DIR}/tmpfs ]; then
	umount  ${TC_DIR}/tmpfs 2>/dev/null
	DoCmd rm -rf  ${TC_DIR}/tmpfs 2>/dev/null
fi

DoCmd mkdir -m 777 ${TC_DIR}/tmpfs

DoCmd  mount -t tmpfs -o size=1M tmpfs -o elmacfsroot=0:0:3f ${TC_DIR}/tmpfs/

DoEcho "./${TEST_NAME} port ${port} mode ${mode} timeout ${timeout}"
$CWD/elmac_net_tc ${port} ${mode} ${timeout} ${verbose} > ${TMP_DIR}/elmac_net_ret_raw 2>&1
ret=$?
if [[ $ret != 0 ]]; then
	DoEcho "./${TEST_NAME} - returned $ret"
	if [[ $verbose == 1 ]]; then
		cat ${TMP_DIR}/elmac_net_ret
	else
		DoEcho "RESULTS - ${TMP_DIR}/elmac_net_ret"
	fi
	DoExit 1
fi

grep INFO ${TMP_DIR}/elmac_net_ret_raw  > ${TMP_DIR}/elmac_net_ret

if [[ $mode == 0 ]]; then
	DoEcho "${TEST_NAME} test mode: fast ($mode)"
	TEST_MODE="fastmode"
elif [[ $mode == 1 ]]; then
	DoEcho "${TEST_NAME} test mode: full ($mode)"
	TEST_MODE="fullmode"
else
	DoExit 1 "Test mode not correct"
fi


if  [[ ! -f ${SYSWRITEUP_FILE} ]]; then
	DoExit 1 "File ${SYSWRITEUP_FILE} not found"
fi
syswrup=`cat ${SYSWRITEUP_FILE}`
ret=$?
if [[ $ret != 0 ]]; then
	DoExit 1 "Error read file ${SYSWRITEUP_FILE}"
fi
if [[ $syswrup == 0 ]]; then
	DoEcho "${TEST_NAME} system wr-up mode: disable ($syswrup)"
	SYSWRITEUP_MODE="disable"
else
	DoEcho "${TEST_NAME} system wr-up mode: enable ($syswrup)"
	SYSWRITEUP_MODE="enable"
fi


if  [[ ! -f ${SOCKETWRITEUP_FILE} ]]; then
	DoExit 1 "Error read file ${SOCKETWRITEUP_FILE}"
fi
socketwrup=`cat ${SOCKETWRITEUP_FILE}`
ret=$?
if [[ $ret != 0 ]]; then
	DoExit 1 "Error read file ${SOCKETWRITEUP_FILE}"
fi
if [[ $socketwrup == 0 ]]; then
	DoEcho "${TEST_NAME} socket wr-up mode: disable ($socketwrup)"
	SOCKETWRITEUP_MODE="disable"
else
	DoEcho "${TEST_NAME} socket wr-up mode: enable ($socketwrup)"
	SOCKETWRITEUP_MODE="enable"
fi


elmac_net_ret_etalon=elmac_net_ret_etalon_${TEST_MODE}_syswrup_${SYSWRITEUP_MODE}_sockwrup_${SOCKETWRITEUP_MODE}


DoCmd diff ${TMP_DIR}/elmac_net_ret $CWD/${elmac_net_ret_etalon}

DoExit 0
