#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/capability.h>
#include <netinet/in.h>
#include <signal.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>
#include <attr/xattr.h>
#include <sys/wait.h>

#define BUFSIZE 1024
#define STIMEOUT_S 0
#define STIMEOUT_U 100000
#define CTIMEOUT_S 0
#define CTIMEOUT_U 200000
int stimeout_s = STIMEOUT_S;
int ctimeout_s = CTIMEOUT_S;

#define NAME "../tmpfs/elmac-test-socket"

#ifdef FAKELIB
#include "elmac.h"
#else
#include <linux/elmac.h>
#endif

#ifdef FAKELIB
/**
 * From lib. No correct!!!
 */

#define ELMAC_LE	0
#define ELMAC_BE	1
#define SELF_LABEL_FILE "/proc/self/attr/current"

/*
 * elmac_get_endian - Get system endianness
 */
int elmac_get_endian(void)
{
	int ret;

	union {
		int i;
		char c[sizeof(int)];
	} foo;

	foo.i = 1;

	if (foo.c[0] == 1) {
		ret = ELMAC_LE;
	} else {
		ret = ELMAC_BE;
	}
	return ret;
}

/*
 * le64 - Convert endianness.
 */
#define CONVERT_MAX_SIZE	8
void le64(char *val, ssize_t len)
{
	char buff[CONVERT_MAX_SIZE] = {0};
	int i;
	ssize_t save_len;

	save_len = len;

	i = 0;
	while (len --) {
		buff[i] = val[len];
		i ++;
	}

	memcpy(val, buff, save_len);
}

/*
 * LE_LABEL - Convert label.
 */
void LE_LABEL(elmac_label_t *label)
{
	if (elmac_get_endian() == ELMAC_LE)
		return;

	if (label == NULL)
		return;

	le64((char *)&label->category, sizeof(__u64));
}

/*
 * LE_CONTEXT - Convert context.
 */
void LE_CONTEXT(elmac_context_t *context)
{
	if (elmac_get_endian() == ELMAC_LE)
		return;

	if (context == NULL)
		return;

	le64((char *)&context->attr, sizeof(__u32));
	le64((char *)&context->mac.category, sizeof(__u64));
}

/*
 * elmac_get_label_from_self -
 */

ssize_t elmac_get_label_from_self(elmac_label_t *label)
{
	elmac_context_t dummy;
	size_t len;
	int fd;
	ssize_t ret = 0;

	if (label == NULL)
		return (ssize_t)(-1);

	len = sizeof(elmac_context_t);

	fd = open(SELF_LABEL_FILE, O_RDONLY);
	if (fd < 0) {
		return (ssize_t)(-1);
	}

	ret = read(fd, &dummy, len);

	close(fd);
	if (ret < 0) {
		return ret;
	}

	LE_CONTEXT(&dummy);

	len = sizeof(elmac_label_t);
	memcpy(label, &dummy.mac, len);

	return ret;
}

/*
 * elmac_set_context_for_self -
 */
int elmac_set_context_for_self(elmac_context_t *context)
{
	size_t len;
	int fd;
	ssize_t ret;

	if (context == NULL)
		return (int)(-1);

	len = sizeof(elmac_context_t);

	fd = open(SELF_LABEL_FILE, O_WRONLY);
	if (fd < 0) {
		return (int)(-1);
        }

	LE_CONTEXT(context);

	ret = write(fd, context, len);

	close(fd);

	return (ret < 0) ? -1 : 0;
}

/*
 * elmac_set_label_for_self -
 */
int elmac_set_label_for_self(elmac_label_t *label)
{
	elmac_context_t *dummy;
	size_t len;
	int ret;

	if (label == NULL)
		return (int) (-1);

	len = sizeof(elmac_context_t);

	dummy = calloc(1, len);
	if (dummy == NULL)
		return (int)(-ENOMEM);

	memcpy(&dummy->mac, label, sizeof(elmac_label_t));

	ret = elmac_set_context_for_self(dummy);

	if (dummy)
		free(dummy);

	return (ret < 0) ? -1 : 0;
}

/*
 * elmac_set_context_for_path -
 */
int elmac_set_context_for_path(const char *path, const char *xattr, int follow,
			       elmac_context_t *context)
{
	size_t len;
	int ret;

	if (context == NULL)
		return (int)(-1);

	len = sizeof(elmac_context_t);

	LE_CONTEXT(context);

	ret = follow ?
		setxattr(path, xattr, context, len, 0) :
		lsetxattr(path, xattr, context, len, 0);

	return ret;
}
#endif

typedef __u8 elmac_lev_t;
typedef __u64 elmac_cat_t;
typedef __u32 elmac_attr_t;
typedef int (*fserver)(elmac_lev_t, elmac_cat_t, int, char **);
typedef int (*fclient)(elmac_lev_t, elmac_cat_t, elmac_lev_t, elmac_cat_t,
	     char **);

int portno;
int mode;
int debug = 0;
int cmd_exit;
elmac_lev_t max_level;
elmac_cat_t max_category;

/*
 * Error handler
 */
/*!
 * The function release error handler
 *
 * @param msg error message
 * @return Returns 1
 */
void error(const char *msg)
{
	perror(msg);
	exit(1);
}

/*!
 * The function release signal handler
 *
 * @param sig signal number
 */
void signal_handler(int sig)
{
	switch(sig) {
		case SIGHUP:
			/*
			 * Set ignore signal SIGHUP
			 */
			signal(sig, SIG_IGN);
			cmd_exit = 1;
			break;
	}
}

/*!
 * The function release capability CAP_MAC_OVERRIDE drop
 */
void cap_drop()
{
	cap_t new_caps = cap_get_proc();
	cap_value_t cap_list[] = { CAP_MAC_OVERRIDE };

	if (cap_set_flag(new_caps, CAP_EFFECTIVE, 1, cap_list, CAP_CLEAR) != 0) {
		      error("cap_drop, fail to set cap_set_flag() for CAP_EFFECTIVE\n");
	}
	if (cap_set_flag(new_caps, CAP_PERMITTED, 1, cap_list, CAP_CLEAR) != 0) {
		      error("cap_drop, fail to set cap_set_flag() for CAP_PERMITTED\n");
	}
	if (cap_set_flag(new_caps, CAP_INHERITABLE, 1, cap_list, CAP_CLEAR) != 0) {
		      error("cap_drop, fail to set cap_set_flag() for CAP_INHERITABLE\n");
	}
	if (cap_set_proc(new_caps) != 0) {
		      error("cap_drop, fail to set cap_set_proc()\n");
	}

	cap_free(new_caps);
}

/*!
 * The function set MAC label equal current server
 *
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @return Returns zero value correct and negative on error.
 */
int set_mac_label(elmac_lev_t level, elmac_cat_t cat, elmac_attr_t attr)
{
	elmac_context_t
		context;	/* MAC context */


	/*
	 * Server context
	 */
	context.mac.level = (elmac_lev_t) level;
	context.mac.category = (elmac_cat_t) cat;
	context.attr = attr;

	/*
	 * Set label
	 */
	return elmac_set_context_for_self(&context);
}

/*!
 * The function logging
 *
 * @param msgp message pool
 * @param type type message (INFO, DEBUG, ERROR)
 * @param msgt message text
 */
void journal_print(char **msgp, const char *type, const char *msgt)
{
	int len;
	char *msgpt;

	usleep(10000);
	if (!memcmp(type, "DEBUG:  ", strlen("DEBUG:  ")) && (!debug))
		return;

	len = strlen(*msgp) + strlen(type) + strlen(msgt) + 1;
	msgpt = (char *) realloc (*msgp, len);
	if (msgpt == NULL) {
		*msgp = msgpt;
		error("ERROR:  realloc\n");
	}

	strcat(msgpt, type);
	strcat(msgpt, msgt);

	*msgp = strdup(msgpt);

	return;
}

/**------------------------ INET TCP ------------------------*/

/*!
 * The function release inet tcp echo server
 *
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param priv_s flag PRIVSOCK
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int net_tcp_server(elmac_lev_t slevel, elmac_cat_t scat, int priv_s, char **msg)
{
	int parentfd = -1;		/* parent socket */
	int childfd;		/* child socket */
	socklen_t clientlen;	/* byte size of client's address */
	struct sockaddr_in
		serveraddr;	/* server's addr */
	struct sockaddr_in
		clientaddr;	/* client addr */
	char buf[BUFSIZE];	/* message buffer */
	char pr_buf[BUFSIZE * 2];	/* print message buffer */
	int optval;		/* flag value for setsockopt */
	int n;			/* message byte size */
	fd_set readfds;
	struct timeval timeout;	/* timeout */
	int i = 0;
	char *msg_serv;		/* messages on journal */
	int rc = 0;

	msg_serv = malloc(1);
	if (msg_serv == NULL) {
		rc = -1;
		goto server_exit;
	}

	memset(msg_serv, 0, 1);

	/*
	 * Set label
	 */
	if (set_mac_label(slevel, scat, priv_s ? ELMAC_ATTR_PRIVSOCK : 0x0) < 0) {
		journal_print(&msg_serv, "ERROR:  ",
			      "Could not set ELMAC context\n");
		rc = -1;
		goto server_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Server-setlabel() context %d:0x%llx is OK...\n",
		 slevel, scat);
	journal_print(&msg_serv, "DEBUG:  ", pr_buf);

	/*
	 * Create the parent socket
	 */
	parentfd = socket(AF_INET, SOCK_STREAM, 0);
	if (parentfd < 0) {
		journal_print(&msg_serv, "ERROR:  ", "Opening socket\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-socket() is OK...\n");

	/* Setsockopt: Handy debugging trick that lets
	 * us rerun the server immediately after we kill it;
	 * otherwise we have to wait about 20 secs.
	 * Eliminates "ERROR on binding: Address already in use" error.
	 */
	optval = 1;
	setsockopt(parentfd, SOL_SOCKET, SO_REUSEADDR,
	     (const void *)&optval , sizeof(int));

	/*
	 * Build the server's Internet address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	/*
	 * Set Internet address
	 */
	serveraddr.sin_family = AF_INET;

	/*
	 * Let the system figure out our IP address
	 */
	serveraddr.sin_addr.s_addr = htonl(INADDR_ANY);

	/*
	 * Set port we will listen on
	 */
	serveraddr.sin_port = htons((unsigned short)portno);

	/*
	 * Associate the parent socket with a port
	 */
	if (bind(parentfd, (struct sockaddr *) &serveraddr,
	    sizeof(serveraddr)) < 0) {
		journal_print(&msg_serv, "ERROR:  ", "on binding\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-bind() is OK...\n");

	/*
	 * Make this socket ready to accept connection requests
	 */
	if (listen(parentfd, 10) < 0) { /* allow 5 requests to queue up */
		journal_print(&msg_serv, "ERROR:  ", "on listen\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-listen() is OK...\n");

	/*
	 * Initialize some things for the main loop
	 */
	clientlen = sizeof(clientaddr);
	//fflush(stdout);

	/*
	 * Wait for connection request
	 */
	while(1) {
		i ++;
		if (cmd_exit == 1)
			goto server_exit;

		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			"Server-select() step %d is OK...\n", i);
		journal_print(&msg_serv, "DEBUG:  ", pr_buf);

		timeout.tv_sec = stimeout_s;
		timeout.tv_usec = STIMEOUT_U;

		/*
		 * Initialize the fd set
		 */
		FD_ZERO(&readfds);

 		/*
		 * Add socket fd
		 */
		FD_SET(parentfd, &readfds);
		if (select(parentfd + 1, &readfds, 0, 0, &timeout) < 0) {
			journal_print(&msg_serv, "DEBUG:  ",
				     "in select...\n");
			if (cmd_exit == 1)
				goto server_exit;
		}

		/*
		 * If a connection request has arrived, process it
		 */
		if (FD_ISSET(parentfd, &readfds)) {
			/*
			 * Accept, wait for a connection request
			 */
			childfd = accept(parentfd,
					 (struct sockaddr *) &clientaddr, &clientlen);
			if (childfd < 0) {
				journal_print(&msg_serv, "ERROR:  ", "on accept\n");
				rc = -1;
				goto server_exit;
			}
			journal_print(&msg_serv, "DEBUG:  ",
				     "Server-accept() is OK...\n");

			/*
			 * Set zero on rec buffer
			 */
			bzero(buf, BUFSIZE);

			/*
			 * Reciev buffer
			 */
			n = read(childfd, buf, BUFSIZE);
			if (n < 0) {
				journal_print(&msg_serv, "ERROR:  ",
					     "server reading from socket\n");
				rc = -1;
				goto server_exit;
			}
			memset(pr_buf, 0, sizeof(pr_buf));
			snprintf(pr_buf, sizeof(pr_buf),
				"Server-rec-messages(%s) is OK...\n", buf);
			journal_print(&msg_serv, "DEBUG:  ", pr_buf);

			/*
			 * Send buffer
			 */
			n = write(childfd, buf, strlen(buf));
			if (n < 0) {
				journal_print(&msg_serv, "ERROR:  ",
					       "writing to socket\n");
				rc = -1;
				goto server_exit;
			}
			snprintf(pr_buf, sizeof(pr_buf),
				"Server-send-messages(%s) is OK...\n", buf);
			journal_print(&msg_serv, "DEBUG:  ", pr_buf);


			/*
			 * Close child sock
			 */
			close(childfd);
		}
	}
server_exit:
	journal_print(&msg_serv, "DEBUG:  ", "Terminating server.\n");

	/*
	 * Close parent sock
	 */
	if (parentfd > -1)
		close(parentfd);

	journal_print(&msg_serv, " ", "\n\0");
	*msg = strdup(msg_serv);
	if (msg_serv)
		free(msg_serv);

	return rc;
}

/*!
 * The function release inet tcp echo client
 *
 * @param clevel client MAC level
 * @param ccat client MAC categories
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @return Returns zero value correct and negative on error.
 */
int net_tcp_client(elmac_lev_t clevel, elmac_cat_t ccat, elmac_lev_t slevel,
		   elmac_cat_t scat, char **msg)
{
	int sockfd = -1;	/* socket */
	struct sockaddr_in
		serveraddr;	/* server's addr */
	struct hostent *server;
	char buf_send[BUFSIZE];	/* message buffer for send*/
	char buf_rec[BUFSIZE];	/* message buffer for reciev*/
	char pr_buf[BUFSIZE * 2 + 256];	/* print message buffer */
	struct timeval timeout;	/* timeout */
	int n;			/* counter */
	char *msg_client;	/* messages on journal */
	int rc = 0;		/* result connet`s */

	msg_client = malloc(1);
	if (msg_client == NULL) {
		rc = -1;
		goto client_exit;
	}

	memset(msg_client, 0, 1);

	/*
	 * Set label
	 */
	rc = set_mac_label(clevel, ccat, 0x0);

	if(rc < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, could not set ELMAC label\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-setlabel() context %d:0x%llx is OK...\n",
		 clevel, ccat);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Set timeout
	 */
	timeout.tv_sec = ctimeout_s;
	timeout.tv_usec = CTIMEOUT_U;

	/*
	 * Create sock
	 */
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client opening socket\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ", "Client-socket() is OK...\n");

	server = gethostbyname("localhost");
	if (server == NULL) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, no such host\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-gethostbyname() is OK...\n");

	/*
	 * Set zero server address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	/*
	 * Set Internet address
	 */
	serveraddr.sin_family = AF_INET;
	bcopy((char *)server->h_addr,
	       (char *)&serveraddr.sin_addr.s_addr, server->h_length);

	/*
	 * Set port we will listen on
	 */
	serveraddr.sin_port = htons(portno);

	/*
	 * Set timeout for reciv
	 */
	if (setsockopt (sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() reciv is OK...\n");

	/*
	 * Set timeout for send
	 */
	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() send is OK...\n");

	/*
	 * Set zero on rec buffers
	 */
	bzero(buf_send, BUFSIZE);
	bzero(buf_rec, BUFSIZE);

	/*
	 * Connect
	 */
	if (connect(sockfd, (struct sockaddr *) &serveraddr,
	    sizeof(serveraddr)) < 0) {
		journal_print(&msg_client, "DEBUG:  ",
			     "Client-connect() is FAIL...\n");
		rc = 1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-connect() is OK...\n");

	/*
	 * Create send buffer
	 */
	snprintf(buf_send, sizeof(buf_send), "\"Client #%d, MAC label %d:0x%llx\"",
		 clevel, clevel, ccat);

	/*
	 * Send buffer
	 */
	n = write(sockfd, buf_send, strlen(buf_send));
	if (n < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, writing to socket\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-send-message(%s) is OK...\n", buf_send);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Reciev buffer
	 */
	n = read(sockfd, buf_rec, BUFSIZE - 1);
	if (n < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, reading from socket\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-reciv-message(%s) is OK...\n", buf_rec);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Compare buf_rec, buf_send
	 */
	if (memcmp(buf_rec, buf_send, BUFSIZE)) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, buf reciev no equal buf send\n");
		rc = -1;
	}
	//sleep(1);

client_exit:
	/*
	 * Print access array
	 */
	if (rc) {
		journal_print(&msg_client, "INFO:  ",
			     "--->connection not established\n");
	} else {
		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			 "--->connection established, client message (%s) "
			 " server response (%s)\n", buf_send, buf_rec);
		journal_print(&msg_client, "INFO:  ", pr_buf);
	}
	/*
	 * Close sock
	 */
	if (sockfd > -1)
		close(sockfd);

	journal_print(&msg_client, "", "\0");
	*msg = strdup(msg_client);
	if (msg_client)
		free(msg_client);


	return rc;
}

/**------------------------ INET UDP ------------------------*/
/*!
 * The function release inet udp echo server
 *
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param priv_s flag PRIVSOCK
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int net_udp_server(elmac_lev_t slevel, elmac_cat_t scat, int priv_s, char **msg)
{
	int parentfd = -1;	/* parent socket */
	struct sockaddr_in
		serveraddr;	/* server's addr */
	struct sockaddr_in
		clientaddr;	/* client addr */
	char buf[BUFSIZE];	/* message buffer */
	char pr_buf[BUFSIZE * 2];	/* print message buffer */
	int optval;		/* flag value for setsockopt */
	int n;			/* message byte size */
	struct timeval timeout;	/* timeout */
	int i = 0;
	char *msg_serv;		/* messages on journal */
	int rc = -1;

	msg_serv = malloc(1);
	if (msg_serv == NULL) {
		rc = -1;
		goto server_exit;
	}

	memset(msg_serv, 0, 1);

	timeout.tv_sec = stimeout_s;
	timeout.tv_usec = STIMEOUT_U;

	/*
	 * Set label
	 */
	if (set_mac_label(slevel, scat, priv_s ? ELMAC_ATTR_PRIVSOCK : 0x0) < 0) {
		journal_print(&msg_serv, "ERROR:  ",
			      "Could not set ELMAC context\n");
		rc = -1;
		goto server_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Server-setlabel() context %d:0x%llx is OK...\n",
		 slevel, scat);
	journal_print(&msg_serv, "DEBUG:  ", pr_buf);

	/*
	 * Create the parent socket
	 */
	parentfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (parentfd < 0) {
		journal_print(&msg_serv, "ERROR:  ", "opening socket\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-socket() is OK...\n");

	/* Setsockopt: Handy debugging trick that lets
	 * us rerun the server immediately after we kill it;
	 * otherwise we have to wait about 20 secs.
	 * Eliminates "ERROR on binding: Address already in use" error.
	 */
	optval = 1;
	setsockopt(parentfd, SOL_SOCKET, SO_REUSEADDR,
	     (const void *)&optval , sizeof(int));

	/*
	 * Set timeout for reciv
	 */
	if (setsockopt (parentfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_serv, "ERROR:  ", "setsockopt failed\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ",
		      "Server-setsockopt() reciv is OK...\n");

	/*
	 * Build the server's Internet address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	/*
	 * Set Internet address
	 */
	serveraddr.sin_family = AF_INET;

	/*
	 * Let the system figure out our IP address
	 */
	serveraddr.sin_addr.s_addr = htonl(INADDR_ANY);

	/*
	 * Set port we will listen on
	 */
	serveraddr.sin_port = htons((unsigned short)portno);

	/*
	 * Associate the parent socket with a port
	 */
	if (bind(parentfd, (struct sockaddr *) &serveraddr,
	    sizeof(serveraddr)) < 0) {
		journal_print(&msg_serv, "ERROR:  ", "on binding\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-bind() is OK...\n");

	/*
	 * Initialize some things for the main loop
	 */
	//fflush(stdout);

	/*
	 * Wait for connection request
	 */
	while(1) {

		i ++;

		if (cmd_exit == 1)
			goto server_exit;

		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			 "Server-select() step %d is OK...\n", i);
		journal_print(&msg_serv, "DEBUG:  ", pr_buf);

		/*
		 * If a connection request has arrived, process it
		 */
		socklen_t len = sizeof(clientaddr);

		/*
		 * Set zero on rec buffer
		 */
		bzero(buf, BUFSIZE);

		/*
		 * Reciev buffer
		 */
		n = recvfrom(parentfd, buf, sizeof(buf) - 1, 0,
			     (struct sockaddr *)&clientaddr, &len);
		if (n > 0) {
			memset(pr_buf, 0, sizeof(pr_buf));
			snprintf(pr_buf, sizeof(pr_buf),
				 "--->connection established, a message (%s) "
				 " has been received from the client\n", buf);
			journal_print(&msg_serv, "INFO:  ", pr_buf);
			rc = 0;
		}
	}
server_exit:
	if (rc) {
		journal_print(&msg_serv,
			      "INFO:  ", "--->connection not established\n");
	}
	journal_print(&msg_serv,
		      "DEBUG:  ", "Terminating server.\n");

	/*
	 * Close parent sock
	 */
	if (parentfd > -1)
		close(parentfd);

	journal_print(&msg_serv, " ", "\n\0");
	*msg = strdup(msg_serv);
	if (msg_serv)
		free(msg_serv);

	return rc;
}

/*!
 * The function release inet udp echo client
 *
 * @param clevel client MAC level
 * @param ccat client MAC categories
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int net_udp_client(elmac_lev_t clevel, elmac_cat_t ccat, elmac_lev_t slevel,
		   elmac_cat_t scat, char **msg)
{
	int sockfd = -1;	/* socket */
	struct sockaddr_in
		serveraddr;	/* server's addr */
	struct hostent *server;
	char buf_send[BUFSIZE];	/* message buffer for send*/
	char buf_rec[BUFSIZE];	/* message buffer for reciev*/
	char pr_buf[BUFSIZE];	/* print message buffer */
	struct timeval timeout;	/* timeout */
	int n;			/* counter */
	char *msg_client;	/* messages on journal */
	int rc = 0;		/* result connet`s */

	msg_client = malloc(1);
	if (msg_client == NULL) {
		rc = -1;
		goto client_exit;
	}

	memset(msg_client, 0, 1);

	/*
	 * Server label
	 */
	rc = set_mac_label(clevel, ccat, 0x0);

	if(rc < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, could not set ELMAC label\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-setlabel() context %d:0x%llx is OK...\n",
		 clevel, ccat);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Set timeout
	 */
	timeout.tv_sec = ctimeout_s;
	timeout.tv_usec = CTIMEOUT_U;

	/*
	 * Create sock
	 */
	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (sockfd < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client opening socket\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ", "Client-socket() is OK...\n");

	server = gethostbyname("localhost");
	if (server == NULL) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, no such host\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-gethostbyname() is OK...\n");

	/*
	 * Set zero server address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	/*
	 * Set Internet address
	 */
	serveraddr.sin_family = AF_INET;
	bcopy((char *)server->h_addr,
	       (char *)&serveraddr.sin_addr.s_addr, server->h_length);

	/*
	 * Set port we will listen on
	 */
	serveraddr.sin_port = htons(portno);

#if 1
	/*
	 * Set timeout for reciv
	 */
	if (setsockopt (sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() reciv is OK...\n");

	/*
	 * Set timeout for send
	 */
	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() send is OK...\n");
#endif

	/*
	 * Set zero on rec buffers
	 */
	bzero(buf_send, BUFSIZE);
	bzero(buf_rec, BUFSIZE);

	/*
	 * Create send buffer
	 */
	snprintf(buf_send, sizeof(buf_send), "\"Client #%d, MAC label %d:0x%llx\"",
		 clevel, clevel, ccat);

	/*
	 * Send buffer
	 */
	n = sendto(sockfd, buf_send, strlen(buf_send), 0,
		   (struct sockaddr *)&serveraddr, sizeof(serveraddr));
	if (n < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, writing to socket\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-send-message(%s) is OK...\n", buf_send);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

client_exit:
	/*
	 * Close sock
	 */
	if (sockfd > -1)
		close(sockfd);

	if (msg_client)
		journal_print(&msg_client, "", "\0");
	*msg = strdup(msg_client);
	if (msg_client)
		free(msg_client);

	return rc;
}

/**------------------------ UNIX TCP ------------------------*/

/*!
 * The function release unix tcp echo server
 *
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param priv_s flag PRIVSOCK
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int unix_tcp_server(elmac_lev_t slevel, elmac_cat_t scat, int priv_s,
		    char **msg)
{
	int parentfd = -1;	/* parent socket */
	int childfd;		/* child socket */
	struct sockaddr_un
		serveraddr;	/* server's addr */
	char buf[BUFSIZE];	/* message buffer */
	char pr_buf[BUFSIZE * 2];	/* print message buffer */
	int optval;		/* flag value for setsockopt */
	int n;			/* message byte size */
	fd_set readfds;
	struct timeval timeout;	/* timeout */
	int i = 0;
	char *msg_serv;		/* messages on journal */
	int rc = 0;

	msg_serv = malloc(1);
	if (msg_serv == NULL) {
		rc = -1;
		goto server_exit;
	}

	memset(msg_serv, 0, 1);

	/*
	 * Set label
	 */
	if (set_mac_label(slevel, scat, priv_s ? ELMAC_ATTR_PRIVSOCK : 0x0) < 0) {
		journal_print(&msg_serv, "ERROR:  ",
			      "Could not set ELMAC context\n");
		rc = -1;
		goto server_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Server-setlabel() context %d:0x%llx is OK...\n",
		 slevel, scat);
	journal_print(&msg_serv, "DEBUG:  ", pr_buf);

	/*
	 * Create the parent socket
	 */
	parentfd = socket(AF_UNIX, SOCK_STREAM, 0);
	if (parentfd < 0) {
		journal_print(&msg_serv, "ERROR:  ", "Opening socket\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-socket() is OK...\n");

	/* Setsockopt: Handy debugging trick that lets
	 * us rerun the server immediately after we kill it;
	 * otherwise we have to wait about 20 secs.
	 * Eliminates "ERROR on binding: Address already in use" error.
	 */
	optval = 1;
	setsockopt(parentfd, SOL_SOCKET, SO_REUSEADDR,
	     (const void *)&optval , sizeof(int));

	/*
	 * Build the server's Internet address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	/*
	 * Set Internet address
	 */
	serveraddr.sun_family = AF_UNIX;
	strcpy(serveraddr.sun_path, NAME);

	/*
	 * Associate the parent socket with a port
	 */
	if (bind(parentfd, (struct sockaddr *) &serveraddr,
	    sizeof(struct sockaddr_un)) < 0) {
		journal_print(&msg_serv, "ERROR:  ", "on binding\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-bind() is OK...\n");

	/*
	 * Make this socket ready to accept connection requests
	 */
	if (listen(parentfd, 10) < 0) { /* allow 5 requests to queue up */
		journal_print(&msg_serv, "ERROR:  ", "on listen\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-listen() is OK...\n");

	//fflush(stdout);

	/*
	 * Wait for connection request
	 */
	while(1) {
		i ++;
		if (cmd_exit == 1)
			goto server_exit;

		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			"Server-select() step %d is OK...\n", i);
		journal_print(&msg_serv, "DEBUG:  ", pr_buf);

		timeout.tv_sec = stimeout_s;
		timeout.tv_usec = STIMEOUT_U;

		/*
		 * Initialize the fd set
		 */
		FD_ZERO(&readfds);

 		/*
		 * Add socket fd
		 */
		FD_SET(parentfd, &readfds);
		if (select(parentfd + 1, &readfds, 0, 0, &timeout) < 0) {
			journal_print(&msg_serv, "DEBUG:  ",
				     "in select...\n");
			if (cmd_exit == 1)
				goto server_exit;
		}

		/*
		 * If a connection request has arrived, process it
		 */
		if (FD_ISSET(parentfd, &readfds)) {
			/*
			 * Accept, wait for a connection request
			 */
			childfd = accept(parentfd, 0, 0);
			if (childfd < 0) {
				journal_print(&msg_serv, "ERROR:  ", "on accept\n");
				rc = -1;
				goto server_exit;
			}
			journal_print(&msg_serv, "DEBUG:  ",
				     "Server-accept() is OK...\n");
			/*
			 * Set zero on rec buffer
			 */
			bzero(buf, BUFSIZE);

			/*
			 * Reciev buffer
			 */
			n = read(childfd, buf, BUFSIZE);
			if (n < 0) {
				journal_print(&msg_serv, "ERROR:  ",
					     "Server reading from socket\n");
				rc = -1;
				goto server_exit;
			}
			memset(pr_buf, 0, sizeof(pr_buf));
			snprintf(pr_buf, sizeof(pr_buf),
				"Server-rec-messages(%s) is OK...\n", buf);
			journal_print(&msg_serv, "DEBUG:  ", pr_buf);

			/*
			 * Send buffer
			 */
			n = write(childfd, buf, strlen(buf));
			if (n < 0) {
				journal_print(&msg_serv, "ERROR:  ",
					       "writing to socket\n");
				rc = -1;
				goto server_exit;
			}
			snprintf(pr_buf, sizeof(pr_buf),
				"Server-send-messages(%s) is OK...\n", buf);
			journal_print(&msg_serv, "DEBUG:  ", pr_buf);

			/*
			 * Close child sock
			 */
			close(childfd);
		}
	}
server_exit:

	/*
	 * Close parent sock
	 */
	if (parentfd > -1)
		close(parentfd);


	journal_print(&msg_serv, "DEBUG:  ", "Terminating server.\n");

	/*
	 * Close parent sock
	 */
	journal_print(&msg_serv, " ", "\n\0");
	*msg = strdup(msg_serv);
	if (msg_serv)
		free(msg_serv);


	/*
	 * Remove file
	 */
	unlink(NAME);

	return rc;
}

/*!
 * The function release unix tcp echo client
 *
 * @param clevel client MAC level
 * @param ccat client MAC categories
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int unix_tcp_client(elmac_lev_t clevel, elmac_cat_t ccat, elmac_lev_t slevel,
		    elmac_cat_t scat, char **msg)
{
	int sockfd = -1;	/* socket */
	struct sockaddr_un
		serveraddr;	/* server's addr */
	char buf_send[BUFSIZE];	/* message buffer for send*/
	char buf_rec[BUFSIZE];	/* message buffer for reciev*/
	char pr_buf[BUFSIZE * 2 + 256];	/* print message buffer */
	struct timeval timeout;	/* timeout */
	int n;			/* counter */
	char *msg_client;	/* messages on journal */
	int rc = 0;		/* result connet`s */

	msg_client = malloc(1);
	if (msg_client == NULL) {
		rc = -1;
		goto client_exit;
	}

	memset(msg_client, 0, 1);

	/*
	 * Set label
	 */
	rc = set_mac_label(clevel, ccat, 0x0);
	if(rc < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, could not set ELMAC label\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-setlabel() context %d:0x%llx is OK...\n",
		 clevel, ccat);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Set timeout
	 */
	timeout.tv_sec = ctimeout_s;
	timeout.tv_usec = CTIMEOUT_U;

	/*
	 * Create sock
	 */
	sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
	if (sockfd < 0) {
		journal_print(&msg_client, "ERROR:  ",
			      "Client opening socket\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ", "Client-socket() is OK...\n");

	/*
	 * Set zero server address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	serveraddr.sun_family = AF_UNIX;
	strcpy(serveraddr.sun_path, NAME);

	/*
	 * Set Unix address
	 */
	serveraddr.sun_family = AF_UNIX;
	strcpy(serveraddr.sun_path, NAME);

	/*
	 * Set timeout for reciv
	 */
	if (setsockopt (sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, reciv setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() reciv is OK...\n");

	/*
	 * Set timeout for send
	 */
	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() send is OK...\n");

	/*
	 * Set zero on rec buffers
	 */
	bzero(buf_send, BUFSIZE);
	bzero(buf_rec, BUFSIZE);

	/*
	 * Connect
	 */
	if (connect(sockfd, (struct sockaddr *) &serveraddr,
	    sizeof(struct sockaddr_un)) < 0) {
		journal_print(&msg_client, "DEBUG:  ",
			     "Client-connect() is FAIL...\n");
		rc = 1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-connect() is OK...\n");

	/*
	 * Create send buffer
	 */
	snprintf(buf_send, sizeof(buf_send), "\"Client #%d, MAC label %d:0x%llx\"",
		 clevel, clevel, ccat);

	/*
	 * Send buffer
	 */
	n = write(sockfd, buf_send, strlen(buf_send));
	if (n < 0) {
		journal_print(&msg_client, "DEBUG:  ",
			     "Client, writing to socket\n");
		rc = 1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-send-message(%s) is OK...\n", buf_send);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);


	/*
	 * Reciev buffer
	 */
	n = read(sockfd, buf_rec, BUFSIZE - 1);
	if (n < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, reading from socket\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-reciv-message(%s) is OK...\n", buf_rec);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Compare buf_rec, buf_send
	 */
	if (memcmp(buf_rec, buf_send, BUFSIZE)) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, buf reciev no equal buf send\n");
		rc = -1;
	}

client_exit:
	/*
	 * Print access array
	 */
	if (rc) {
		journal_print(&msg_client, "INFO:  ",
			     "--->connection not established\n");
	} else {
		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			 "--->connection established, client message (%s) "
			 " server response (%s)\n", buf_send, buf_rec);
		journal_print(&msg_client, "INFO:  ", pr_buf);
	}
	/*
	 * Close sock
	 */
	if (sockfd > -1)
		close(sockfd);

	journal_print(&msg_client, "", "\0");
	*msg = strdup(msg_client);
	if (msg_client)
		free(msg_client);


	return rc;
}

/**------------------------ UNIX UDP ------------------------*/
/*!
 * The function release unix udp echo server
 *
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param priv_s flag PRIVSOCK
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int unix_udp_server(elmac_lev_t slevel, elmac_cat_t scat, int priv_s, char **msg)
{
	int parentfd = -1;	/* parent socket */
	struct sockaddr_un
		serveraddr;	/* server's addr */
	struct sockaddr_in
		clientaddr;	/* client addr */
	char buf[BUFSIZE];	/* message buffer */
	char pr_buf[BUFSIZE * 2];	/* print message buffer */
	int optval;		/* flag value for setsockopt */
	int n;			/* message byte size */
	struct timeval timeout;	/* timeout */
	int i = 0;
	char *msg_serv;		/* messages on journal */
	int rc = -1;

	msg_serv = malloc(1);
	if (msg_serv == NULL) {
		rc = -1;
		goto server_exit;
	}

	memset(msg_serv, 0, 1);

	/*
	 * Set label
	 */
	if (set_mac_label(slevel, scat, priv_s ? ELMAC_ATTR_PRIVSOCK : 0x0) < 0) {
		journal_print(&msg_serv, "ERROR:  ",
			      "Could not set ELMAC context\n");
		rc = -1;
		goto server_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Server-setlabel() context %d:0x%llx is OK...\n",
		 slevel, scat);
	journal_print(&msg_serv, "DEBUG:  ", pr_buf);

	timeout.tv_sec = stimeout_s;
	timeout.tv_usec = STIMEOUT_U;

	/*
	 * Create the parent socket
	 */
	parentfd = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (parentfd < 0) {
		journal_print(&msg_serv, "ERROR:  ", "opening socket\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-socket() is OK...\n");

	/* Setsockopt: Handy debugging trick that lets
	 * us rerun the server immediately after we kill it;
	 * otherwise we have to wait about 20 secs.
	 * Eliminates "ERROR on binding: Address already in use" error.
	 */
	optval = 1;
	setsockopt(parentfd, SOL_SOCKET, SO_REUSEADDR,
	     (const void *)&optval , sizeof(int));

	/*
	 * Set timeout for reciv
	 */
	if (setsockopt (parentfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_serv, "ERROR:  ", "setsockopt failed\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ",
		      "Server-setsockopt() reciv is OK...\n");

	/*
	 * Build the server's Internet address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	/*
	 * Set Internet address
	 */
	serveraddr.sun_family = AF_UNIX;
	strcpy(serveraddr.sun_path, NAME);

	/*
	 * Associate the parent socket with a port
	 */
	if (bind(parentfd, (struct sockaddr *) &serveraddr,
	    sizeof(struct sockaddr_un)) < 0) {
		journal_print(&msg_serv, "ERROR:  ", "on binding\n");
		rc = -1;
		goto server_exit;
	}
	journal_print(&msg_serv, "DEBUG:  ", "Server-bind() is OK...\n");

	/*
	 * Initialize some things for the main loop
	 */
	//fflush(stdout);

	/*
	 * Wait for connection request
	 */
	while(1) {

		i ++;

		if (cmd_exit == 1)
			goto server_exit;

		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			 "Server-select() step %d is OK...\n", i);
		journal_print(&msg_serv, "DEBUG:  ", pr_buf);

		/*
		 * If a connection request has arrived, process it
		 */
		socklen_t len = sizeof(clientaddr);

		/*
		 * Set zero on rec buffer
		 */
		bzero(buf, BUFSIZE);

		/*
		 * Reciev buffer
		 */
		n = recvfrom(parentfd, buf, sizeof(buf) - 1, 0,
			     (struct sockaddr *)&clientaddr, &len);
		if (n > 0) {
			memset(pr_buf, 0, sizeof(pr_buf));
			snprintf(pr_buf, sizeof(pr_buf),
				 "--->connection established, a message (%s) "
				 " has been received from the client\n", buf);
			journal_print(&msg_serv, "INFO:  ", pr_buf);
			rc = 0;
		}
	}
server_exit:
	if (rc) {
		journal_print(&msg_serv,
			      "INFO:  ", "--->connection not established\n");
	}
	journal_print(&msg_serv,
		      "DEBUG:  ", "Terminating server.\n");

	/*
	 * Close parent sock
	 */
	if (parentfd > -1)
		close(parentfd);

	journal_print(&msg_serv, " ", "\n\0");
	*msg = strdup(msg_serv);
	if (msg_serv)
		free(msg_serv);

	/*
	 * Remove file
	 */
	unlink(NAME);

	return rc;
}

/*!
 * The function release unix udp echo client
 *
 * @param clevel client MAC level
 * @param ccat client MAC categories
 * @param slevel server MAC level
 * @param scat server MAC categories
 * @param msg messages pool
 * @return Returns zero value correct and negative on error.
 */
int unix_udp_client(elmac_lev_t clevel, elmac_cat_t ccat, elmac_lev_t slevel,
		    elmac_cat_t scat, char **msg)
{
	int sockfd = -1;	/* socket */
	struct sockaddr_un
		serveraddr;	/* server's addr */
	char buf_send[BUFSIZE];	/* message buffer for send*/
	char buf_rec[BUFSIZE];	/* message buffer for reciev*/
	char pr_buf[BUFSIZE];	/* print message buffer */
	struct timeval timeout;	/* timeout */
	int n;			/* counter */
	char *msg_client;	/* messages on journal */
	int rc = 0;		/* result connet`s */

	msg_client = malloc(1);
	if (msg_client == NULL) {
		rc = -1;
		goto client_exit;
	}

	memset(msg_client, 0, 1);

	/*
	 * Server label
	 */
	rc = set_mac_label(clevel, ccat, 0x0);

	if(rc < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, could not set ELMAC label\n");
		rc = -1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-setlabel() context %d:0x%llx is OK...\n",
		 clevel, ccat);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

	/*
	 * Set timeout
	 */
	timeout.tv_sec = ctimeout_s;
	timeout.tv_usec = CTIMEOUT_U;

	/*
	 * Create sock
	 */
	sockfd = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sockfd < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client opening socket\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ", "Client-socket() is OK...\n");

	/*
	 * Set zero server address
	 */
	bzero((char *) &serveraddr, sizeof(serveraddr));

	serveraddr.sun_family = AF_UNIX;
	strcpy(serveraddr.sun_path, NAME);

	/*
	 * Set Unix address
	 */
	serveraddr.sun_family = AF_UNIX;
	strcpy(serveraddr.sun_path, NAME);

	/*
	 * Set timeout for reciv
	 */
	if (setsockopt (sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() reciv is OK...\n");
	/*
	 * Set timeout for send
	 */
	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout,
	    sizeof(timeout)) < 0) {
		journal_print(&msg_client, "ERROR:  ",
			     "Client, setsockopt failed\n");
		rc = -1;
		goto client_exit;
	}
	journal_print(&msg_client, "DEBUG:  ",
		      "Client-setsockopt() send is OK...\n");

	/*
	 * Set zero on rec buffers
	 */
	bzero(buf_send, BUFSIZE);
	bzero(buf_rec, BUFSIZE);

	/*
	 * Create send buffer
	 */
	snprintf(buf_send, sizeof(buf_send), "\"Client #%d, MAC label %d:0x%llx\"",
		 clevel, clevel, ccat);

	/*
	 * Send buffer
	 */
	n = sendto(sockfd, buf_send, strlen(buf_send), 0,
		   (struct sockaddr *)&serveraddr, sizeof(serveraddr));
	if (n < 0) {
		journal_print(&msg_client, "DEBUG:  ",
			     "Client, writing to socket\n");
		rc = 1;
		goto client_exit;
	}
	memset(pr_buf, 0, sizeof(pr_buf));
	snprintf(pr_buf, sizeof(pr_buf),
		 "Client-send-message(%s) is OK...\n", buf_send);
	journal_print(&msg_client, "DEBUG:  ", pr_buf);

client_exit:
	/*
	 * Close sock
	 */
	if (sockfd > -1)
		close(sockfd);

	if (msg_client)
	journal_print(&msg_client, "", "\0");
	*msg = strdup(msg_client);
	if (msg_client)
		free(msg_client);

	return rc;
}


/*!
 * The function release iteration tests
 *
 * @param tserver pointer type function server
 * @param tclient pointer type function client
 * @param family AF_INET, AF_UNIX
 * @param type SOCK_STREAM, SOCK_DGRAM
 */
void test_run(fserver tserver, fclient tclient, char *family, char *type)
{
#define PRIV_FLAG_ENABLE	2
#define PRIV_FLAG_DISABLE	1
	int pid;		/* PID servers */
	elmac_lev_t cl, sl;	/* level on client and server */
	elmac_cat_t cc, sc;	/* categories on client and server */
	int priv_s = 0;		/* flags privilage socket server */
	int priv_flags;		/* flags privilage socket server */
	char pr_buf[BUFSIZE];	/* print message buffer */
	char *msg_journal;	/* messages on journal */

	msg_journal = malloc(1);
	if (msg_journal == NULL) {
		error("Memmory alloc");
	}

	memset(msg_journal, 0, 1);

	mode ? (priv_flags = PRIV_FLAG_ENABLE) : (priv_flags = PRIV_FLAG_DISABLE);

	for (priv_s = 0; priv_s < priv_flags; priv_s ++) {
		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			 "\n\nINFO:  *** TEST %s,%s. Server mac-privilage socket %s\n",
			 family, type, priv_s ? "YES" : "NO");
		journal_print(&msg_journal, "", pr_buf);
		memset(pr_buf, 0, sizeof(pr_buf));
		snprintf(pr_buf, sizeof(pr_buf),
			 "INFO:  -----------------------------------------"
		         "------------------------------------------------\n\n");
		journal_print(&msg_journal, "", pr_buf);
		for (sl = 0; sl <= max_level; sl ++) {
			for (sc = 0; sc <= max_category; sc ++) {
				int rc;
				int status;

				for (cl = 0; cl <= max_level; cl ++) {
					for (cc = 0; cc <= max_category; cc ++) {
						int fd[2];

						cmd_exit = 0;
						pipe(fd);
						pid = fork();
						if (pid < 0)
							exit(1);
						if (pid == 0) {
							char	*msg = NULL;
							size_t len;

							close(fd[0]);
							rc = tserver(sl, sc, priv_s, &msg);
							write(fd[1], &rc, sizeof(rc));
							if (msg) {
								len = strlen (msg);
								write(fd[1], &len, sizeof(len));
								write(fd[1], (void *) msg, len);
								free(msg);
							} else {
								len = 0;
								write(fd[1], &len, sizeof(len));
							}
							close(fd[1]);
							if (cmd_exit)
								signal(SIGHUP, SIG_DFL);
							exit(0);
						} else {
							size_t len;
							char pr_buf[BUFSIZE];	/* print message buffer */
							char *msgc =NULL;	/* client messages on journal */
							char *msgs =NULL;	/* server messages on journal */

							close(fd[1]);

							snprintf(pr_buf, sizeof(pr_buf),
								"INFO:  %s,%s. "
								"Connection client(%d:0x%llx) "
								"to server(%d:0x%llx,%s) ... \n",
								family, type, cl, cc, sl, sc,
								priv_s ? "PRIVSOCK" : "NO PRIVSOCK");
								journal_print(&msg_journal, "", pr_buf);
							usleep(500000);
							if (tclient(cl, cc, sl, sc, &msgc) == -1) {
								memset(pr_buf, 0, sizeof(pr_buf));
								snprintf(pr_buf, sizeof(pr_buf),
									 "ERROR:   %s,%s. %s\n",
	 								 family, type, msgc);
								error(pr_buf);
							}
							if (msgc) {
								journal_print(&msg_journal, "", msgc);
								free(msgc);
							}
							/*
							 * For enable send signal
							 */
							set_mac_label(sl, sc, 0x0);
							kill(pid, SIGHUP);
							while (((rc = waitpid(pid, &status, 0)) == (pid_t)-1) &&
								(errno == EINTR));
							if (rc == (pid_t)-1) {
								exit(1);
							}
							if (!WIFEXITED(status) || WIFSIGNALED(status) > 0) {
								exit(1);
							}
							read(fd[0], &rc, sizeof(rc));
							read(fd[0], &len, sizeof(len));
							if (len) {
								msgs = (char *) malloc(len + 1);
								if (msgs == NULL) {
									error("Memmory alloc error");
								}
								memset(msgs, 0, len + 1);
								while (read(fd[0], msgs, len + 1) != 0);;
							}
							if (msgs) {
								journal_print(&msg_journal, "", msgs);
								free(msgs);
							}
							close(fd[0]);
						}
					}
				}
			}
		}
	}
	write(1, msg_journal, strlen(msg_journal));
	if (msg_journal)
		free(msg_journal);
}


/*!
 * Main
 */
int main(int argc, char *argv[])
{
	fserver f_net_tcp_server, f_unix_tcp_server;
	fclient f_net_tcp_client, f_unix_tcp_client;
	fserver f_net_udp_server, f_unix_udp_server;
	fclient f_net_udp_client, f_unix_udp_client;
	int timeout;
	char *endptr, *str;

	if (argc < 4) {
		fprintf(stderr,"Usage: %s port mode debug\n"
			       "options:\n"
			       "\tport - number port,\n"
			       "\tmode - test mode: \"0\" fast mode, \"1\" full mode\n"
			       "\ttimeout - timeout \n"
			       "\tdebug - debug mode: \"0\" disable, \"1\" enable\n", argv[0]);
		exit(1);
	}

	/*
	 * Get port
	 */
	portno = atoi(argv[1]);

	/*
	 * Get mode
	 */
	mode = atoi(argv[2]);
	if ((mode < 0) || (mode > 1)) {
		fprintf(stderr,"Usage: %s port mode debug\n"
			       "options:\n"
			       "\tport - number port,\n"
			       "\tmode - test mode: \"0\" fast mode, \"1\" full mode\n"
			       "\ttimeout - timeout \n"
			       "\tdebug - debug mode: \"0\" disable(default), \"1\" enable\n", argv[0]);
		exit(1);
	}

	str = argv[3];
	errno = 0;
	timeout = strtol(str, &endptr, 10);
	if ((errno != 0) || (endptr == str)) {
			fprintf(stderr,"Usage: %s port mode debug\n"
			       "options:\n"
			       "\tport - number port,\n"
			       "\tmode - test mode: \"0\" fast mode, \"1\" full mode\n"
			       "\ttimeout - timeout \n"
			       "\tdebug - debug mode: \"0\" disable(default), \"1\" enable\n", argv[0]);
		exit(1);
    }

	stimeout_s = timeout;
	ctimeout_s = timeout;

	if (argc > 4)
		debug = atoi(argv[4]);


	/*
	 * Set signal handler
	 */
	signal(SIGHUP, signal_handler);

	/*
	 * Capabilities drop
	 */
	cap_drop();

	/*
	 * Test level, categories
	 */
	if (mode) {
		max_level = 2;
		max_category = 0x3;
	} else {
		max_level = 2;
		max_category = 0x0;
	}

	/*
	 * Inet strem socket testmode
	 */
	f_net_tcp_server = net_tcp_server;
	f_net_tcp_client = net_tcp_client;
	test_run(f_net_tcp_server, f_net_tcp_client, "AF_INET", "SOCK_STREAM");

	/*
	 * Inet dgram socket testmode
	 */
	f_net_udp_server = net_udp_server;
	f_net_udp_client = net_udp_client;
	test_run(f_net_udp_server, f_net_udp_client, "AF_INET", "SOCK_DGRAM");

	/*
	 * Unix strem socket testmode
	 */
	f_unix_tcp_server = unix_tcp_server;
	f_unix_tcp_client = unix_tcp_client;
	test_run(f_unix_tcp_server, f_unix_tcp_client, "AF_UNIX", "SOCK_STREAM");

	/*
	 * Unix dgram socket testmode
	 */
	f_unix_udp_server = unix_udp_server;
	f_unix_udp_client = unix_udp_client;
	test_run(f_unix_udp_server, f_unix_udp_client, "AF_UNIX", "SOCK_DGRAM");

	return 0;
}
