\set VERBOSITY terse

-- Create pgaudit extension
CREATE EXTENSION IF NOT EXISTS pgaudit;

-- Make sure events don't get logged twice when session logging
SET pgaudit.log = 'all';
SET pgaudit.log_client = ON;
SET pgaudit.log_level = 'notice';

CREATE TABLE tmp (id int, data text);
CREATE TABLE tmp2 AS (SELECT * FROM tmp);

MAC LABEL FOR maclinux ON TABLE tmp is '0:0:3f';
MAC LABEL FOR maclinux ON TABLE tmp SHOW;
MAC LABEL FOR maclinux ON TABLE tmp2 is '0:0:3f';
MAC LABEL FOR maclinux ON TABLE tmp2 SHOW;

-- Reset log_client first to show that audits logs are not set to client
RESET pgaudit.log_client;

DROP TABLE tmp;
DROP TABLE tmp2;

RESET pgaudit.log;
RESET pgaudit.log_level;

SELECT current_user \gset

--
-- Set pgaudit parameters for the current (super)user.
ALTER ROLE :current_user SET pgaudit.log = 'Role';
ALTER ROLE :current_user SET pgaudit.log_level = 'notice';
ALTER ROLE :current_user SET pgaudit.log_client = ON;
 
\connect - :current_user;

--
-- Create auditor role
CREATE ROLE auditor;

--
-- Create first test user
CREATE USER user1 password 'password';
ALTER ROLE user1 SET pgaudit.log = 'ddl, ROLE';
ALTER ROLE user1 SET pgaudit.log_level = 'notice';

ALTER ROLE user1 PassWord 'password2' NOLOGIN;
ALTER USER user1 encrypted /* random comment */PASSWORD
         /* random comment */
    'md565cb1da342495ea6bb0418a6e5718c38' LOGIN;
ALTER ROLE user1 SET pgaudit.log_client = ON;

--
-- Create, select, drop (select will not be audited)
\connect - user1

CREATE TABLE public.test
(
        id INT
);

SELECT *
  FROM test;

DROP TABLE test;

--
-- Create second test user
\connect - :current_user

CREATE ROLE user2 LOGIN password 'password';
ALTER ROLE user2 SET pgaudit.log = 'Read, write';
ALTER ROLE user2 SET pgaudit.log_catalog = OFF;
ALTER ROLE user2 SET pgaudit.log_client = ON;
ALTER ROLE user2 SET pgaudit.log_level = 'warning';
ALTER ROLE user2 SET pgaudit.role = auditor;
ALTER ROLE user2 SET pgaudit.log_statement_once = ON;

--
-- Setup role-based tests
CREATE TABLE test2
(
        id INT
);

GRANT SELECT, INSERT, UPDATE, DELETE
    ON test2
    TO user2, user1;

GRANT SELECT (id), INSERT (id), UPDATE (id)
    ON test2
    TO user2, user1;

GRANT SELECT, UPDATE
    ON TABLE public.test2
    TO auditor;

CREATE TABLE test3
(
         id INT
);

GRANT SELECT, INSERT, UPDATE, DELETE
    ON test3
    TO user2;

GRANT SELECT (id), INSERT (id), UPDATE (id)
    ON test3
    TO user2;

GRANT INSERT
    ON TABLE public.test3
    TO auditor;

CREATE FUNCTION test2_insert() RETURNS TRIGGER AS $$
BEGIN
         UPDATE test2
         SET id = id + 90
         WHERE id = new.id;

         RETURN new;
END $$ LANGUAGE plpgsql security definer;
ALTER FUNCTION test2_insert() OWNER TO user1;

CREATE TRIGGER test2_insert_trg
         AFTER INSERT ON test2
         FOR EACH ROW EXECUTE PROCEDURE test2_insert();

CREATE FUNCTION test2_change(change_id int) RETURNS void AS $$
BEGIN
         UPDATE test2
            SET id = id + 1
          WHERE id = change_id;
END $$ LANGUAGE plpgsql security definer;
ALTER FUNCTION test2_change(int) OWNER TO user2;

CREATE VIEW vw_test3 AS
SELECT *
   FROM test3;

GRANT SELECT
   ON vw_test3
   TO user2;

GRANT SELECT
   ON vw_test3
   TO auditor;

\connect - user2

--
-- Role-based tests
SELECT count(*)
  FROM
(
         SELECT relname
           FROM pg_class
           LIMIT 1
) SUBQUERY;

SELECT *
    FROM test3, test2;

--
-- Object logged because of:
-- select on vw_test3
-- select on test2
SELECT *
    FROM vw_test3, test2;


--
-- Change permissions of user 2 so that only object logging will be done
\connect - :current_user
ALTER ROLE user2 SET pgaudit.log = 'NONE';

\connect - user2

--
-- Create test4 and add permissions
CREATE TABLE test4
(
         id int,
         name text
);

GRANT SELECT (name)
    ON TABLE public.test4
    TO auditor;

GRANT UPDATE (id)
    ON TABLE public.test4
    TO auditor;

GRANT insert (name)
    ON TABLE public.test4
    TO auditor;

--
-- Not object logged
SELECT id
   FROM public.test4;

--
-- Object logged because of:
-- select (name) on test4
SELECT name
   FROM public.test4;

--
-- Not object logged
INSERT INTO public.test4 (id)
                               VALUES (1);

--
-- Object logged because of:
-- insert (name) on test4
INSERT INTO public.test4 (name)
                               VALUES ('test');

--
-- Not object logged
UPDATE public.test4
    SET name = 'foo';

--
-- Object logged because of:
-- update (id) on test4
UPDATE public.test4
    SET id = 1;

--
-- Object logged because of:
-- update (name) on test4
-- update (name) takes precedence over select (name) due to ordering
update public.test4 set name = 'foo' where name = 'bar';


-- Change back to superuser to do exhaustive tests
\connect - :current_user

DROP EXTENSION pgaudit;

-- Cleanup
-- Set client_min_messages up to warning to avoid noise
SET client_min_messages = 'warning';

ALTER ROLE :current_user RESET pgaudit.log;
ALTER ROLE :current_user RESET pgaudit.log_catalog;
ALTER ROLE :current_user RESET pgaudit.log_client;
ALTER ROLE :current_user RESET pgaudit.log_level;
ALTER ROLE :current_user RESET pgaudit.log_parameter;
ALTER ROLE :current_user RESET pgaudit.log_relation;
ALTER ROLE :current_user RESET pgaudit.log_statement_once;
ALTER ROLE :current_user RESET pgaudit.role;

RESET pgaudit.log;
RESET pgaudit.log_catalog;
RESET pgaudit.log_level;
RESET pgaudit.log_parameter;
RESET pgaudit.log_relation;
RESET pgaudit.log_statement_once;
RESET pgaudit.role;

DROP VIEW vw_test3;

DROP TABLE test2;
DROP TABLE test3;
DROP TABLE test4;

DROP FUNCTION test2_insert();
DROP FUNCTION test2_change(int);

DROP USER user1;
DROP USER user2;
DROP ROLE auditor;

