#!/bin/bash

set +xv

major_version()
{
    local ver=${1}

    # Удаляем из номера версии всё кроме точек
    dots=${ver//[^.]}

    [ -n "${dots}" ] && ver=$(echo ${ver} | cut -d'.' -f -${#dots})

    echo ${ver}
}

opt_user_postgresql_version=$(psql --version | cut -d' ' -f 3)
opt_user_custom_installation="false"

while getopts "v:" opt
do
    case $opt in
        v) opt_user_postgresql_version=${OPTARG};;
        j) opt_user_custom_installation="true";;
    esac
done

opt_user_postgresql_version=$(major_version ${opt_user_postgresql_version})

[ "${opt_user_custom_installation}" = "true" ] && {
    echo "ВНИМАНИЕ: Тестирование Postgres версии ${opt_user_postgresql_version}, расположенного по нестандартному пути"
}

[ -d /sys/fs/elmacfs ] && elmacfs="/sys/fs/elmacfs" || elmacfs="/elmac"

zcat /proc/config.gz | grep -q _ELMAC=y || {
	echo "Отсутствует поддержка ELMAC в ядре"
	exit 1
}

cat ${elmacfs}/mac_write_up &>/dev/null || {
	echo "ELMAC не активирован либо неисправен"
	exit 1
}

echo "ELMAC активен"

dpkg -l | grep -q postgresql || {
	echo "Пакет postgresql не установлен"
	exit 1
}

[ "${opt_user_custom_installation}" = "false" ] && {
    [ "${opt_user_postgresql_version}" -lt 14 ] && {
        dpkg -l | grep postgresql 2>/dev/null | grep pgaudit 2>/dev/null || {
            echo "Пакет postgresql-*-pgaudit не установлен"
            exit 1
        }
    }

    plperl_package="postgresql-plperl-${opt_user_postgresql_version}"
    dpkg -l | grep ${plperl_package} 2>/dev/null || {
        echo "Не установлен пакет ${plperl_package}"
        exit 1
    }
}

exit 0
