#!/bin/bash

TEST_PATH=`cd  $(dirname $0); pwd`
CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)

TEST_DIR=.

source ${TEST_DIR}/lib_log

PG_CREATECLUSTER_BIN=pg_createcluster
PG_DROPCLUSTER_BIN=pg_dropcluster
PG_LSCLUSTERS=pg_lsclusters

which psql 1>/dev/null || { echo "Постгрес не установлен"; exit 1; }

which elmacexec 1>/dev/null || {
    echo "elmacexec is not installed"
    echo "apt-get install elmac-utils"
    exit 1
}

default_localhost_name="localhost"

# Do not stop DB after testing
opt_user_do_not_stop_db="false"
# MAC is enabled by default
opt_user_enable_mac_row="true"
# Sleep before each test
opt_sleep_sec_before_test=0
# Setup DB only. Do not perform any test
opt_user_setup_only="false"
# Skip setup, go to tests directly
opt_user_skip_setup="false"
# Run only this test
opt_user_num_of_test=
# Specify port number
opt_user_port=
# hostname
opt_user_hostname=${default_localhost_name}
# Postgresql version
opt_user_version=
# Custom installation path
opt_user_installation_prefix=
# Exclude tests
opt_user_exclude_tests=
# psql connect timeout
opt_psql_connect_timeout=10

# Exclude tests hardcode
exclude_tests=

# GCOV
opt_user_gcov="false"
# GCOV coverage directory
opt_user_gcov_output_dir=

# FTRACE results directory
opt_user_ftrace_output_dir="results"

failure="false"
pg_ver_dir=
opt_user_print_tests="false"

SUCCESS=0
FAILED=1

IS_POSTGRESQL_9_2="false"
which ${PG_DROPCLUSTER_BIN} 1>/dev/null 2>&1 || IS_POSTGRESQL_9_2="true"

# Импортировать тестовые функции
source ${TEST_DIR}/test_object_database
source ${TEST_DIR}/test_object_tablespace
source ${TEST_DIR}/test_object_schema
source ${TEST_DIR}/test_object_table
source ${TEST_DIR}/test_object_column
source ${TEST_DIR}/test_object_foreign_table
source ${TEST_DIR}/test_object_view
source ${TEST_DIR}/test_object_sequence
source ${TEST_DIR}/test_object_row_view
source ${TEST_DIR}/test_object_type
source ${TEST_DIR}/test_object_function
source ${TEST_DIR}/test_object_language
source ${TEST_DIR}/test_audit
if [ "${IS_POSTGRESQL_9_2}" = "true" ]; then
    source ${TEST_DIR}/test_object_row_table_9_2
else
    source ${TEST_DIR}/test_object_row_table
fi
source ${TEST_DIR}/test_object_table_ftrace

#
# Массив с тестовыми функциями.
# Новые тесты добавлять сюда.
#
tests=(
    test_object_database
    test_object_tablespace
    test_object_schema
    test_object_table
    test_object_column
    test_object_foreign_table
    test_object_view
    test_object_sequence
    test_object_row_view
    test_object_row_table
    test_object_type
    test_object_function
    test_object_language
    test_audit
)

usage() {
    echo -e "СПРАВКА: ${0##*/} ОПЦИИ\n"
    echo -e "  -l\t\tВывести на экран названия и номера тестов."
    echo -e "  -n ЧИСЛО\tВыполнить только данный тест."
    echo -e "  -s\t\tВыполнить только настройку пользователей и кластера." \
        "\n\t\t(Подготовить окружение и БД к тесту)."
    echo -e "  -k\t\tПропустить настройку пользователей и кластера."
    echo -e "  -i СЕРВЕР\tIP-адрес либо имя хоста сервера БД. Имя хоста по умолчанию ${default_localhost_name}."
    echo -e "  -p ЧИСЛО\tНомер порта."
    echo -e "  -t ЧИСЛО\tТаймаут подключения к БД."
    echo -e "  -d\t\tОстановить тестовую БД."
    echo -e "  -c\t\tНе останавлить БД после тестов и не удалять тестовых пользователей."
    echo -e "  -r\t\tУдалить тестовых пользователей."
    echo -e "  -u\t\tСоздать тестовых пользователей."
    echo -e "  -v ЧИСЛО\tУказать версию Postgres."
    echo -e "  -j ПУТЬ\tПуть до кастомной инсталляции Postgres."
    echo -e "  -e СПИСОК\tИсключить тесты. Список имён тестов через запятую." \
        "\n\t\tИмя теста - имя файла теста (например, test_object_function)."
    echo -e "  -h\t\tОтобразить данную справку и выйти."

    #"-m is only for 9.2"
    echo ""
    pg_lsclusters
}

print_tests()
{
    local n=1

    echo "Номера тестовых процедур:"

    for t in ${tests[@]}; do
        test_name=$(${t}_description)
        # Сделать первую букву заглавной
        test_name=${test_name^}
        indent=""

        [ ${n} -gt 9 ] || indent=" "

        echo -e "\t$(echo_info ${indent}${n}): ${test_name}"
        ((n++))
    done
}

#
# pg_createcluster ищет initdb по "стандартному" пути
# /usr/lib/postgresql/номер_версии/bin
#
setup_custom_installation()
{
    local pg_dir="${1}"
    local pg_custom_dir="${opt_user_installation_prefix}/${pg_dir}"

    [ -x "${pg_dir}/bin" ] && [ -x "${pg_dir}/lib" ] && return 0

    mkdir -p ${pg_dir}

    ln -s "${pg_custom_dir}/bin" ${pg_dir}
    ln -s "${pg_custom_dir}/lib" ${pg_dir}

    setcap cap_mac_override,cap_mac_admin+p "${pg_custom_dir}/bin/postgres"
}

clear_custom_installation()
{
    local pg_custom_dir="${1}"

    unlink "${pg_custom_dir}/bin"
    unlink "${pg_custom_dir}/lib"
}

postgresql_version()
{
    local ver=

    if [ -z "${opt_user_version}" ]; then
        ver=$(psql --version | cut -d' ' -f 3)
    else
        ver=${opt_user_version}
    fi

    [[ ${ver} =~ ^[0-9]+\.?[0-9]*$ ]] || {
        echo_err "Неверный формат версии Postgres: ${ver}"
        return 1
    }

    echo ${ver}
    return 0
}

major_version()
{
    local ver=${1}

    # Удаляем из номера версии всё кроме точек
    dots=${ver//[^.]}

    [ -n "${dots}" ] && ver=$(echo ${ver} | cut -d'.' -f -${#dots})

    echo ${ver}
}

check_tests()
{
    local pg_ver=${1}

    [ ${pg_ver} -eq 14 ] && {
        if [ -z "${opt_user_exclude_tests}" ]; then
            opt_user_exclude_tests="${exclude_tests}"
        else
            opt_user_exclude_tests="${opt_user_exclude_tests}, ${exclude_tests}"
        fi
    }

    IFS="," read -ra exclude_list <<< "${opt_user_exclude_tests}"

    new_tests=()
    for t in ${tests[@]}
    do
        match=
        for te in ${exclude_list[@]}
        do
            [ "${te}" = "${t}" ] && {
                match=1
                break
            }
        done

        [ -z "${match}" ] && new_tests+=($t)
    done

    tests=("${new_tests[@]}")
    unset new_tests

    return 0
}

while getopts "zgdrshlmkcp:t:w:n:o:f:i:v:j:e:" opt
do
    case $opt in
        f) opt_user_ftrace_output_dir=${OPTARG};;
        w) opt_sleep_sec_before_test=${OPTARG};;
        o) opt_user_gcov_output_dir=${OPTARG};;
        n) opt_user_num_of_test=${OPTARG};;
        m) opt_user_enable_mac_row="true";;
        c) opt_user_do_not_stop_db="true";;
        i) opt_user_hostname=${OPTARG};;
        s) opt_user_setup_only="true";;
        k) opt_user_skip_setup="true";;
        p) opt_user_port=${OPTARG};;
        t) opt_psql_connect_timeout=${OPTARG};;
        g) opt_user_gcov="true";;
        v) opt_user_version=${OPTARG};;
        j) opt_user_installation_prefix=${OPTARG};;
        z) setup_mac_write_up 0; exit 0;;
        r) remove_users; exit 0;;
        u) add_users; exit 0;;
        l) opt_user_print_tests="true";;
        e) opt_user_exclude_tests=${OPTARG};;
        d) pg_stop; exit 0;;
        h) usage; exit 0;;
        *) usage; exit 0;;
    esac
done

DONE="готово"
ERROR="ошибка"
DOESN_T_EXISTS="не существует"
ALREADY_EXISTS="уже существует"

USERADMINDB="elmac_postgresql_tc_admindb"
USERNAME="elmac_postgresql_tc_testermac"
POSTGRESQL_DIR="/opt/postgresql"
DATABASE_DIR=${POSTGRESQL_DIR}/pgsql_${USERADMINDB}

pg_conf_path="${DATABASE_DIR}/postgresql.conf"
pg_hba_path="${DATABASE_DIR}/pg_hba.conf"

ADMIN_MAC_LABEL="'0:0:0'"
USERADMINDB_LOGIN_L0C0_PREFIX="elmacexec -l 0 -c 0 -- sudo -u ${USERADMINDB}"
#USERNAME_LOGIN_L1C280_PREFIX="elmacexec -l 1 -c 280 -- sudo -u ${USERNAME}"

LAST_COMMAND_OUTPUT=""

POSTGRESQL_VERSION=$(postgresql_version) || exit
echo_warn "Версия Postgres ${POSTGRESQL_VERSION}"
POSTGRESQL_VERSION=$(major_version ${POSTGRESQL_VERSION})

check_tests ${POSTGRESQL_VERSION}
[ "${opt_user_print_tests}" = "true" ] && { print_tests; exit 0; }

pg_ver_dir="/usr/lib/postgresql/${POSTGRESQL_VERSION}"
postgres_bin_path="${opt_user_installation_prefix}/${pg_ver_dir}/bin"

export PATH=${postgres_bin_path}:${PATH}

[ -n "${opt_user_installation_prefix}" ] && setup_custom_installation ${pg_ver_dir}

CLUSTER_NAME="elmac_postgresql_tc"
CLUSTER_ETC="/etc/postgresql/${POSTGRESQL_VERSION}"
CLUSTER_ETC_PERM_CACHED=

#Settings data
TEST_DATA_FILE=${TEST_DIR}/data
declare -A TEST_DATA_MATRIX
TEST_DATA_MATRIX_ROW_NUM=
TEST_DATA_MATRIX_COL_NUM=

MAC_WRITE_UP_CACHED=
if [ -e "/sys/fs/elmacfs/mac_write_up" ]; then
    MAC_WRITE_UP_FILE="/sys/fs/elmacfs/mac_write_up"
else
    MAC_WRITE_UP_FILE="/elmac/mac_write_up"
fi

MAC_USERS_FILE=/etc/security/elmac/mac_users

run_command()
{
    local cmd="${1}"
    local failed_str="${2}"
    local success_str="${3}"

    [ -z "${failed_str}" ] && failed_echo_cmd="echo_err ${ERROR}" || failed_echo_cmd="echo ${failed_str}"
    [ "${failed_str}" = "..." ] && failed_echo_cmd=""

    [ -z "${success_str}" ] && success_echo_cmd="echo ${DONE}" || success_echo_cmd=="echo ${success_str}"
    [ "${success_str}" = "..." ] && success_echo_cmd=""

    log "# ${cmd}"

    eval ${cmd} >> ${log_file} 2>&1 && { eval ${success_echo_cmd}; return 0; }

    eval ${failed_echo_cmd}
    return 1
}

#
# Получить номер порта работающего кластера
#
port_num()
{
    echo $(pg_conftool ${POSTGRESQL_VERSION} ${CLUSTER_NAME} show port | cut -d'=' -f 2)
}

isDBServerRemote()
{
    [ "${opt_user_hostname}" = "${default_localhost_name}" ] && return 1

    return 0
}

#
# Checking
#
check_cmd_output()
{
    local expected="${1}"

    expected=$(echo ${expected:1:-1})

    [ "${expected}" = "${LAST_COMMAND_OUTPUT}" ] || {
        log "${ERROR}. Ожидаем ${expected}, получили ${LAST_COMMAND_OUTPUT}"
        #return 1
        return 0
    }

    return 0
}

check_mac_label()
{
    local mac_label=${1}
    mac_label=${mac_label:1:-1}

    if [ ${POSTGRESQL_VERSION} = "9.6" ]; then
        #
        # Не ищем точного совпадения, так как в выводе команды есть
        # "could not change directory to ... Permission denied"
        # Потому что у пользователя нет доступа в директорию запуска теста (elmac)
        #
        echo "${LAST_COMMAND_OUTPUT}" | grep -q ${mac_label} || return 1

        return 0
    else
        #
        # Вывод метки в Postgres 13 (для всех объектов, кроме строк)
        #
        # level | category | attribute
        #-------+----------+-----------
        # 0     | 0        | 3f
        #(1 row)
        #
        #
        # Вывод метки для строк (здесь '...' - все столбцы таблицы,
        # из которой запрашиваются метки строк)
        #
        # ... | level | category | attribute
        #-----+-------+----------+-----------
        # ...   0     | 0        | 3f
        #(1 row)
        #

        found="false"

        while read -r line
        do
            if echo ${line} | grep -q "\-------"; then
                found="true"
            else
                if [ "${found}" = "true" ]; then
                    break;
                fi
            fi
        done < <(echo "${LAST_COMMAND_OUTPUT}")

        # Убираем пробелы
        label=${line// /}

        mac_lev1=$(echo ${label} | rev | cut -d'|' -f 3 | rev)
        mac_cat1=$(echo ${label} | rev | cut -d'|' -f 2 | rev)
        mac_attr1=$(echo ${label} | rev | cut -d'|' -f 1 | rev)

        mac_lev2=$(echo ${mac_label} | cut -d':' -f 1)
        mac_cat2=$(echo ${mac_label} | cut -d':' -f 2)
        mac_attr2=$(echo ${mac_label} | cut -d':' -f 3)

        # Сравниваем метки
        [ ${mac_lev1} -eq ${mac_lev2} -a \
          ${mac_cat1} -eq ${mac_cat2} -a \
         "${mac_attr1}" = "${mac_attr2}" ] || return 1

        return 0
    fi

    return 1
}

check_cmd_output_string()
{
    local expected="${1}"

    [ "${expected}" = "${LAST_COMMAND_OUTPUT}" ] || {
        log "${ERROR}. Ожидаем ${expected}, получили ${LAST_COMMAND_OUTPUT}"
        return 1
    }

    return 0
}

check_cmd_output_contains()
{
    local pattern="${1}"

    echo "${LAST_COMMAND_OUTPUT}" | grep -q "${pattern}" || {
        #echo "!!! Cannot find \"${pattern}\" !!!"
        return 1
    }

    return 0
}

check_cmd_output_contains_count()
{
    local pattern="${1}"

    count=$(echo "${LAST_COMMAND_OUTPUT}" | grep "${pattern}" | wc -l)

    echo ${count}
}

#
# Test's utilities
#
postgres_run_command_admin()
{
    local mac_label=${1}
    #local login="${USERADMINDB_LOGIN_L0C0_PREFIX}"
    local user=${USERADMINDB}
    local db=${2}
    local cmd="${3}"

    local login=
    local mac_level=
    local mac_cat=

    mac_level=$(echo ${mac_label} | cut -d':' -f 1)
    mac_level=$(echo ${mac_level:1})

    mac_cat=$(echo ${mac_label} | cut -d':' -f 2)

    login="elmacexec -l ${mac_level} -c ${mac_cat} -- sudo -u ${user}"

    local connection_string="host=${opt_user_hostname} port=${opt_user_port} dbname=${db} user=${user} connect_timeout=${opt_psql_connect_timeout}"

    c=(${login} psql -d '"'${connection_string}'"' -c "${cmd}")

    log "\n# ${c[@]}"

    # Postgresql query
    LAST_COMMAND_OUTPUT=$(eval "${c[@]}" 2>&1)
    ret=$?

    #
    # Admin cannot fail (It is believed so).
    # But if failure is ok, then one can check return value.
    #
    log_n "${LAST_COMMAND_OUTPUT}"
    if echo "${LAST_COMMAND_OUTPUT}" | grep -wq ERROR; then
        log_n " - пропускаем..."
    fi
    log ""

    return ${ret}
}

postgres_run_file_admin()
{
    local mac_label=${1}
    local user=${USERADMINDB}
    local file="${2}"

    local login=
    local mac_level=
    local mac_cat=

    mac_level=$(echo ${mac_label} | cut -d':' -f 1)
    mac_level=$(echo ${mac_level:1})

    mac_cat=$(echo ${mac_label} | cut -d':' -f 2)

    login="elmacexec -l ${mac_level} -c ${mac_cat} -- sudo -u ${user}"

    local connection_string="host=${opt_user_hostname} port=${opt_user_port} user=${user} connect_timeout=${opt_psql_connect_timeout}"

    c=(${login} psql -d '"'${connection_string}'"' -a -f ${file})

    log "\n# ${c[@]}"

    # Postgresql query
    LAST_COMMAND_OUTPUT=$(eval "${c[@]}" 2>&1)
    ret=$?

    log_n "${LAST_COMMAND_OUTPUT}"
    #if echo "${LAST_COMMAND_OUTPUT}" | grep -wq ERROR; then
    #    log_n " - пропускаем..."
    #fi
    log ""

    return ${ret}
}

postgres_run_command()
{
    local mac_label=${1}
    local user=${USERNAME}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"

    local login=
    local mac_level=
    local mac_cat=

#echo "mac_label=${mac_label}, user=${USERNAME}, db=${db}, expected_ret=${expected_ret}, cmd=\"${cmd}\""

    mac_level=$(echo ${mac_label} | cut -d':' -f 1)
    mac_level=$(echo ${mac_level:1})

    mac_cat=$(echo ${mac_label} | cut -d':' -f 2)

    login="elmacexec -l ${mac_level} -c ${mac_cat} -- sudo -u ${user}"

    local connection_string="host=${opt_user_hostname} port=${opt_user_port} dbname=${db} user=${user} connect_timeout=${opt_psql_connect_timeout}"

    c=(${login} psql -d '"'${connection_string}'"' -c "${cmd}")

    log "\n# ${c[@]}"

    # Postgresql query
    LAST_COMMAND_OUTPUT=$(eval "${c[@]}" 2>&1)

    ret=$?

    log "${LAST_COMMAND_OUTPUT}"

    [ ${ret} -eq ${expected_ret} ] || {
        log "${ERROR}. Операция вернула ${ret}, ожидали ${expected_ret}"

        [ ${ret} -eq 0 ] || {
            [ $(get_mac_write_up_value) = "1" ] || {
                log_stdout "Внимание: запись вверх запрещена."
            }
        }

        return 1
    }

    return 0
}

postgres_run_command_ftrace()
{
    local mac_label=${1}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"

    postgres_run_command ${1} ${2} ${3} "${4}" "ftrace"
}

postgres_run_command_drop()
{
    local mac_label=${1}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"
    local retval=0

    echo -n "drop"

    postgres_run_command ${1} ${2} ${3} "${4}" || retval=1

    echo -ne "\b\b\b\b    \b\b\b\b"

    return ${retval}
}

postgres_run_command_read()
{
    local mac_label=${1}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"
    local retval=0

    echo -n "read"

    postgres_run_command ${1} ${2} ${3} "${4}" || retval=1

    echo -ne "\b\b\b\b    \b\b\b\b"

    return ${retval}
}

postgres_run_command_write()
{
    local mac_label=${1}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"
    local retval=0

    echo -n "write"

    postgres_run_command ${1} ${2} ${3} "${4}" || retval=1

    echo -ne "\b\b\b\b\b     \b\b\b\b\b"

    return ${retval}
}

postgres_run_command_get_output()
{
    postgres_run_command ${1} ${2} ${3} "${4}"

    echo "${LAST_COMMAND_OUTPUT}"
}

postgres_run_command_insert()
{
    local mac_label=${1}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"
    local retval=0

    echo -n "insert"

    postgres_run_command ${1} ${2} ${3} "${4}" || retval=1

    echo -ne "\b\b\b\b\b\b      \b\b\b\b\b\b"

    return ${retval}
}

postgres_run_command_create()
{
    local mac_label=${1}
    local db=${2}
    local expected_ret=${3}
    local cmd="${4}"
    local retval=0

    echo -n "create"

    postgres_run_command ${1} ${2} ${3} "${4}" || retval=1

    echo -ne "\b\b\b\b\b\b      \b\b\b\b\b\b"

    return ${retval}
}

# Check config version
postgresql_check_old_version()
{
    local path_to_conf=${1}

    if [ -z "${path_to_conf}" ]; then
        path_to_conf="${CLUSTER_ETC}/${CLUSTER_NAME}/postgresql.conf"
    fi

    cat "${path_to_conf}" | grep "default_with_oids" 1>/dev/null
}

#
# Postgres start/stop utilities
#
# pg_ctl version
#
pg_ctl_start()
{
    local initdb_options=""
    local server_logfile=${DATABASE_DIR}/logfile
    local pg_ctl_opts="-D ${DATABASE_DIR} -l ${server_logfile}"
    local server_pidfile=${DATABASE_DIR}/postmaster.pid
    local max_tries=5
    local n=0

    [ -z "${opt_user_port}" ] || initdb_options+="${initdb_options} -p ${opt_user_port}"

    if ${USERADMINDB_LOGIN_L0C0_PREFIX} which pg_ctl > /dev/null; then
        cmd="${USERADMINDB_LOGIN_L0C0_PREFIX} pg_ctl ${pg_ctl_opts} -o '${initdb_options}' start"
    else
        cmd="${USERADMINDB_LOGIN_L0C0_PREFIX} ${postgres_bin_path}/pg_ctl ${pg_ctl_opts} -o '${initdb_options}' start"
    fi

    run_command "${cmd}" || return 1

    #
    # Wait server to start
    #
    echo -n "Ожидаем старт кластера Postgres... 1"
    while [ ! -e ${server_pidfile} ];
    do
        [ ${n} -lt ${max_tries} ] || {
            echo -e "\nНе удалось запустить кластер Postgres."
            echo -e "Содержимое лог файла кластера ${server_logfile}:\n"
            cat ${server_logfile}
            return 1
        }

        ((n++))

        echo -en "\bок"
        sleep 1
    done

    [ -n "${opt_user_port}" ] || {
        opt_user_port=$(sed -n '4p' < ${server_pidfile})
        log_stdout "\nНомер порта: ${opt_user_port}"
    }

    return 0
}

pg_ctl_stop()
{
    local cmd=$(echo "${USERADMINDB_LOGIN_L0C0_PREFIX} pg_ctl -D ${DATABASE_DIR} -l ${DATABASE_DIR}/logfile stop")

    [ -d ${DATABASE_DIR} ] || { echo "${DOESN_T_EXISTS}"; return 0; }

    run_command "${cmd}" "${ERROR}" "..."

    return 0
}

#
# Cluster version
#
pg_createcluster_start()
{
    local cmd=
    local pg_ctl_opts="-d ${DATABASE_DIR} -l ${DATABASE_DIR}/logfile"

    mkdir -p ${CLUSTER_ETC}
    CLUSTER_ETC_PERM_CACHED=`stat -c "%a %n" ${CLUSTER_ETC} | cut -d' ' -f 1`

    chmod a+w ${CLUSTER_ETC}

    [ -z "${opt_user_port}" ] || pg_ctl_opts="${pg_ctl_opts} -p ${opt_user_port}"

    cmd=$(echo "${USERADMINDB_LOGIN_L0C0_PREFIX} ${PG_CREATECLUSTER_BIN} ${pg_ctl_opts} --start ${POSTGRESQL_VERSION} ${CLUSTER_NAME}")

    log_n_stdout "Стартуем кластер... "

    run_command "${cmd}" || return 1

    opt_user_port=$(port_num)
    [ -n "${opt_user_port}" ] || {
        echo_err "Ошибка получения номера порта"
        return 1
    }

    return 0
}

pg_cluster_stop()
{
    [ -z "${CLUSTER_ETC_PERM_CACHED}" ] || chmod ${CLUSTER_ETC_PERM_CACHED} ${CLUSTER_ETC}

    local cmd=$(echo "${PG_LSCLUSTERS} | grep ${POSTGRESQL_VERSION} | grep -qw ${CLUSTER_NAME}")

    #Кластер не существует - нечего удалять
    run_command "${cmd}" "..." "..." || return 0

    local cmd=$(echo "${PG_DROPCLUSTER_BIN} --stop ${POSTGRESQL_VERSION} ${CLUSTER_NAME}")

    run_command "${cmd}" "..." "..." || return 1

    return 0
}

#
# General postgres launcher
#
pg_start()
{
    if [ "${IS_POSTGRESQL_9_2}" = "false" ]; then
        pg_createcluster_start || return 1
    else
        pg_ctl_start || return 1
    fi

    return 0
}

#
# Quit database smoothly
#
pg_stop()
{
    log_n_stdout "Удаляем кластер... "

    if [ "${IS_POSTGRESQL_9_2}" = "false" ]; then
        pg_cluster_stop || {
            # Ещё одна попытка
            rm -rf ${DATABASE_DIR}

            pg_cluster_stop || { echo "${ERROR}"; return 1; }
        }
    else
        pg_ctl_stop || return 1
    fi

    rm -rf ${DATABASE_DIR}

    echo "${DONE}"

    return 0
}

#
# Setup database
#
prepare_db()
{
#    pg_stop

    #rm -rf ${POSTGRESQL_DIR}
    mkdir -p ${POSTGRESQL_DIR}
    chmod 0777 ${POSTGRESQL_DIR}
    chmod 0777 /run/postgresql

    # Could not create lock file "/var/run/postgresql/.s.PGSQL.5432.lock"
    #TODO: Does it have to be performed on e2k & sparc?
    #TODO: remember permissions to restore after test
    mkdir -p /var/run/postgresql
    chmod a+w /var/run/postgresql

    #${USERADMINDB_LOGIN_L0C0_PREFIX} mkdir -p ${DATABASE_DIR}

    if ! ${USERADMINDB_LOGIN_L0C0_PREFIX} which initdb > /dev/null; then
        log_n_stdout "Создаём кластер БД в каталоге ${DATABASE_DIR}... "

        run_command "${USERADMINDB_LOGIN_L0C0_PREFIX} ${postgres_bin_path}/initdb -D ${DATABASE_DIR}" || return 1
    else
        echo "# ${USERADMINDB_LOGIN_L0C0_PREFIX} initdb -D ${DATABASE_DIR}"
        ${USERADMINDB_LOGIN_L0C0_PREFIX} initdb -D ${DATABASE_DIR} 2>/dev/null || return 1
    fi

    # Setup configuration
    echo "shared_preload_libraries = '\$libdir/macpgsql.so,\$libdir/pgaudit.so'" >> ${pg_conf_path}
    postgresql_check_old_version "${pg_conf_path}" && echo "default_with_oids = on" >> ${pg_conf_path}
    echo "elmac_ignore_socket_label = on" >> ${pg_conf_path}

    if [ "${IS_POSTGRESQL_9_2}" = "true" ]; then
        if [ ${opt_user_enable_mac_row} = "true" ]; then
            echo -e "*** opt_user_enable_mac_row = on ***\n"
            echo "opt_user_enable_mac_row = on" >> ${pg_conf_path}
        else
            echo -e "*** opt_user_enable_mac_row = off ***\n"
            echo "opt_user_enable_mac_row = off" >> ${pg_conf_path}
        fi
    fi

    # Доступ к серверу с любого интерфейса
    echo -e "local all all peer\nhost all all 0.0.0.0/0 trust\nhost all all ::1/128 trust" > ${pg_hba_path}
    echo "listen_addresses = '*'" >> ${pg_conf_path}

    pg_start || return 1

    return 0
}

remove_user()
{
    local user=${1}

    log_n_stdout "Удаляем пользователя \"${user}\"... "

    if [ -n "$(which est-dir-admin)" ]; then
        cmd="est-dir-admin -a del -b local -t user --name ${user}"

        run_command "${cmd}" "${DOESN_T_EXISTS}"
    else
        cmd="userdel ${user}"

        run_command "${cmd}" "${DOESN_T_EXISTS}"

        sed -i "/\<${user}\>/d" ${MAC_USERS_FILE}
    fi
}

add_user()
{
    local user=${1}
    local caps=0x300000000

    local min_level=0
    local max_level=255
    local min_cat=0x0
    local max_cat=0xfff

    remove_user ${user}

    log_n_stdout "Добавляем пользователя \"${user}\"... "

    if [ -n "$(which est-dir-admin 2>/dev/null)" ]; then
        cmd="est-dir-admin -b local -t user -a add --name ${user} --pas 123 --caps=${caps} --min_l ${min_level} --max_l ${max_level} --min_c ${min_cat} --max_c ${max_cat}"

        run_command "${cmd}" || {
            cat /etc/passwd | grep -q ${user} && echo "${ALREADY_EXISTS}" || { echo_err "${ERROR}"; return 1; }
        }
    else
        cmd="useradd ${user}"

        run_command "${cmd}" || {
            cat /etc/passwd | grep -q ${user} && echo "${ALREADY_EXISTS}" || { echo_err "${ERROR}"; return 1; }
        }

        echo "${user}:${min_level}:${min_cat}:${max_level}:${max_cat}:${caps}" >> ${MAC_USERS_FILE} || {
            log_stdout "Ошибка добавления пользователя ${user} в файл ${MAC_USERS_FILE}"
            return 1
        }
    fi

    cat ${MAC_USERS_FILE} | grep -q ${user} || {
        log_stdout "Описание пользователя ${user} отсутствует в файле ${MAC_USERS_FILE}. Ошибка"
        return 1
    }

    return 0
}

add_users()
{
    [ -e ${MAC_USERS_FILE} ] || {
        log "Отсутсвует файл ${MAC_USERS_FILE}. Ошибка"
        return 1
    }

    pg_stop
    add_user ${USERADMINDB} || return 1
    add_user ${USERNAME} || return 1
}

remove_users()
{
    remove_user ${USERNAME}
    remove_user ${USERADMINDB}
}

ctrl_c()
{
    echo -e "\n\nВыполнение прервано пользователем"

    restore_mac_write_up

    remove_users

    exit 0
}

#
# Setup for test
#
setup()
{
    trap ctrl_c INT

    isDBServerRemote && {
        log_stdout_warn "Тест будет выполняться на удалённом кластере БД ${opt_user_hostname}:${opt_user_port}"

        opt_user_skip_setup="true"
        opt_user_do_not_stop_db="true"

        add_users
    }

    [ "${opt_user_skip_setup}" = "false" ] || {
        isDBServerRemote || log_stdout_warn "Пропуск настройки и старта кластера Postgres"

        [ -n "${opt_user_port}" ] || {
            opt_user_port=$(port_num)

            [[ ${opt_user_port} =~ ^[0-9]+$ ]] || {
                log_stdout_err "Неверное значение номера порта \"${opt_user_port}\"."
                echo "Укажите номер порта через опцию \"-p\"."
                return 1
            }
        }

        setup_mac_write_up

        return 0
    }

    clear_log

    [ "${opt_user_setup_only}" = "false" ] || log_stdout_warn "Только настройка и старт кластера Postgres"

    log "# $@"

    add_users || return 1

    setup_mac_write_up

    prepare_db || {
        [ -e "${DATABASE_DIR}/logfile}" ] && tail ${DATABASE_DIR}/logfile
        return 1
    }

    postgres_run_command_admin "${ADMIN_MAC_LABEL}" postgres '"create user '${USERNAME}' createdb;"' || return 1

    [ "${opt_user_setup_only}" = "false" ] || {
        restore_mac_write_up
        echo_warn "Номер порта: $(port_num)"
        exit 0
    }
}

print_results()
{
    local i=0
    local results=("$@")

    log "\nРезультаты:"

    for t in ${tests[@]}; do
        local test_name=`echo ${t} | cut -d'_' -f2-`

        log "\tТест #${i}. ${test_name}: ${results[$i]}"

        ((i++))
    done

    echo ""
}

#
# Load test's data from file
#
load_test_data()
{
    local file=${TEST_DATA_FILE}
    local i=-1
    local j=0
    local commentLineSeqNum=0

    test -e ${file} || { echo_err "Файл с параметрами тестов \"${file}\" не существует"; return 1; }

    while IFS='' read -r line || [[ -n "$line" ]]; do
        ! echo ${line} | grep -q "^#" || {
            [ ${commentLineSeqNum} -ne 0 ] || {
                if [ $i -eq -1 ]; then
                    i=0
                else
                    TEST_DATA_MATRIX_COL_NUM[$i]=$j
                    TEST_DATA_MATRIX_ROW_NUM=`expr $i + 1`
                    #echo "---- ${TEST_DATA_MATRIX_COL_NUM[$i]}, $j"
                    ((i++))
                    j=0
                fi

            }

            commentLineSeqNum=1
            continue
        }

        commentLineSeqNum=0
        #echo "Text read from file: $line, i=$i, j=$j"

        TEST_DATA_MATRIX[$i,$j]=${line}
        ((j++))
    done < ${file}

    TEST_DATA_MATRIX_COL_NUM[$i]=$j
    TEST_DATA_MATRIX_ROW_NUM=`expr $i + 1`

    if false; then
    #if true; then
        echo "Length: ${#TEST_DATA_MATRIX[@]}, TEST_DATA_MATRIX_ROW_NUM=${TEST_DATA_MATRIX_ROW_NUM}"

        for ((i=0; i<${TEST_DATA_MATRIX_ROW_NUM}; i++)) do
            echo [${TEST_DATA_MATRIX_COL_NUM[$i]}]
            for ((j=0; j<${TEST_DATA_MATRIX_COL_NUM[$i]}; j++)) do
                echo "$i,$j: ${TEST_DATA_MATRIX[$i,$j]}"
            done
        done
    fi

    return 0
}

#
# Tests launcher
#
postgresql_run_test()
{
    local n=-1
    local results=
    local exit_code=0
    declare -A TEST_DATA

    echo -e "\n\t\t\t\tТЕСТЫ"

    for t in ${tests[@]}; do

        ((n++))

        [ -z "${opt_user_num_of_test}" ] || {
            [ $(expr ${n} + 1) = ${opt_user_num_of_test} ] || {
                results[$n]="-"
                continue
            }
        }

        unset TEST_DATA

        for ((j=0;j<${TEST_DATA_MATRIX_COL_NUM[$n]};j++)) do
            #echo "$n: ${TEST_DATA_MATRIX[$n,$j]}"
            log "${TEST_DATA_MATRIX[$n,$j]}"
            TEST_DATA[$j]="${TEST_DATA_MATRIX[$n,$j]}"
        done

        [ ${opt_sleep_sec_before_test} -eq 0 ] || {
            log_n_stdout "Отложить вызов теста на ${opt_sleep_sec_before_test} секунд"
            sleep ${opt_sleep_sec_before_test}
            echo -ne "\033[1K"
        }

        test_name=$(${t}_description)
        [ $? -ne 0 ] && test_err="true"
        # Сделать первую букву заглавной
        test_name=${test_name^}

        # Выравнивание числа и одной цифрой и с двумя.
        # n считается от нуля, поэтому после 8 дополнительный пробел не ставим.
        [ ${n} -lt 9 ] && indent=" " || indent=""

        log_n_stdout "\t${indent}$(echo_info_n $(expr ${n} + 1)): Тест мандатного разграничения на объект \"${test_name}\"... "

        test_start_time=$(date +%s)

        # Запуск теста
        if [ "${test_err}" != "true" ] && $t ${TEST_DATA[*]}; then
            echo_info "ок"
            results[$n]="OK"
        else
            echo_err "${ERROR}"
            results[$n]="FAIL"
            exit_code=1
        fi

        test_stop_start=$(date +%s)

        #log_stdout "Заняло $(expr ${test_stop_start} - ${test_start_time}) секунд"

        #echo "[${results[@]}]"
    done

    print_results "${results[@]}"

    return ${exit_code}
}

get_mac_write_up_value()
{
    echo "$(cat ${MAC_WRITE_UP_FILE})"
}

#
# Если аргумент не передан, то восстанавливаем
# mac_write_up из кэша.
#
setup_mac_write_up()
{
    local updown=${1}

    [ ! -z ${updown} ] || {
        [ ! -e ${MAC_WRITE_UP_FILE} ] || {
            MAC_WRITE_UP_CACHED=$(get_mac_write_up_value)
            echo 1 > ${MAC_WRITE_UP_FILE}
        }

        return
    }

    echo ${updown} > ${MAC_WRITE_UP_FILE}
}

restore_mac_write_up()
{
    [ ! -e ${MAC_WRITE_UP_FILE} ] || {
        [ -z ${MAC_WRITE_UP_CACHED} ] || {
            echo ${MAC_WRITE_UP_CACHED} > ${MAC_WRITE_UP_FILE}
        }
    }
}

cleanup()
{
    [ "${opt_user_do_not_stop_db}" = "false" ] || return

    pg_stop

    remove_users

    restore_mac_write_up

    [ -n "${opt_user_installation_prefix}" ] && clear_custom_installation ${pg_ver_dir}
}

check_user_opts()
{
    [ -z ${opt_user_port} ] || {
        ! isDBServerRemote && {
            pg_lsclusters | grep ${CLUSTER_NAME} | grep -q ${opt_user_port} || {
                log_stdout_err "Кластер не запущен, либо запущен на порту отличном от ${opt_user_port}"
                pg_lsclusters
                return 1
            }
        }
    }

    [ "${opt_user_gcov}" = "false" ] || {
        [ -n "${opt_user_gcov_output_dir}" ] || {
            log_stdout "GCOV output directory is not specified. Exit"
            return 1
        }
    }

	which run_OSLTC 1>/dev/null || {
        :
		#log_stdout 'You must set PATH first:\nexport PATH=${PATH}:/opt/mcst/check/OSLTC/bin'
		#return 1
	}

    return 0
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # #
#                     Main                            #
# # # # # # # # # # # # # # # # # # # # # # # # # # # #

[ "${failure}" != "true" ] && { check_user_opts || failure="true"; }
[ "${failure}" != "true" ] && { setup || failure="true"; }
[ "${failure}" != "true" ] && { load_test_data || failure="true"; }
[ "${failure}" != "true" ] && { postgresql_run_test || failure="true"; }

cleanup

[ "${failure}" = "true" ] && {
    log_stdout "\nОТКАЗ"

    make_failed_log_file

    exit 1
}

log_stdout "\nУСПЕХ"
exit 0

