#!/bin/bash

set +xv

CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TMP_DIR=${TC_DIR}/tmp
CFG_DIR=${TC_DIR}/cfg
ENV_FILE=${CFG_DIR}/env

tmpf=/tmp/_elmac_tmpf

CWD=$(pwd)

if [ -d /sys/fs/elmacfs ]; then
ELMACFS="/sys/fs/elmacfs"
else
ELMACFS="/elmac"
fi

verbose=0
################################################################################
usage()
{
	echo " usage: $progname [-v]"
	echo
	exit 1
}
OPTIONS="vC"
while getopts $OPTIONS option
do
	case $option in
		v) verbose=1; opt_v="-v";;
		C) clean_flg=1;;
		*) echo "$prog: illegal option -- $option"; usage; exit 1;;
	esac
done
################################################################################
DoEcho()
{
	echo "$progname: === $*"
}
DoEchoVer()
{
	[[ $verbose == 1 ]] && DoEcho $*
}
SigHandler()
{
	DoExit 1
}
DoExit()
{
	[[ ! -z $2 ]] && DoEcho $2
	[[ $1 == 0 ]] && DoEcho "PASS"
	[[ $1 != 0 ]] && DoEcho "FAIL"
	UserDel
	cd $CPWD
	exit $1
}
DoCmd()
{
	local cmd ret
	cmd="$*"
	DoEchoVer "$cmd"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		DoEcho "$cmd - returned $ret"
		cat $tmpf
		DoExit 1
	fi
	[[ $verbose == 1 ]] && cat $tmpf
	return 0
}

UserAdd()
{
	local user
	user=elmac-tester
	id $user > /dev/null 2>/dev/null
	ret=$?
	if [ $ret -ne "0" ]; then
		DoEcho "add user elmac-tester."
		DoCmd useradd  $user
	else
		DoEcho "elmac-tester exist."
	fi
}

UserDel()
{
	local user
	user=elmac-tester
	id $user > /dev/null 2>/dev/null
	ret=$?
	if [ $ret -eq "0" ]; then
		DoEcho "delete user elmac-tester."
		DoCmd userdel  $user
	else
		DoEcho "elmac-tester not exist."
	fi
}


trap "SigHandler" "SIGINT"

export ROOT_E90=$CWD
[[ $verbose == 1 ]] && DoEcho ROOT_E90=$ROOT_E90

if  [[ ! -f ${ELMACFS}/mac_write_up ]]; then
	DoEcho "Elmac security subsystem disable.)"
	DoExit 1
fi


DoCmd rm -rf $TMP_DIR
DoCmd mkdir -p -m 0777 $TMP_DIR

DoEcho "Check elmac-tester"
UserAdd

DoEcho "./elmac"
$CWD/elmac  > ${TMP_DIR}/elmac_ret 2>&1
ret=$?
if [[ $ret != 0 ]]; then
	DoEcho "./elmac - returned $ret"
	if [[ $verbose == 1 ]]; then
		cat ${TMP_DIR}/elmac_ret
	else
		DoEcho "RESULTS - ${TMP_DIR}/elmac_ret"
	fi
	DoExit 1
fi

wrup=`cat ${ELMACFS}/mac_write_up`
ret=$?
if [[ $ret != 0 ]]; then
	DoExit 1
else
	DoEcho "./elmac - returned $ret"
	if [[ $wrup == 0 ]]; then
		DoEcho "Elmac mode: disable wr_up ($wrup)"
		elmac_ret_etalon=elmac_ret_etalon_wrup_disable
	else
		DoEcho "Elmac mode: enable  wr_up ($wrup)"
		elmac_ret_etalon=elmac_ret_etalon_wrup_enable
	fi
fi

hfiles=""
hf=`cat ${ELMACFS}/hfiles 2>/dev/null`
ret=$?
if [[ $ret == 0 ]]; then
	if [[ $hf != 0 ]]; then
		hfiles="_hf_present"
	fi
fi
elmac_ret_etalon=${elmac_ret_etalon}${hfiles}


DoCmd diff ${TMP_DIR}/elmac_ret $CWD/${elmac_ret_etalon}

DoExit 0
