#include <sys/time.h>
#include <time.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/wait.h>
#include <pwd.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <setjmp.h>
#include <string.h>

#include <sys/types.h>
#include <attr/xattr.h>
#include <sys/stat.h>
#include <limits.h>

#define _XOPEN_SOURCE 700
#define __USE_XOPEN_EXTENDED 1
#include <ftw.h>

#define FAKELIB
#ifdef FAKELIB
#include "elmac.h"
#else
#include <linux/elmac.h>
#endif

#ifndef __u8
#define __u8    unsigned char
#endif /* __8 */

#ifndef __u32
#define __u32   unsigned int
#endif /* __u32 */

#ifndef __u64
#define __u64   unsigned long long
#endif /* __u32 */

#define ELAMC_LEVELS_LIMIT  	"/sys/fs/elmacfs/mac_lev_limit"
#define ELAMC_CATEGORIES_LIMIT	"/sys/fs/elmacfs/mac_cat_limit"
#define ELAMC_CONTAINERS        "/sys/fs/elmacfs/containers"

#define MIN_LEVEL		0
#define MAX_LEVEL		1
#define MIN_CAT			0
#define MAX_CAT			0x3
#define CAT_NUM_TEST_MAX	256
#define TEST_DIR		"/elmac-test-dir"
#define USER_MAC_TESTER 	"elmac-tester"

#define	MSGMAX	1024
#define	STRSIZE	1024
#define	SHMSIZE	128

#define	TMP_BUF	(PATH_MAX + 512)
char *input_result;
char input_result_buf[TMP_BUF];

typedef __u8 elmac_lev_t;
typedef __u64 elmac_cat_t;
typedef __u32 elmac_attr_t;
typedef __u64 elmac_max_t;


elmac_lev_t lev_min;
elmac_lev_t lev_max;
elmac_cat_t cat_set;

int	p_print_f = 0;
int	p_print_shm = 0;
int	p_print_sem = 0;
int	p_print_msg = 0;

__uid_t tester_uid;	/* User ID.  */
__gid_t tester_gid;	/* Group ID.  */

int		cat_num;

key_t		i_shm_create;
key_t		i_msg_create;
key_t		i_sem_create;

int		*ipc_mem_id_array;
unsigned char 	*ipc_mem_index_cat_array_mem;
elmac_cat_t 	*ipc_mem_index_cat_array;
int		ipc_mem_id_array_size;

struct	msqbuf
{
	long	mtype;
	char	mtext[MSGMAX];
};

struct	msqbuf	msg_snd = {1, "m"};
struct	msqbuf	msg_rcv = {0, "m"};


void print_d(int p, const char *format, ...)
{
	va_list	ap;
	char s[1024];
	if (p) {
		va_start(ap, format);
		if (strlen(format) > 1020)
			(void) fprintf(stderr,
				"print_d: strlen(format) > 1020\n");
		else {
			sprintf(s, "%s", format);
			(void) vfprintf(stderr, s, ap);
		}
		va_end(ap);
	}
}

#ifdef FAKELIB
/**
 * From lib. No correct!!!
 */

#define ELMAC_LE 0
#define ELMAC_BE 1
#define SELF_LABEL_FILE "/proc/self/attr/current"

/*
 * elmac_get_endian - Get system endianness
 */
int elmac_get_endian(void)
{
	int ret;

	union {
		int i;
		char c[sizeof(int)];
	} foo;

	foo.i = 1;

	if (foo.c[0] == 1) {
		ret = ELMAC_LE;
	} else {
		ret = ELMAC_BE;
	}
	return ret;
}

/*
 * le64 - Convert endianness.
 */
#define CONVERT_MAX_SIZE	8
void le64(char *val, ssize_t len)
{
	char buff[CONVERT_MAX_SIZE] = {0};
	int i;
	ssize_t save_len;

	save_len = len;

	i = 0;
	while (len --) {
		buff[i] = val[len];
		i ++;
	}

	memcpy(val, buff, save_len);
}

/*
 * LE_LABEL - Convert label.
 */
void LE_LABEL(elmac_label_t *label)
{
	if (elmac_get_endian() == ELMAC_LE)
		return;

	if (label == NULL)
		return;

	le64((char *)&label->category, sizeof(__u64));
}

/*
 * LE_CONTEXT - Convert context.
 */
void LE_CONTEXT(elmac_context_t *context)
{
	if (elmac_get_endian() == ELMAC_LE)
		return;

	if (context == NULL)
		return;

	le64((char *)&context->attr, sizeof(__u32));
	le64((char *)&context->mac.category, sizeof(__u64));
}

/*
 * elmac_get_label_from_self -
 */

ssize_t elmac_get_label_from_self(elmac_label_t *label)
{
	elmac_context_t dummy;
	size_t len;
	int fd;
	ssize_t ret = 0;

	if (label == NULL)
		return (ssize_t)(-1);

	len = sizeof(elmac_context_t);

	fd = open(SELF_LABEL_FILE, O_RDONLY);
	if (fd < 0) {
		return (ssize_t)(-1);
	}

	ret = read(fd, &dummy, len);

	close(fd);
	if (ret < 0) {
		return ret;
	}

	LE_CONTEXT(&dummy);

	len = sizeof(elmac_label_t);
	memcpy(label, &dummy.mac, len);

	return ret;
}

/*
 * elmac_set_context_for_self -
 */
int elmac_set_context_for_self(elmac_context_t *context)
{
	size_t len;
	int fd;
	ssize_t ret;

	if (context == NULL)
		return (int)(-1);

	len = sizeof(elmac_context_t);

	fd = open(SELF_LABEL_FILE, O_WRONLY);
	if (fd < 0) {
		return (int)(-1);
        }

	LE_CONTEXT(context);

	ret = write(fd, context, len);

	close(fd);

	return (ret < 0) ? -1 : 0;
}

/*
 * elmac_set_label_for_self -
 */
int elmac_set_label_for_self(elmac_label_t *label)
{
	elmac_context_t *dummy;
	size_t len;
	int ret;

	if (label == NULL)
		return (int) (-1);

	len = sizeof(elmac_context_t);

	dummy = calloc(1, len);
	if (dummy == NULL)
		return (int)(-ENOMEM);

	memcpy(&dummy->mac, label, sizeof(elmac_label_t));

	ret = elmac_set_context_for_self(dummy);

	if (dummy)
		free(dummy);

	return (ret < 0) ? -1 : 0;
}

/*
 * elmac_get_context_from_path -
 */
ssize_t elmac_get_context_from_path(const char *path, const char *xattr,
				    int follow, elmac_context_t *context)
{
	size_t len;
	ssize_t ret = 0;
	errno = 0;

	if (context == NULL) {
		errno = EINVAL;
		return (ssize_t)(-1);
	}

	len = sizeof(elmac_context_t);

	ret = follow ?
		getxattr(path, xattr, context, len) :
		lgetxattr(path, xattr, context, len);

	LE_CONTEXT(context);

	return ret;
}

/*
 * elmac_set_context_for_path -
 */
int elmac_set_context_for_path(const char *path, const char *xattr, int follow,
			       elmac_context_t *context)
{
	size_t len;
	int ret;

	if (context == NULL)
		return (int)(-1);

	len = sizeof(elmac_context_t);

	LE_CONTEXT(context);

	ret = follow ?
		setxattr(path, xattr, context, len, 0) :
		lsetxattr(path, xattr, context, len, 0);

	return ret;
}

/*
 * elmac_get_lev_limit -
 */
int elmac_get_lev_limit(__u8 *lev_limit)
{
	int fd;
	unsigned lev_limit_raw = 0;
	ssize_t ret = 0;
	char buf[80];

	memset(&buf, 0, sizeof(buf));
	fd = open(ELAMC_LEVELS_LIMIT, O_RDONLY);
	if (fd < 0) {
		return -1;
	}

	ret = read(fd, buf, sizeof(buf));

	close(fd);
	if (ret < 0) {
		return -1;
	}

    errno = 0;
    lev_limit_raw = strtoul(buf, NULL, 10);
    if (errno != 0) {
        return -1;
    }

    if (lev_limit_raw > ELMAC_MAX_LEVEL)
        return -1;

    *lev_limit = (__u8) lev_limit_raw;

	return 0;
}

/*
 * elmac_get_cat_limit -
 */
int elmac_get_cat_limit(__u64 *cat_limit)
{
	int fd;
	ssize_t ret = 0;
	char buf[80];

	memset(&buf, 0, sizeof(buf));
	fd = open(ELAMC_CATEGORIES_LIMIT, O_RDONLY);
	if (fd < 0) {
		return -1;
	}

	ret = read(fd, buf, sizeof(buf));

	close(fd);
	if (ret < 0) {
		return -1;
	}
    errno = 0;
    *cat_limit = (__u64) strtoull(buf, NULL, 16);
    if (errno != 0) {
        return -1;
    }

	return 0;
}

/*
 * elmac_get_containers_mode -
 */
int elmac_get_containers_mode()
{
	int fd;
	ssize_t ret = 0;
	char buf[80];

	memset(&buf, 0, sizeof(buf));
	fd = open(ELAMC_CONTAINERS, O_RDONLY);
	if (fd < 0) {
		return 0;
	}

	ret = read(fd, buf, sizeof(buf));

	close(fd);
	if (ret < 0) {
		return -1;
	}

    return (atoi(buf) == 1) ? 1 : 0;
}
#endif

/**
 * FS objects
 */

static int rm_files(const char *pathname, const struct stat *sbuf,
                    int type, struct FTW *ftwb)
{
    if(remove(pathname) < 0)
    {
		(void) fprintf( stderr,"ELMAC: prepare_test_dir (fail). "
				"Could't remove (%s): %s.\n",
				pathname, strerror(errno));
        return -1;
    }
    return 0;
}

int rm_testdir()
{
	if ((access(TEST_DIR, F_OK)) == 0) {
	    print_d(p_print_f, "ELMAC: creat_df access(f_OK) == 0\n");
        if (nftw(TEST_DIR, rm_files, 10, FTW_DEPTH|FTW_MOUNT|FTW_PHYS) < 0) {
            return (1);
        }
    }

    if (access(TEST_DIR, F_OK) == 0) {
        if (rmdir(TEST_DIR)) {
            return (1);
        }
    }

    return (0);
}


int prepare_test_dir()
{
	elmac_context_t context;

    if (rm_testdir()) {
        return (1);
    }

	if ((mkdir(TEST_DIR, 0)) < 0) {
		(void) fprintf( stderr,"ELMAC: creat_df (fail). "
				"Could't mkdir(%s): %s.\n",
				TEST_DIR, strerror(errno));
		return (1);
	}
	print_d(p_print_f, "ELMAC: creat_df mkdir %s.\n",
		TEST_DIR);

	if ((chmod(TEST_DIR, S_IRWXU | S_IRWXG | S_IRWXO)) < 0) {
		(void) fprintf( stderr, "ELMAC: creat_df (fail). "
				"Culd't chmod(%s): %s.\n",
				TEST_DIR, strerror(errno));
		return (1);
	}
	print_d(p_print_f, "ELMAC: creat_df chmod "
		"(S_IRWXU | S_IRWXG | S_IRWXO) %s.\n", 	TEST_DIR);

   if (elmac_get_containers_mode() == 1) {
        if (elmac_get_context_from_path("/", XATTR_NAME_ELMAC, 1, &context)
            != sizeof(context)) {
            (void) fprintf(stderr, "ELMAC: creat_df (fail). "
                           "Culd't get ELMAC label to path(%s).\n", "/");
		        return (1);
	        }
        if ((context.mac.level < lev_max)
                || ((context.mac.category & cat_set) != cat_set)
	            || ((context.attr & ELMAC_ATTR_IGNORE) != ELMAC_ATTR_IGNORE)) {
		        (void) fprintf(stderr, "ELMAC: creat_df (fail). "
				               "Inccorect ELMAC label(%d, %llx) to path(%s)."
				               "Level < Level test or Cat < Cat_test.\n",
				               context.mac.level, context.mac.category, "/");
        }
	    context.attr = ELMAC_ATTR_IGNORE;
        if (elmac_set_context_for_path(TEST_DIR, XATTR_NAME_ELMAC, 1, &context)) {
		    (void) fprintf(stderr, "ELMAC: creat_df (fail). "
			               "Culd't set ELMAC label(%d, %llx) to path(%s).\n",
			               context.mac.level, context.mac.category, TEST_DIR);
		    return (1);
        }
	}
    return 0;
}

int creat_df(elmac_lev_t lev, elmac_cat_t cat, elmac_attr_t attr)
{
	char name_df[PATH_MAX];
	char path_buff[PATH_MAX];
	elmac_context_t context;
	int rc;

	memset(name_df, 0, PATH_MAX);
	memset(path_buff, 0, PATH_MAX);

#if 0
	if ((access(TEST_DIR, F_OK)) != 0) {
		print_d(p_print_f, "ELMAC: creat_df access(f_OK) != 0\n");
		if ((mkdir(TEST_DIR, 0)) < 0) {
			(void) fprintf( stderr,"ELMAC: creat_df (fail). "
					"Could't mkdir(%s): %s.\n",
					TEST_DIR, strerror(errno));
			return (1);
		}
		print_d(p_print_f, "ELMAC: creat_df mkdir %s.\n",
			TEST_DIR);

		if ((chmod(TEST_DIR, S_IRWXU | S_IRWXG | S_IRWXO)) < 0) {
			(void) fprintf( stderr, "ELMAC: creat_df (fail). "
					"Culd't chmod(%s): %s.\n",
					TEST_DIR, strerror(errno));
			return (1);
		}
		print_d(p_print_f, "ELMAC: creat_df chmod "
			"(S_IRWXU | S_IRWXG | S_IRWXO) %s.\n",
			TEST_DIR);

	}
#endif

	strcpy(name_df, TEST_DIR);
	strcat(name_df, "/");
	snprintf(path_buff, PATH_MAX, "%d.%llx.%d", lev, cat, attr);
	strcat(name_df, path_buff);

	if ((rc = open(name_df, O_WRONLY | O_CREAT | O_TRUNC, 0)) < 0) {
		(void) fprintf( stderr, "ELMAC: creat_df (fail). "
				"Open(%s): %s.\n",
				name_df, strerror(errno));
		return (1);
	}
	print_d(p_print_f, "ELMAC: creat_df open "
		"(O_WRONLY | O_CREAT | O_TRUNC) %s.\n",
		name_df);

	errno = 0;
	close(rc);

	print_d(p_print_f, "ELMAC: creat_df open %s; close %s.\n",
		name_df, strerror(errno));

	if ((access(name_df, F_OK)) != 0) {
		(void) fprintf( stderr,"ELMAC: creat_df (fail). "
				"Could't access(%s): %s.\n",
				name_df, strerror(errno));
		return (1);
	}
#if 0
	if ((chown(name_df, tester_uid, tester_gid)) < 0) {
		(void) fprintf( stderr, "ELMAC: creat_df (fail). "
				"Culd't chown(%s): %s.\n",
				name_df, strerror(errno));
		return (1);
	}
	print_d(p_print_f, "ELMAC: creat_df chown "
		"(%s) %s.\n",
		USER_MAC_TESTER, name_df);
#endif
	if ((chmod(name_df, S_IRWXU | S_IRWXG | S_IRWXO)) < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_df (fail). "
				"Culd't chmod(%s): %s\n",
				name_df, strerror(errno));
		return (1);
	}
	print_d(p_print_f, "ELMAC: creat_df chmod "
		"(S_IRWXU | S_IRWXG | S_IRWXO) %s.\n",
		name_df);

	/*
	 * Create label
	 */

	context.mac.level = lev;
	context.mac.category = cat;
	context.attr = attr;

	if (elmac_set_context_for_path(name_df, XATTR_NAME_ELMAC, 1, &context)) {
		(void) fprintf( stderr,
				"ELMAC: creat_df (fail). "
				"Culd't set ELMAC label(%d,%llx) to path(%s).\n",
				lev, cat, name_df);
		return (1);
	}

	return (0);
}


int creat_file_obj()
{
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	int		rc = 0;


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {
				if (creat_df(l, c, 0)) {
					rc = 1;
					goto creat_file_obj_err;
				}
			}
		}
	}

	creat_file_obj_err:
	return (rc);
}


void access_ret(char *path_file_mode, char *p_ret_d)
{
    *(p_ret_d + 0) = 'e';
    *(p_ret_d + 1) = 'e';
    *(p_ret_d + 2) = 'e';
	//if (access(path_file_mode, F_OK) == 0) {
	//	print_d(p_print_f || p_print_f,
	//		"ELMAC: %s F_OK  YES\n", path_file_mode);
	    errno = 0;
		if (access(path_file_mode, R_OK) == 0) {
			print_d(p_print_f || p_print_f,
				"ELMAC: %s R_OK YES\n", path_file_mode);
			*(p_ret_d + 0) = 'r';
		} else {
			if (errno == ENOENT) {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s R_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 0) = 'n';
			} else if (errno == EACCES) {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s R_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 0) = 'p';
			} else {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s R_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 0) = 'e';
			}
		}
		if (access(path_file_mode, W_OK) == 0) {
			print_d(p_print_f || p_print_f,
				"ELMAC: %s W_OK YES\n", path_file_mode);
			*(p_ret_d + 1) = 'w';
		} else {
			if (errno == ENOENT) {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s W_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 1) = 'n';
			} else if (errno == EACCES) {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s W_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 1) = 'p';
			} else {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s R_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 1) = 'e';
			}
		}
		if (access(path_file_mode, X_OK) == 0) {
			print_d(p_print_f || p_print_f,
				"ELMAC: %s X_OK YES\n", path_file_mode);
			*(p_ret_d + 2) = 'x';
		} else {
			if (errno == ENOENT) {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s X_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 2) = 'n';
			} else if (errno == EACCES) {
				print_d(p_print_f || p_print_f,
					"ELMAC: %sX_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 2) = 'p';
			} else {
				print_d(p_print_f || p_print_f,
					"ELMAC: %s X_OK %s\n",
					path_file_mode, strerror(errno));
				*(p_ret_d + 2) = 'e';
			}
		}
	//} else
	//	print_d(p_print_f || p_print_f,
	//		"ELMAC: %s F_OK %s\n",
	//		path_file_mode, strerror(errno));
}


int check_access_file(elmac_lev_t lev, elmac_cat_t cat, char **buf)
{
	char path_file[PATH_MAX];
	char path_buff[PATH_MAX];
	char *S;
	char *p_ret_d;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;

	p_ret_d = malloc(10);
	if (p_ret_d == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_file "
				"(fail). Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}

	memset(p_ret_d, 0, 10);

	S = malloc(1);
	if (S == (char*)NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_file (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}
	memset(S, 0, 1);

	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {
				size_t len_tmp;

				strcpy(path_file, TEST_DIR);
				strcat(path_file, "/");
				snprintf(path_buff, PATH_MAX, "%d.%llx.%d",
					 l, c, 0);
				strcat(path_file, path_buff);
				access_ret(path_file, p_ret_d);
				//printf("proc (%d.%llx) to file(%s) "
				//	"access: %s\n",
				//	lev, cat, path_file, p_ret_d);
				memset(input_result_buf, 0, TMP_BUF);
				snprintf(input_result_buf, TMP_BUF,
					 "proc (%d.%llx) to file(%s) "
					 "access: %s\n",
					 lev, cat, path_file, p_ret_d);
				len_tmp = strlen(S);
				S = (char *) realloc (S, len_tmp +
						 strlen(input_result_buf) + 1);
				if (S == (char *)NULL) {
					(void) fprintf(stderr,
							"ELMAC: check_access_file (fail). "
							"realloc error.\n");
							return (1);
				}
				strcat (S, input_result_buf);
			}
		}
	}

	*buf = strdup(S);

	free(p_ret_d);
	free(S);

	return (0);
}


int check_access_proc_file()
{
	elmac_label_t label;
	pid_t	pid;
	pid_t	pidwait;
	int	status;
	char *input_result_file;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	size_t		len;
	int		fd[2];
	int		rc;

	input_result_file = malloc(1);
	if (input_result_file == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_proc_file (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
				return 1;
	}
	memset(input_result_file, 0, 1);

	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_f,
					"Proc: level: %d category: %llx\n", l, c);
				switch (pid = fork()) {
				case 0: {
				char *S;

				if (input_result_file == NULL) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_file (fail). "
						"Could not memmory alloc for access "
						"value (%s).\n",
						strerror(errno));
					exit(1);
				}

				memset(input_result_file, 0, 1);

				label.level = l;
				label.category = c;


				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_file "
						"(fail). Could not set ELMAC "
						"label (%d,%llx) to "
						"proc (%d).\n",
						l, c, getpid());
					exit(1);
				}
				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_file "
						"(fail). Cannot set "
						"uid (%i) gid (%i), "
						"to proc (%d) errno %d (%s).\n",
						tester_uid, tester_gid,
						getpid(),
						errno, strerror(errno));
					exit(1);
				}
				if (check_access_file(l, c, &S) != 0) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_file "
						"(fail). check_access_file() != 0\n");
					exit(1);
				}

				len = strlen(S);
				write(fd[1], &len, sizeof(len));
				write(fd[1], (void *) S, len);

				exit(0);
				}
				case -1:
				(void) fprintf( stderr,
						"ELMAC: check_access_proc_file "
						"(fail). Cannot fork() "
						"errno %d (%s)\n",
						errno, strerror(errno));
						return (1);
				break;
				default:
repeat_waitpid_file:
				read(fd[0], &len, sizeof(size_t));
				if (len) {
					char *S;
					size_t len_tmp;

					S = malloc(len + 1);
					if (S == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_file (fail). "
							"malloc error.\n");
							exit (1);
					}
					memset(S, 0, len + 1);
					len_tmp = strlen(input_result_file);
					input_result_file =
						(char *) realloc (input_result_file,
						 len_tmp + len + 1);
					if (input_result_file == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_file (fail). "
							"realloc error.\n");
							exit (1);
					}
					read(fd[0], S, len);
					strcat (input_result_file, S);
					close(fd[0]);
				} else
					exit(1);

				pidwait = waitpid(pid, &status, 0);
 				if (pidwait < 0) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_file "
						"(fail). waitpid < 0 "
						"errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_file;
				}
				if (WIFEXITED(status))
					print_d(p_print_f,
						"ELMAC: check_access_proc_file. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait,
						WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_f,
						"ELMAC: check_access_proc_file. "
						"Parent %i waited child %i "
 						"WTERMSIG(status): %i\n",
 						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_f,
						"ELMAC: check_access_proc_file. "
						"Parent %i waited child %i "
 						"WSTOPSIG(status): %i\n",
 						getpid(), pidwait, WSTOPSIG(status));
				}
			}
		}
	}

	len = strlen(input_result);
	len += strlen(input_result_file);
	input_result = (char *) realloc (input_result, len + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
				"ELMAC: "
				" check_access_proc_file (fail). "
				"realloc error.\n");
				exit (1);
	}
	strcat (input_result, input_result_file);

	return (0);
}

/**
 * SHMEM, semapfore share functions
 */
/*
 * Convert level to index array
 */
elmac_lev_t get_index_lev(elmac_lev_t lev)
{
	return lev - lev_min;
}


int get_cat_num()
{
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	int		num_cat_iterator = 0x0;


	mask_id = cat_set & 0xffffffff;

	for (c = 0; c <= mask_id; c ++) {
		if ((c & mask_id) == c) {
			num_cat_iterator ++;
		}
	}

	return num_cat_iterator;
}

int set_index_cat(void)
{
	elmac_cat_t	c;
	elmac_cat_t	mask_id;
	int		i = 0;


	mask_id = cat_set & 0xffffffff;

	for (c = 0; c <= mask_id; c ++) {
		if ((c & mask_id) == c) {
			*(ipc_mem_index_cat_array + i) = c;
			i ++;
			if (i >= cat_num)
				return (-1);
		}
	}

	return 0;
}


int get_index_cat(elmac_cat_t cat)
{
	int		i;

	for (i = 0; i < cat_num; i ++) {
		if (*(ipc_mem_index_cat_array + i) == cat)
			return i;
	}
	return (-1);
}


elmac_cat_t get_cat_index(int i)
{
	return *(ipc_mem_index_cat_array  + i);
}


/**
 * SHMEM
 */

#if 0
int print_array_id_shm()
{
	elmac_lev_t	l;
	int		c;

	for (l = lev_min; l <= lev_max; l ++) {
			print_d(p_print_shm,"-------------------------\n");
		for (c = 0; c < cat_num; c ++) {
			print_d(p_print_shm,
				"ELMAC: print_array_id_shm. "
				"array id shm: lev %d cat: %llx "
				"ipc_mem_id_array[%d,%d]: %d\n",
				(int)l, get_cat_index(c), get_index_lev(l), c,
				*(ipc_mem_id_array + get_index_lev(l) * cat_num + c));
		}
	}
}
#endif

int creat_shm(elmac_lev_t lev, elmac_cat_t cat)
{
	int	p_shm;
	int	i;

	if (lev < lev_min || lev > lev_max) {
		(void) fprintf(stderr,
			"ELMAC: creat_shm (fail). Bad level "
			"(lev: %d lev_min: %d, lev_max: %d)\n",
			(int) lev, (int) lev_min, (int) lev_max);
		return (1);
	}

	/*
	 * Paranoic
	 */
	if (cat_set < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_shm (fail). "
				"Bad category set(cat_set: %llx)\n",
				cat_set);
		return (1);
	}


	i_shm_create ++;


	if ((p_shm = shmget(i_shm_create, 0, 0)) > 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_shm (fail). lev: %d cat: %llx "
				"i_shm: %i - EXIST. Try IPC_RMID\n",
				lev, cat, i_shm_create);
		if ((shmctl(p_shm, IPC_RMID, 0)) < 0) {
			(void) fprintf(stderr,
				"ELMAC:  creat_shm (fail). lev: %d cat: %llx "
				"shmctl(%i, IPC_RMID, 0) < 0: %s errno: %d\n",
				lev, cat, p_shm, strerror(errno), errno);
			return (1);
		}
	}
	if ((p_shm = shmget(i_shm_create, SHMSIZE, IPC_CREAT | 0777)) < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_shm (fail). lev: %d cat: %llx"
				"shmget(%i, %i, IPC_CREAT | 0777)) < 0: "
				"%s errno: %i\n",
				lev, cat, i_shm_create, SHMSIZE,
				strerror(errno), errno);
		return (1);
	}

	i = get_index_cat(cat);
	if (i < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_shm (fail). lev: %d cat: %llx. "
				"Bad cat index (%i < 0).\n",
				lev, cat, i);
		return (1);
	}

	*(ipc_mem_id_array + get_index_lev(lev) * cat_num  + i) = i_shm_create;

	print_d(p_print_shm,
		"ELMAC: creat_shm. shmget(IPC_CREAT | 0777) %d. ",
		"lev: %d cat: %llx p_shm: %i i_shm: %i "
		"ipc_mem_id_array[%d,%d]: %d\n",
		lev, cat, i_shm_create, p_shm, i_shm_create, get_index_lev(lev), i,
		*(ipc_mem_id_array + get_index_lev(lev) * cat_num + i));

	return (0);
}


int creat_shm_obj()
{
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	elmac_label_t	label;
	pid_t		pid;
	pid_t		pidwait;
	int		status;
	int		fd[2];
	int		rc = 0;


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_shm,
					"ELMAC: creat_shm_obj. "
					"Proc shm: level:%d category: %llx\n",
					l, c);

				switch (pid = fork()) {
				case 0:

				label.level = l;
				label.category = c;


				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf( stderr,
						"ELMAC: creat_shm_obj (fail). "
						"Could not set ELMAC "
						"label (%d,%llx) to proc (%d).\n",
						l, c, getpid());
					exit(1);
				}

				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: creat_shm_obj (fail). "
						"Cannot set uid %i gid %i, "
						"errno %d (%s)\n"
						"exit(1) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (creat_shm(l, c)) {
					(void) fprintf(stderr,
						"ELMAC: creat_shm_obj (fail). "
						"Creat_shm() != 0\n");
					exit(1);
				}

				write(fd[1], &i_shm_create, sizeof(i_shm_create));
				write(fd[1], (void *) ipc_mem_id_array,
				      ipc_mem_id_array_size);

				exit(0);
				case -1:
				(void) fprintf( stderr,
						"ELMAC: creat_shm_obj (fail). "
						"Cannot fork() errno %d (%s).\n",
						errno, strerror(errno));
						return (1);
				break;
				default:
repeat_waitpid_shm:
				read(fd[0], &i_shm_create, sizeof(i_shm_create));
				read(fd[0], (void *) ipc_mem_id_array, ipc_mem_id_array_size);

				pidwait = waitpid(pid, &status, 0);
 				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: creat_shm_obj (fail). "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_shm;
				}
				if (WIFEXITED(status))
					print_d(p_print_shm,
						"ELMAC: creat_shm_obj. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait, WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_shm,
						"ELMAC: creat_shm_obj. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_shm,
						"ELMAC: creat_shm_obj. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}

	return (rc);
}


int delete_shm()
{
	int		p_shm;
	key_t		i_shm;
	elmac_lev_t	l;
	int		c;
	int		rc = 0;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c < cat_num; c ++) {

			i_shm = *(ipc_mem_id_array + get_index_lev(l) * cat_num + c);

			if ((p_shm = shmget(i_shm, 0, 0)) < 0) {
				print_d(p_print_shm,
					"ELMAC: delete_shm (fail). "
					"lev %d cat: %llx "
					"could't shmget (%i, 0, 0) for %s\n",
					l, get_cat_index(c),
					i_shm, strerror(errno));
					rc ++;
				goto delete_shm_con;
			}
			print_d(p_print_shm,
				"ELMAC: delete_shm. "
				"%i = shmget(%i, 0, 0): lev %d "
				"cat: %llx OK\n",
				p_shm, i_shm, l, get_cat_index(c));
			if ((shmctl(p_shm, IPC_RMID, 0)) < 0) {
				print_d(p_print_shm,
					"ELMAC: delete_shm (fail). "
					"lev %d cat: %llx "
					"could't shmctl(%i, 0, IPC_RMID, 0): %s\n",
					l, get_cat_index(c),
					p_shm, strerror(errno));
				rc ++;
				goto delete_shm_con;
			}
			print_d(p_print_shm,
				"ELMAC: delete_shm. "
				"lev: %d cat: %llx shmctl(%i, 0, IPC_RMID, 0) "
				"ipc_mem_id_array[%d,%d]: %d\n",
				l, get_cat_index(c), p_shm, get_index_lev(l), c,
				*(ipc_mem_id_array + get_index_lev(l) * cat_num + c));

delete_shm_con:
			;;
		}
	}
	return (rc);
}


int check_shm(key_t i_shm, char *p_ret_d)
{
	int	p_shm;
	struct	shmid_ds	shmbuf;
	char	*p_shmat;

	memset(p_ret_d, 0, 10);

	if ((p_shm = shmget(i_shm, SHMSIZE, 0)) < 0) {
		print_d(p_print_shm,
			"ELMAC: check_shm. "
			"shmget(%i, 0): %s.\n",
			i_shm, strerror(errno));
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 0) = 'p';
			break;
		case ENOENT:
			*(p_ret_d + 0) = '#';
			break;
		default:
			*(p_ret_d + 0) = '?';
		}
		*(p_ret_d + 1) = '-';
		*(p_ret_d + 2) = '-';
		*(p_ret_d + 3) = '-';
		*(p_ret_d + 4) = '-';
		*(p_ret_d + 5) = '-';
	}
	*(p_ret_d + 0) = 'g';

	if (shmctl(p_shm, IPC_STAT, &shmbuf) < 0) {
		print_d(p_print_shm,
			"ELMAC: check_shm. "
			"shmctl(%i, 0, IPC_STAT): %s\n",
			i_shm, strerror(errno));
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 1) = 'p';
			break;
		default:
			*(p_ret_d + 1) = '?';
		}
	} else
		*(p_ret_d + 1) = 'c';

	if (shmctl(p_shm, IPC_SET, &shmbuf) < 0) {
		print_d(p_print_shm,
			"ELMAC: check_shm. "
			"i_shm %i shmctl(IPC_SET): %s\n",
			i_shm, strerror(errno));
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 2) = 'p';
			break;
		default:
			*(p_ret_d + 2) = '?';
		}
	} else
		*(p_ret_d + 2) = 's';
	if ((p_shmat = (char *)shmat(p_shm, 0, 0)) == (char *)-1) {
		print_d(p_print_shm,
			"ELMAC: check_shm. "
			"i_shm %i shmat(): %s\n",
			i_shm, strerror(errno));
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 3) = 'p';
			break;
		default:
			*(p_ret_d + 3) = '?';
		}
		*(p_ret_d + 4) = '-';
	} else
		*(p_ret_d + 3) = 'a';
	if (shmdt(p_shmat) < 0) {
		print_d(p_print_shm,
			"ELMAC: check_shm. "
			"i_shm %i shmdt(p_shmat): %s\n",
			i_shm, strerror(errno));
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 4) = 'p';
			break;
		default:
			*(p_ret_d + 4) = '?';
		}
	} else
		*(p_ret_d + 4) = 'd';

	return (0);
}

int check_access_shm(elmac_lev_t lev, elmac_cat_t cat, char **buf)
{
	elmac_label_t label;
	char *S;
	char *p_ret_d;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	key_t 		i_shm;
	elmac_cat_t	c;

	p_ret_d = malloc(10);
	if (p_ret_d == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_shm (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}

	memset(p_ret_d, 0, 10);

	S = malloc(1);
	if (S == (char*)NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_msg (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}
	memset(S, 0, 1);

	if (elmac_get_label_from_self(&label) < 0) {
		(void) fprintf(stderr,
			"ELMAC: check_access_shm (fail). "
			"Could not get ELMAC label (%d,%llx) from proc.\n",
			lev, cat);
			return (1);
	}


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {
				size_t len_tmp;

				i_shm = *(ipc_mem_id_array + get_index_lev(l) * cat_num + c);

				check_shm(i_shm, p_ret_d);
				//printf("proc(%d.%llx) to  i_shm(%d.%llx) "
				//	"access: %s\n",
				//	label.level, label.category,
				//	l, c, p_ret_d);
				memset(input_result_buf, 0, TMP_BUF);
				snprintf(input_result_buf, TMP_BUF,
					 "proc(%d.%llx) to  i_shm(%d.%llx) "
					 "access: %s\n",
					 label.level, label.category,
					 l, c, p_ret_d);
				len_tmp = strlen(S);
				S = (char *) realloc (S, len_tmp +
						 strlen(input_result_buf) + 1);
				if (S == (char *)NULL) {
					(void) fprintf(stderr,
							"ELMAC: check_access_shm (fail). "
							"realloc error.\n");
							return (1);
				}
				strcat (S, input_result_buf);

				print_d(p_print_shm, "proc(%d.%llx) to "
					"i_shm(%d %d.%llx) access: %s\n",
					label.level, label.category,
					i_shm, l, c, p_ret_d);
			}
		}
	}

	*buf = strdup(S);

	free(p_ret_d);
	free(S);

	return (0);
}

int check_access_proc_shm()
{
	elmac_label_t label;
	pid_t	pid;
	pid_t	pidwait;
	int	status;
	char	*input_result_shm;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	int		fd[2];
	size_t		len;
	int		rc;

	input_result_shm = malloc(1);
	if (input_result_shm == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_proc_shm (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
				return 1;
	}
	memset(input_result_shm, 0, 1);

	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_f,
					"ELMAC: check_access_proc_shm. "
					"Proc: level:%d category: %llx\n", l, c);
				switch (pid = fork()) {
				case 0: {

				char *S;

				if (input_result_shm == NULL) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_shm (fail). "
						"Could not memmory alloc for access "
						"value (%s).\n",
						strerror(errno));
					exit(1);
				}

				memset(input_result_shm, 0, 1);

				label.level = l;
				label.category = c;

				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_shm "
						"(fail). Could not set ELMAC "
						"label (%d,%llx). \n",
						l, c);
					exit(1);
				}
				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_shm "
						"(fail). Cannot set uid %i "
						"gid %i, errno %d (%s) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (check_access_shm(l, c, &S) != 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_shm "
						"(fail). check_access_shm() != 0\n");
					exit(1);
				}

				len = strlen(S);
				write(fd[1], &len, sizeof(len));
				write(fd[1], (void *) S, len);

				exit(0);
				}
				case -1:
				(void) fprintf(stderr,
						"ELMAC: check_access_proc_shm "
						"(fail). Cannot fork() "
						"errno %d (%s)\n",
					       errno, strerror(errno));
				break;
				default:
repeat_waitpid_shm_proc:
				read(fd[0], &len, sizeof(size_t));
				if (len) {
					char *S;
					size_t len_tmp;

					S = malloc(len + 1);
					if (S == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_msg (fail). "
							"malloc error.\n");
							exit (1);
					}
					memset(S, 0, len + 1);
					len_tmp = strlen(input_result_shm);
					input_result_shm =
						(char *) realloc (input_result_shm,
						 len_tmp + len + 1);
					if (input_result_shm == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_shm (fail). "
							"realloc error.\n");
							exit (1);
					}
					read(fd[0], S, len);
					strcat (input_result_shm, S);
					close(fd[0]);
				} else
					exit(1);
				pidwait = waitpid(pid, &status, 0);
				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_shm. "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_shm_proc;
				}
				if (WIFEXITED(status))
					print_d(p_print_shm,
						"ELMAC: check_access_proc_shm. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait,
						WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_shm,
						"ELMAC: check_access_proc_shm. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_shm,
						"ELMAC: check_access_proc_shm. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}

	len = strlen(input_result);
	len += strlen(input_result_shm);
	input_result = (char *) realloc (input_result, len + 1);
	if (input_result_shm == (char *)NULL) {
		(void) fprintf(stderr,
				"ELMAC: "
				" check_access_proc_shm (fail). "
				"realloc error.\n");
				exit (1);
	}
	strcat (input_result, input_result_shm);

	return (0);
}



/**
 * Semaphore
 */

int creat_sem(elmac_lev_t lev, elmac_cat_t cat)
{
	int	p_sem;
	int	i;

	if (lev < lev_min || lev > lev_max) {
		(void) fprintf(stderr,
			"ELMAC: creat_sem (fail). Bad level "
			"(lev: %d lev_min: %d, lev_max: %d)\n",
			(int) lev, (int) lev_min, (int) lev_max);
		return (1);
	}

	/*
	 * Paranoic
	 */
	if (cat_set < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_sem (fail). "
				"Bad category set(cat_set: %llx)\n",
				cat_set);
		return (1);
	}


	i_sem_create ++;


	if ((p_sem = semget(i_sem_create, 0, 0)) > 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_sem (fail). lev: %d cat: %llx "
				"i_sem: %i - EXIST. Try IPC_RMID\n",
				lev, cat, i_sem_create);
		if ((semctl(p_sem, IPC_RMID, 0)) < 0) {
			(void) fprintf(stderr,
				"ELMAC:  creat_sem (fail). lev: %d cat: %llx "
				"semctl(%i, IPC_RMID, 0) < 0: %s errno: %d\n",
				lev, cat, p_sem, strerror(errno), errno);
			return (1);
		}
	}
	if ((p_sem = semget(i_sem_create, 2, IPC_CREAT | 0777)) < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_sem (fail). lev: %d cat: %llx"
				"semget(%i, %i, IPC_CREAT | 0777)) < 0: "
				"%s errno: %i\n",
				lev, cat, i_sem_create, 2,
				strerror(errno), errno);
		return (1);
	}

	i = get_index_cat(cat);
	if (i < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_sem (fail). lev: %d cat: %llx. "
				"Bad cat index (%i < 0).\n",
				lev, cat, i);
		return (1);
	}

	*(ipc_mem_id_array + get_index_lev(lev) * cat_num  + i) = i_sem_create;

	print_d(p_print_sem,
		"ELMAC: creat_sem. semget(IPC_CREAT | 0777) %d. ",
		"lev: %d cat: %llx p_sem: %i i_sem: %i "
		"ipc_mem_id_array[%d,%d]: %d\n",
		lev, cat, i_sem_create, p_sem, i_sem_create, get_index_lev(lev), i,
		*(ipc_mem_id_array + get_index_lev(lev) * cat_num + i));

	return (0);
}


int creat_sem_obj()
{
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	elmac_label_t	label;
	pid_t		pid;
	pid_t		pidwait;
	int		status;
	int		fd[2];
	int		rc = 0;


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_sem,
					"ELMAC: creat_sem_obj. "
					"Proc sem: level:%d category: %llx\n",
					l, c);

				switch (pid = fork()) {
				case 0:

				label.level = l;
				label.category = c;


				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf( stderr,
						"ELMAC: creat_sem_obj (fail). "
						"Could not set ELMAC "
						"label (%d,%llx) to proc (%d).\n",
						l, c, getpid());
					exit(1);
				}

				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: creat_sem_obj (fail). "
						"Cannot set uid %i gid %i, "
						"errno %d (%s)\n"
						"exit(1) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (creat_sem(l, c)) {
					(void) fprintf(stderr,
						"ELMAC: creat_sem_obj (fail). "
						"Creat_sem() != 0\n");
					exit(1);
				}

				write(fd[1], &i_sem_create, sizeof(i_sem_create));
				write(fd[1], (void *) ipc_mem_id_array,
				      ipc_mem_id_array_size);

				exit(0);
				case -1:
				(void) fprintf( stderr,
						"ELMAC: creat_sem_obj (fail). "
						"Cannot fork() errno %d (%s).\n",
						errno, strerror(errno));
						return (1);
				break;
				default:
repeat_waitpid_sem:
				read(fd[0], &i_sem_create, sizeof(i_sem_create));
				read(fd[0], (void *) ipc_mem_id_array, ipc_mem_id_array_size);

				pidwait = waitpid(pid, &status, 0);
 				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: creat_sem_obj (fail). "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_sem;
				}
				if (WIFEXITED(status))
					print_d(p_print_sem,
						"ELMAC: creat_sem_obj. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait, WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_sem,
						"ELMAC: creat_sem_obj. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_sem,
						"ELMAC: creat_sem_obj. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}

	return (rc);
}


int delete_sem()
{
	int		p_sem;
	key_t		i_sem;
	elmac_lev_t	l;
	int		c;
	int		rc = 0;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c < cat_num; c ++) {

			i_sem = *(ipc_mem_id_array + get_index_lev(l) * cat_num + c);

			if ((p_sem = semget(i_sem, 0, 0)) < 0) {
				print_d(p_print_sem,
					"ELMAC: delete_sem (fail). "
					"lev %d cat: %llx "
					"could't semget (%i, 0, 0) for %s\n",
					l, get_cat_index(c),
					i_sem, strerror(errno));
					rc ++;
				goto delete_sem_con;
			}
			print_d(p_print_sem,
				"ELMAC: delete_sem. "
				"%i = semget(%i, 0, 0): lev %d "
				"cat: %llx OK\n",
				p_sem, i_sem, l, get_cat_index(c));
			if ((semctl(p_sem, IPC_RMID, 0)) < 0) {
				print_d(p_print_sem,
					"ELMAC: delete_sem (fail). "
					"lev %d cat: %llx "
					"could't semctl(%i, 0, IPC_RMID, 0): %s\n",
					l, get_cat_index(c),
					p_sem, strerror(errno));
				rc ++;
				goto delete_sem_con;
			}
			print_d(p_print_sem,
				"ELMAC: delete_sem. "
				"lev: %d cat: %llx semctl(%i, 0, IPC_RMID, 0) "
				"ipc_mem_id_array[%d,%d]: %d\n",
				l, get_cat_index(c), p_sem, get_index_lev(l), c,
				*(ipc_mem_id_array + get_index_lev(l) * cat_num + c));

delete_sem_con:
			;;
		}
	}
	return (rc);
}

int check_sem(key_t i_sem, char *p_ret_d)
{
	int	p_sem;
	union	semun {
		int	val;
		struct	semid_ds	*mbuf;
		ushort	*array;
	}	arg;
	struct	semid_ds	mbuf;
	struct	sembuf	plus_sem[1] = {{0, 1, SEM_UNDO | IPC_NOWAIT}};
	struct	sembuf	minus_sem[1] = {{0, -1, SEM_UNDO | IPC_NOWAIT}};
	struct	sembuf	null_sem[1] = {{0, 0, SEM_UNDO | IPC_NOWAIT}};

	memset(p_ret_d, 0, 10);

	arg.mbuf = &mbuf;

	if ((p_sem = semget(i_sem, 0, 0)) < 0) {
		print_d(p_print_sem,
			"ELMAC: check_sem. "
			"shmget(%i, 0): %s.\n",
			i_sem, strerror(errno));
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 0) = 'p';
			break;
		case ENOENT:
			*(p_ret_d + 0) = '#';
			break;
		default:
			*(p_ret_d + 0) = '?';
			break;
		}
		*(p_ret_d + 1) = '-';
		*(p_ret_d + 2) = '-';
		*(p_ret_d + 3) = '-';
		*(p_ret_d + 4) = '-';
		*(p_ret_d + 5) = '-';
	}
	*(p_ret_d + 0) = 'g';

	if (semctl(p_sem, 0, IPC_STAT, arg) < 0) {
		print_d(p_print_sem,
			"ELMAC: check_sem. "
			"semctl(%i, 0, IPC_STAT): %s "
			"errno: %i\n",
			i_sem, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 1) = 'p';
			break;
		default:
			*(p_ret_d + 1) = '?';
		}
	} else
		*(p_ret_d + 1) = 'c';

	if (semctl(p_sem, 0, IPC_SET, arg) < 0) {
		print_d(p_print_sem,
			"ELMAC: check_sem. "
			"semctl(%i, 0, IPC_SET): %s "
			"errno: %i\n",
			i_sem, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 2) = 'p';
			break;
		default:
			*(p_ret_d + 2) = '?';
		}
	} else
		*(p_ret_d + 2) = 's';
	if (semop(p_sem, plus_sem, 1) < 0) {
		print_d(p_print_sem,
			"ELMAC: check_sem. "
			"semop(%i, plus_sem, 1): %s "
			"errno: %i\n",
			i_sem, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 3) = 'p';
			break;
		default:
			*(p_ret_d + 3) = '?';
		}
	} else
		*(p_ret_d + 3) = '+';
	if (semop(p_sem, minus_sem, 1) < 0) {
		print_d(p_print_sem,
			"ELMAC: check_sem. "
			"semop(%i, minus_sem, 1): %s "
			"errno: %i\n",
			i_sem, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 4) = 'p';
			break;
		default:
			*(p_ret_d + 4) = '?';
		}
	} else
		*(p_ret_d + 4) = 'm';
	if (semop(p_sem, null_sem, 1) < 0) {
		print_d(p_print_sem,
			"ELMAC: check_sem. "
			"semop(%i, null_sem, 1): %s "
			"errno: %i\n",
			i_sem, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 5) = 'p';
			break;
		default:
			*(p_ret_d + 5) = '?';
		}
	} else
		*(p_ret_d + 5) = 'n';

	return (0);
}

int check_access_sem(elmac_lev_t lev, elmac_cat_t cat, char **buf)
{
	elmac_label_t label;
	char *S;
	char *p_ret_d;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	key_t 		i_sem;
	elmac_cat_t	c;

	p_ret_d = malloc(10);
	if (p_ret_d == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_sem (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}

	memset(p_ret_d, 0, 10);

	S = malloc(1);
	if (S == (char*)NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_msg (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}
	memset(S, 0, 1);

	if (elmac_get_label_from_self(&label) < 0) {
		(void) fprintf(stderr,
			"ELMAC: check_access_sem (fail). "
			"Could not get ELMAC label (%d,%llx) from proc.\n",
			lev, cat);
			return (1);
	}


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {
				size_t len_tmp;

				i_sem = *(ipc_mem_id_array + get_index_lev(l) * cat_num + c);

				check_sem(i_sem, p_ret_d);
				//printf("proc(%d.%llx) to  i_sem(%d.%llx) "
				//	"access: %s\n",
				//	label.level, label.category,
				//	l, c, p_ret_d);
				memset(input_result_buf, 0, TMP_BUF);
				snprintf(input_result_buf, TMP_BUF,
					 "proc(%d.%llx) to  i_sem(%d.%llx) "
					 "access: %s\n",
					 label.level, label.category,
					 l, c, p_ret_d);
				len_tmp = strlen(S);
				S = (char *) realloc (S, len_tmp +
						 strlen(input_result_buf) + 1);
				if (S == (char *)NULL) {
					(void) fprintf(stderr,
							"ELMAC: check_access_msg (fail). "
							"realloc error.\n");
							return (1);
				}
				strcat (S, input_result_buf);

				print_d(p_print_sem, "proc(%d.%llx) to "
					"i_sem(%d %d.%llx) access: %s\n",
					label.level, label.category,
					i_sem, l, c, p_ret_d);
			}
		}
	}

	*buf = strdup(S);

	free(p_ret_d);
	free(S);

	return (0);
}

int check_access_proc_sem()
{
	elmac_label_t label;
	pid_t	pid;
	pid_t	pidwait;
	int	status;
	char	*input_result_sem;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	int		fd[2];
	size_t		len;
	int		rc;

	input_result_sem = malloc(1);
	if (input_result_sem == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_proc_sem (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
				return 1;
	}
	memset(input_result_sem, 0, 1);

	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_f,
					"ELMAC: check_access_proc_sem. "
					"Proc: level:%d category: %llx\n", l, c);
				switch (pid = fork()) {
				case 0: {
				char *S;

				if (input_result_sem == NULL) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_sem (fail). "
						"Could not memmory alloc for access "
						"value (%s).\n",
						strerror(errno));
					exit(1);
				}

				memset(input_result_sem, 0, 1);

				label.level = l;
				label.category = c;

				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_sem "
						"(fail). Could not set ELMAC "
						"label (%d,%llx). \n",
						l, c);
					exit(1);
				}
				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_sem "
						"(fail). Cannot set uid %i "
						"gid %i, errno %d (%s) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (check_access_sem(l, c, &S) != 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_sem "
						"(fail). check_access_shm() != 0\n");
					exit(1);
				}

				len = strlen(S);
				write(fd[1], &len, sizeof(len));
				write(fd[1], (void *) S, len);

				exit(0);
				}
				case -1:
				(void) fprintf(stderr,
						"ELMAC: check_access_proc_sem "
						"(fail). Cannot fork() "
						"errno %d (%s)\n",
					       errno, strerror(errno));
				break;
				default:
repeat_waitpid_sem_proc:
				read(fd[0], &len, sizeof(size_t));
				if (len) {
					char *S;
					size_t len_tmp;

					S = malloc(len + 1);
					if (S == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_sem (fail). "
							"malloc error.\n");
							exit (1);
					}
					memset(S, 0, len + 1);
					len_tmp = strlen(input_result_sem);
					input_result_sem =
						(char *) realloc (input_result_sem,
						 len_tmp + len + 1);
					if (input_result_sem == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_sem (fail). "
							"realloc error.\n");
							exit (1);
					}
					read(fd[0], S, len);
					strcat (input_result_sem, S);
					close(fd[0]);
				} else
					exit(1);
				pidwait = waitpid(pid, &status, 0);
				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_sem. "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_sem_proc;
				}
				if (WIFEXITED(status))
					print_d(p_print_sem,
						"ELMAC: check_access_proc_sem. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait,
						WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_sem,
						"ELMAC: check_access_proc_sem. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_sem,
						"ELMAC: check_access_proc_sem. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}

	len = strlen(input_result);
	len += strlen(input_result_sem);
	input_result = (char *) realloc (input_result, len + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
				"ELMAC: "
				" check_access_proc_sem (fail). "
				"realloc error.\n");
				exit (1);
	}
	strcat (input_result, input_result_sem);

	return (0);
}

/**
 * Messages
 */

int creat_msg(elmac_lev_t lev, elmac_cat_t cat)
{
	int	p_msg;
	int	i;

	if (lev < lev_min || lev > lev_max) {
		(void) fprintf(stderr,
			"ELMAC: creat_msg (fail). Bad level "
			"(lev: %d lev_min: %d, lev_max: %d)\n",
			(int) lev, (int) lev_min, (int) lev_max);
		return (1);
	}

	/*
	 * Paranoic
	 */
	if (cat_set < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_msg (fail). "
				"Bad category set(cat_set: %llx)\n",
				cat_set);
		return (1);
	}


	i_msg_create ++;


	if ((p_msg = msgget(i_msg_create, 0)) > 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_msg (fail). lev: %d cat: %llx "
				"i_msg: %i - EXIST. Try IPC_RMID\n",
				lev, cat, i_msg_create);
		if ((msgctl(p_msg, IPC_RMID, 0)) < 0) {
			(void) fprintf(stderr,
				"ELMAC:  creat_msg (fail). lev: %d cat: %llx "
				"msgctl(%i, IPC_RMID, 0) < 0: %s errno: %d\n",
				lev, cat, p_msg, strerror(errno), errno);
			return (1);
		}
	}
	if ((p_msg = msgget(i_msg_create, IPC_CREAT | 0777)) < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_msg (fail). lev: %d cat: %llx"
				"msgget(%i, %i, IPC_CREAT | 0777)) < 0: "
				"%s errno: %i\n",
				lev, cat, i_msg_create, 2,
				strerror(errno), errno);
		return (1);
	}

	i = get_index_cat(cat);
	if (i < 0) {
		(void) fprintf( stderr,
				"ELMAC: creat_msg (fail). lev: %d cat: %llx. "
				"Bad cat index (%i < 0).\n",
				lev, cat, i);
		return (1);
	}

	*(ipc_mem_id_array + get_index_lev(lev) * cat_num  + i) = i_msg_create;

	print_d(p_print_msg,
		"ELMAC: creat_msg. msgget(IPC_CREAT | 0777) %d. ",
		"lev: %d cat: %llx p_msg: %i i_msg: %i "
		"ipc_mem_id_array[%d,%d]: %d\n",
		lev, cat, i_msg_create, p_msg, i_msg_create, get_index_lev(lev), i,
		*(ipc_mem_id_array + get_index_lev(lev) * cat_num + i));

	return (0);
}


int creat_msg_obj()
{
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	elmac_label_t	label;
	pid_t		pid;
	pid_t		pidwait;
	int		status;
	int		fd[2];
	int		rc = 0;


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_msg,
					"ELMAC: creat_msg_obj. "
					"Proc msg: level:%d category: %llx\n",
					l, c);

				switch (pid = fork()) {
				case 0:

				label.level = l;
				label.category = c;


				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf( stderr,
						"ELMAC: creat_msg_obj (fail). "
						"Could not set ELMAC "
						"label (%d,%llx) to proc (%d).\n",
						l, c, getpid());
					exit(1);
				}

				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: creat_msg_obj (fail). "
						"Cannot set uid %i gid %i, "
						"errno %d (%s)\n"
						"exit(1) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (creat_msg(l, c)) {
					(void) fprintf(stderr,
						"ELMAC: creat_msg_obj (fail). "
						"Creat_msg() != 0\n");
					exit(1);
				}

				write(fd[1], &i_msg_create, sizeof(i_msg_create));
				write(fd[1], (void *) ipc_mem_id_array,
				      ipc_mem_id_array_size);

				exit(0);
				case -1:
				(void) fprintf( stderr,
						"ELMAC: creat_msg_obj (fail). "
						"Cannot fork() errno %d (%s).\n",
						errno, strerror(errno));
						return (1);
				break;
				default:
repeat_waitpid_msg:
				read(fd[0], &i_msg_create, sizeof(i_msg_create));
				read(fd[0], (void *) ipc_mem_id_array, ipc_mem_id_array_size);

				pidwait = waitpid(pid, &status, 0);
 				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: creat_msg_obj (fail). "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_msg;
				}
				if (WIFEXITED(status))
					print_d(p_print_msg,
						"ELMAC: creat_msg_obj. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait, WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_msg,
						"ELMAC: creat_msg_obj. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_msg,
						"ELMAC: creat_msg_obj. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}

	return (rc);
}


int delete_msg()
{
	int		p_msg;
	key_t		i_msg;
	elmac_lev_t	l;
	int		c;
	int		rc = 0;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c < cat_num; c ++) {

			i_msg = *(ipc_mem_id_array + get_index_lev(l) * cat_num + c);

			if ((p_msg = msgget(i_msg, 0)) < 0) {
				print_d(p_print_msg,
					"ELMAC: delete_msg (fail). "
					"lev %d cat: %llx "
					"could't msgget (%i, 0, 0) for %s\n",
					l, get_cat_index(c),
					i_msg, strerror(errno));
					rc ++;
				goto delete_msg_con;
			}
			print_d(p_print_msg,
				"ELMAC: delete_msg. "
				"%i = msgget(%i, 0, 0): lev %d "
				"cat: %llx OK\n",
				p_msg, i_msg, l, get_cat_index(c));
			if ((msgctl(p_msg, IPC_RMID, 0)) < 0) {
				print_d(p_print_msg,
					"ELMAC: delete_msg (fail). "
					"lev %d cat: %llx "
					"could't msgctl(%i, 0, IPC_RMID, 0): %s\n",
					l, get_cat_index(c),
					p_msg, strerror(errno));
				rc ++;
				goto delete_msg_con;
			}
			print_d(p_print_msg,
				"ELMAC: delete_msg. "
				"lev: %d cat: %llx msgctl(%i, 0, IPC_RMID, 0) "
				"ipc_mem_id_array[%d,%d]: %d\n",
				l, get_cat_index(c), p_msg, get_index_lev(l), c,
				*(ipc_mem_id_array + get_index_lev(l) * cat_num + c));

delete_msg_con:
			;;
		}
	}
	return (rc);
}


int send_msg(elmac_lev_t lev, elmac_cat_t cat)
{
	key_t	i_msg;
	int	p_msg;
	struct	msqid_ds	msg_stat;
	int	i = 0;
	int	rc;


	if (lev < lev_min || lev > lev_max) {
		(void) fprintf(stderr,
			"ELMAC: send_msg (fail). Bad level "
			"(lev: %d lev_min: %d, lev_max: %d)\n",
			(int) lev, (int) lev_min, (int) lev_max);
		return (1);
	}

	/*
	 * Paranoic
	 */
	if (cat_set < 0) {
		(void) fprintf( stderr,
				"ELMAC: send_msg (fail). "
				"Bad category set(cat_set: %llx)\n",
				cat_set);
		return (1);
	}

	i = get_index_cat(cat);
	if (i < 0) {
		(void) fprintf( stderr,
				"ELMAC: send_msg (fail). lev: %d cat: %llx. "
				"Bad cat index (%i < 0).\n",
				lev, cat, i);
		return (1);
	}


	i_msg = *(ipc_mem_id_array + get_index_lev(lev) * cat_num  + i);

	if ((p_msg = msgget(i_msg, 0)) < 0) {
		(void) fprintf( stderr,
			"ELMAC: send_msg (fail). lev: %d cat: %llx. "
			"msgget(%i, 0): "
			"%s errno: %i\n",
			lev, cat,
			p_msg, strerror(errno), errno);
	}

	if ((msgctl(p_msg, IPC_STAT, &msg_stat)) < 0) {
		(void) fprintf( stderr,
			"ELMAC: send_msg (fail). lev: %d cat: %llx. "
			"msgctl(%i, IPC_STAT): "
			"%s errno: %i\n",
			lev, cat,
			p_msg, strerror(errno), errno);
	}

	if (msg_stat.msg_qnum == 0) {
		if ((rc = msgsnd(p_msg, &msg_snd,
			strlen(msg_snd.mtext) + 1, IPC_NOWAIT)) < 0) {
			(void) fprintf( stderr,
				"ELMAC: send_msg (fail). lev: %d cat: %llx. "
				"msgsnd(%i, %i): "
				"%s errno: %i\n",
				lev, cat,
				p_msg, i, strerror(errno), errno);
		}
	}

	return (0);
}


int send_msg_obj()
{
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	elmac_label_t	label;
	pid_t		pid;
	pid_t		pidwait;
	int		status;
	int		rc = 0;


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				print_d(p_print_msg,
					"ELMAC: send_msg_obj. "
					"Proc msg: level:%d category: %llx\n",
					l, c);

				switch (pid = fork()) {
				case 0:

				label.level = l;
				label.category = c;


				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf( stderr,
						"ELMAC: send_msg_obj (fail). "
						"Could not set ELMAC "
						"label (%d,%llx) to proc (%d).\n",
						l, c, getpid());
					exit(1);
				}

				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: send_msg_obj (fail). "
						"Cannot set uid %i gid %i, "
						"errno %d (%s)\n"
						"exit(1) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (send_msg(l, c)) {
					(void) fprintf(stderr,
						"ELMAC: send_msg_obj (fail). "
						"Creat_msg() != 0\n");
					exit(1);
				}

				exit(0);
				case -1:
				(void) fprintf( stderr,
						"ELMAC: send_msg_obj (fail). "
						"Cannot fork() errno %d (%s).\n",
						errno, strerror(errno));
						return (1);
				break;
				default:
repeat_waitpid_msg_send:

				pidwait = waitpid(pid, &status, 0);
 				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: send_msg_obj (fail). "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_msg_send;
				}
				if (WIFEXITED(status))
					print_d(p_print_msg,
						"ELMAC: send_msg_obj. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait, WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_msg,
						"ELMAC: creat_msg_obj. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_msg,
						"ELMAC: send_msg_obj. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}

	return (rc);
}



int check_msg(key_t i_msg, char *p_ret_d)
{
	int	p_msg;
	struct	msqid_ds
		msg_stat;
	int	rc;

	memset(p_ret_d, 0, 10);

	if ((p_msg = msgget(i_msg, 0)) < 0) {
		print_d(p_print_msg,
			"ELMAC: check_msg "
			"msgget(%i, 0): "
			"errno: %i\n",
			p_msg, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 0) = 'p';
			break;
		case ENOENT:
			*(p_ret_d + 0) = '#';
			break;
		default:
			*(p_ret_d + 0) = '?';
		}
		*(p_ret_d + 1) = '-';
		*(p_ret_d + 2) = '-';
		*(p_ret_d + 3) = '-';
		*(p_ret_d + 4) = '-';
	} else
		*(p_ret_d + 0) = 'g';

	if ((msgctl(p_msg, IPC_STAT, &msg_stat)) < 0) {
		print_d(p_print_msg,
			"ELMAC: check_msg. "
			"msgctl(%i, IPC_STAT): "
			"%s errno: %i\n",
			p_msg, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 1) = 'p';
			break;
		default:
			*(p_ret_d + 1) = '?';
		}
	} else
		*(p_ret_d + 1) = 'c';

	if ((msgctl(p_msg, IPC_SET, &msg_stat)) < 0) {
		print_d(p_print_msg,
			"ELMAC: check_msg. "
			"msgctl(%i, IPC_SET): "
			"%s errno: %i\n",
			p_msg, strerror(errno), errno);
		switch (errno) {
		case EACCES:
		case EPERM:
			*(p_ret_d + 2) = 'p';
			break;
		default:
			*(p_ret_d + 2) = '?';
		}
	} else
		*(p_ret_d + 2) = 's';

	if (msg_stat.msg_qnum > 0) {
		if ((rc = msgrcv(p_msg, &msg_rcv,
			MSGMAX, 0, IPC_NOWAIT)) < 0) {
			print_d(p_print_msg,
				"ELMAC: check_msg. "
				"msgrcv(%i, 10): "
				"errno: %i\n",
				p_msg, strerror(errno), errno);
			switch (errno) {
			case EACCES:
			case EPERM:
				*(p_ret_d + 3) = 'p';
				break;
			default:
				*(p_ret_d + 3) = '?';
			}
		} else
			*(p_ret_d + 3) = 'r';

		if ((rc = msgsnd(p_msg, &msg_snd,
			strlen(msg_snd.mtext) + 1, IPC_NOWAIT)) < 0) {
			print_d(p_print_msg,
				"ELMAC: check_msg. "
				"msgsnd(%i, 1): "
				"%s errno: %i\n",
				p_msg, strerror(errno), errno);
			switch (errno) {
			case EACCES:
			case EPERM:
				*(p_ret_d + 4) = 'p';
				break;
			default:
				*(p_ret_d + 4) = '?';
			}
		} else
			*(p_ret_d + 4) = 's';
	} else {
		if ((rc = msgsnd(p_msg, &msg_snd,
			strlen(msg_snd.mtext) + 1, IPC_NOWAIT)) < 0) {
			print_d(p_print_msg,
				"ELMAC: check_msg. "
				"msgsnd(%i, 2): "
				"errno: %i\n",
				p_msg, strerror(errno), errno);
			switch (errno) {
			case EACCES:
			case EPERM:
				*(p_ret_d + 4) = 'p';
				break;
			default:
				*(p_ret_d + 4) = '?';
			}
		} else
			*(p_ret_d + 4) = 's';

		if ((rc = msgrcv(p_msg, &msg_rcv,
			MSGMAX, 0, IPC_NOWAIT)) < 0) {
			print_d(p_print_msg,
				"ELMAC: check_msg. "
				"msgrcv(%i, 2): "
				"%s errno: %i\n",
				p_msg, strerror(errno), errno);
			switch (errno) {
			case EACCES:
			case EPERM:
				*(p_ret_d + 3) = 'p';
				break;
			default:
				*(p_ret_d + 3) = '?';
			}
		} else
			*(p_ret_d + 3) = 'r';
	}

	return (0);
}




int check_access_msg(elmac_lev_t lev, elmac_cat_t cat, char **buf)
{
	elmac_label_t label;
	char *p_ret_d;
	char *S;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	key_t 		i_msg;
	elmac_cat_t	c;

	p_ret_d = malloc(10);
	if (p_ret_d == (char*)NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_msg (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}
	memset(p_ret_d, 0, 1);

	S = malloc(1);
	if (S == (char*)NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_msg (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
		return (1);
	}
	memset(S, 0, 1);

	if (elmac_get_label_from_self(&label) < 0) {
		(void) fprintf(stderr,
			"ELMAC: check_access_msg (fail). "
			"Could not get ELMAC label (%d,%llx) from proc.\n",
			lev, cat);
			return (1);
	}


	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {
				size_t len_tmp;

				i_msg = *(ipc_mem_id_array + get_index_lev(l) * cat_num + c);

				check_msg(i_msg, p_ret_d);
				//printf("proc(%d.%llx) to  i_msg(%d.%llx) "
				//	"access: %s\n",
				//	label.level, label.category,
				//	l, c, p_ret_d);

				memset(input_result_buf, 0, TMP_BUF);
				snprintf(input_result_buf, TMP_BUF,
					 "proc(%d.%llx) to  i_msg(%d.%llx) "
					 "access: %s\n",
					 label.level, label.category,
					 l, c, p_ret_d);
				len_tmp = strlen(S);
				S = (char *) realloc (S, len_tmp +
						 strlen(input_result_buf) + 1);
				if (S == (char *)NULL) {
					(void) fprintf(stderr,
							"ELMAC: check_access_msg (fail). "
							"realloc error.\n");
							return (1);
				}
				strcat (S, input_result_buf);

				print_d(p_print_msg, "proc(%d.%llx) to "
					"i_msg(%d %d.%llx) access: %s\n",
					label.level, label.category,
					i_msg, l, c, p_ret_d);
			}
		}
	}

	*buf = strdup(S);

	free(p_ret_d);
	free(S);

	return (0);
}


int check_access_proc_msg()
{
	elmac_label_t label;
	pid_t	pid;
	pid_t	pidwait;
	int	status;
	elmac_lev_t	l;
	elmac_cat_t	mask_id;
	elmac_cat_t	c;
	char		*input_result_msg;
	size_t		len;
	int		fd[2];
	int		rc;

	input_result_msg = malloc(1);
	if (input_result_msg == NULL) {
		(void) fprintf( stderr,
				"ELMAC: check_access_proc_msg (fail). "
				"Could not memmory alloc for access "
				"value (%s).\n",
				strerror(errno));
				return 1;
	}
	memset(input_result_msg, 0, 1);

	mask_id = cat_set & 0xffffffff;

	for (l = lev_min; l <= lev_max; l ++) {
		for (c = 0; c <= mask_id; c ++) {
			if ((c & mask_id) == c) {

				pipe(fd);

				print_d(p_print_f,
					"ELMAC: check_access_proc_msg. "
					"Proc: level:%d category: %llx\n", l, c);
				switch (pid = fork()) {
				case 0: {
				char *S;

				if (input_result_msg == NULL) {
					(void) fprintf( stderr,
						"ELMAC: check_access_proc_msg (fail). "
						"Could not memmory alloc for access "
						"value (%s).\n",
						strerror(errno));
					exit(1);
				}

				memset(input_result_msg, 0, 1);

				label.level = l;
				label.category = c;

				rc = elmac_set_label_for_self(&label);
				if(rc < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_msg "
						"(fail). Could not set ELMAC "
						"label (%d,%llx). \n",
						l, c);
					exit(1);
				}
				if (setgid(tester_gid) < 0 ||
				    setuid(tester_uid) < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_msg "
						"(fail). Cannot set uid %i "
						"gid %i, errno %d (%s) pid %i\n",
						tester_uid,
						tester_gid,
						errno, strerror(errno), getpid());
					exit(1);
				}
				if (check_access_msg(l, c, &S) != 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_msg "
						"(fail). check_access_shm() != 0\n");
					exit(1);
				}
				len = strlen(S);
				write(fd[1], &len, sizeof(len));
				write(fd[1], (void *) S, len);

				exit(0);
				}
				case -1:
				(void) fprintf(stderr,
						"ELMAC: check_access_proc_msg "
						"(fail). Cannot fork() "
						"errno %d (%s)\n",
					       errno, strerror(errno));
				break;
				default:
repeat_waitpid_msg_proc:
				read(fd[0], &len, sizeof(size_t));
				if (len) {
					char *S;
					size_t len_tmp;

					S = malloc(len + 1);
					if (S == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_msg (fail). "
							"malloc error.\n");
							exit (1);
					}
					memset(S, 0, len + 1);
					len_tmp = strlen(input_result_msg);
					input_result_msg =
						(char *) realloc (input_result_msg,
						 len_tmp + len + 1);
					if (input_result_msg == (char *)NULL) {
						(void) fprintf(stderr,
							"ELMAC: "
							" check_access_proc_msg (fail). "
							"realloc error.\n");
							exit (1);
					}
					read(fd[0], S, len);
					strcat (input_result_msg, S);
					close(fd[0]);
				} else
					exit(1);

				pidwait = waitpid(pid, &status, 0);
				if (pidwait < 0) {
					(void) fprintf(stderr,
						"ELMAC: check_access_proc_msg. "
						"waitpid < 0 errno %d (%s)\n",
						errno, strerror(errno));
				if (errno == 4)
					goto repeat_waitpid_msg_proc;
				}
				if (WIFEXITED(status))
					print_d(p_print_msg,
						"ELMAC: check_access_proc_msg. "
						"Parent %i waited child %i "
						"WEXITSTATUS(status): %i\n",
						getpid(), pidwait,
						WEXITSTATUS(status));
				if (WIFSIGNALED(status))
					print_d(p_print_msg,
						"ELMAC: check_access_proc_msg. "
						"Parent %i waited child %i "
						"WTERMSIG(status): %i\n",
						getpid(), pidwait,
						WTERMSIG(status));
				if (WIFSTOPPED(status))
					print_d(p_print_msg,
						"ELMAC: check_access_proc_msg. "
	 					"Parent %i waited child %i "
						"WSTOPSIG(status): %i\n",
						getpid(), pidwait,
						WSTOPSIG(status));
				}
			}
		}
	}
	len = strlen(input_result);
	len += strlen(input_result_msg);
	input_result = (char *) realloc (input_result, len + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
				"ELMAC: "
				" check_access_proc_msg (fail). "
				"realloc error.\n");
				exit (1);
	}
	strcat (input_result, input_result_msg);

	return (0);
}




int main()
{
	size_t len;
	struct passwd	*passwd_user;


	if ((geteuid() != 0) || (getuid() != 0)) {
		(void) fprintf(stderr,
			"ELMAC: You must be root !!! \n");
		return (1);
	}

	if ((passwd_user = getpwnam(USER_MAC_TESTER)) == NULL) {
		(void) fprintf(stderr,
			"ELMAC fail:: not tests user: %s\n", USER_MAC_TESTER);
		return (1);
	}

    if (prepare_test_dir()) {
        return (1);
    }

	tester_uid = passwd_user->pw_uid;
	tester_gid = passwd_user->pw_gid;

	lev_min = MIN_LEVEL;
	lev_max = MAX_LEVEL;
	cat_set = MAX_CAT;

	if (lev_min < 0 || lev_max < 0 || lev_min > lev_max) {
		(void) fprintf( stderr,
				"ELMAC: main (fail). "
				"Bad level (lev_min: %d, lev_max: %d).\n",
				lev_min, lev_max);
		return (1);
	}

	/**
	 * Add cat_set < 8 bit
	 */

	if (cat_set < 0) {
		(void) fprintf( stderr,
				"ELMAC: main (fail). "
				"category set(cat_set: %llx).\n",
				cat_set);
		return (1);
	}

	cat_num = get_cat_num();
	if (cat_num > CAT_NUM_TEST_MAX) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"cat_num(%d) > CAT_NUM_TEST_MAX(%d)\n",
			cat_num, CAT_NUM_TEST_MAX);
		return (1);
	}

	input_result = (char *) malloc(1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"malloc error.\n");
		return (1);
	}
	memset(input_result, 0, 1);

/**
 * File
 */
	//printf("\nFiles.\n");
	len = strlen(input_result);
	input_result = (char *) realloc (input_result, len + strlen("\nFiles.\n") + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"realloc error.\n");
		return (1);
	}
	strcat (input_result, "\nFiles.\n");

	if ((creat_file_obj()) != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"creat_file_obj() != 0\n");
		return (1);
	}

	if ((check_access_proc_file()) != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"check_access_proc_file() != 0\n");
        rm_testdir();
		return (1);
	}

    rm_testdir();

	ipc_mem_index_cat_array_mem =
			(unsigned char *) malloc(cat_num * sizeof(elmac_cat_t));
	if (ipc_mem_index_cat_array_mem == NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"Memmory alloc from shmem index cat array == NULL\n");
		return (1);
	}

	ipc_mem_index_cat_array = (elmac_cat_t*) ipc_mem_index_cat_array_mem;

	if (!set_index_cat()) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"set_index_cat error(bad index)\n");
		return (1);
	}

	ipc_mem_id_array_size = (lev_max - lev_min + 1) * cat_num * sizeof(int);

	ipc_mem_id_array = (int *) malloc(ipc_mem_id_array_size);
	if (ipc_mem_id_array == NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"Memmory alloc from shmem id array == NULL\n");
		return (1);
	}

/**
 * Shmem
 */
	srand(time(NULL));
	i_shm_create = rand();

	//printf("\nIPC. Shmem.\n");
	len = strlen(input_result);
	input_result = (char *) realloc (input_result, len + strlen("\nIPC. Shmem.\n") + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"realloc error.\n");
		return (1);
	}
	strcat (input_result, "\nIPC. Shmem.\n");

	print_d(p_print_shm,"\nShmem (begin id: %d).\n", i_shm_create);

	memset(ipc_mem_id_array, 0, ipc_mem_id_array_size);

	if (creat_shm_obj() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"creat_shm_obj() != 0\n");
		return (1);
	}

	if (check_access_proc_shm() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"check_access_proc_shm() != 0\n");
		return (1);
	}

	if (delete_shm() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"delete_shm() != 0\n");
		return (1);
	}

/**
 * Semaphore
 */
	srand(time(NULL));
	i_sem_create = rand();

	//printf("\nIPC. Semaphore.\n");
	len = strlen(input_result);
	input_result = (char *) realloc (input_result, len + strlen("\nIPC. Semaphore.\n") + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"realloc error.\n");
		return (1);
	}
	strcat (input_result, "\nIPC. Semaphore.\n");

	print_d(p_print_sem,"\nSemaphore (begin id: %d).\n", i_sem_create);

	memset(ipc_mem_id_array, 0, ipc_mem_id_array_size);

	if (creat_sem_obj() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"creat_sem_obj() != 0\n");
		return (1);
	}

	if (check_access_proc_sem() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"check_access_proc_sem() != 0\n");
		return (1);
	}

	if (delete_sem() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"delete_sem() != 0\n");
		return (1);
	}
/**
 * Messages
 */
	srand(time(NULL));
	i_msg_create = rand();

	//printf("\nIPC. Messages.\n");
	len = strlen(input_result);
	input_result = (char *) realloc (input_result, len + strlen("\nIPC. Messages.\n") + 1);
	if (input_result == (char *)NULL) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"realloc error.\n");
		return (1);
	}
	strcat (input_result, "\nIPC. Messages.\n");
	print_d(p_print_msg, "\nMessages (begin id: %d).\n", i_msg_create);

	memset(ipc_mem_id_array, 0, ipc_mem_id_array_size);

	if (creat_msg_obj() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"creat_msg_obj() != 0\n");
		return (1);
	}

	if (send_msg_obj() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"send_msg_obj() != 0\n");
		return (1);
	}

	if (check_access_proc_msg() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"check_access_proc_msg() != 0\n");
		return (1);
	}

	if (delete_msg() != 0) {
		(void) fprintf(stderr,
			"ELMAC: main (fail). "
			"delete_msg() != 0\n");
		return (1);
	}

	printf("%s", input_result);
	if (input_result != NULL)
		free(input_result);

	free(ipc_mem_index_cat_array_mem);
	free(ipc_mem_id_array);

	return 0;
}
