#!/bin/bash

set +xv

CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TMP_DIR=${TC_DIR}/tmp
CFG_DIR=${TC_DIR}/cfg
ENV_FILE=${CFG_DIR}/env

HOME_MAC=${TMP_DIR}/home_mac

tmpf=/tmp/_trust_tmpf

CWD=$(pwd)

macadm_start_flag=false
macadm_started=false
mac_enable=false

typeset -f UserAdd
typeset -f UserDel

verbose=0
################################################################################
usage()
{
	echo " usage: $progname [-v]"
	echo
	exit 1
}
OPTIONS="vC"
while getopts $OPTIONS option
do
	case $option in
		v) verbose=1; opt_v="-v";;
		C) clean_flg=1;;
		*) echo "$prog: illegal option -- $option"; usage; exit 1;;
	esac
done
################################################################################
DoEcho()
{
	echo "$progname: === $*"
}
DoEchoVer()
{
	[[ $verbose == 1 ]] && DoEcho $*
}
CleanUp()
{
	if [[ ${macadm_started} != true ]]; then
		/sbin/macadm stop > /dev/null 2>&1
	fi
	rm -f $tmpf
}
SigHandler()
{
	CleanUp
	DoExit 1
}
DoExit()
{
	CleanUp
	[[ ! -z $2 ]] && DoEcho $2
	[[ $1 == 0 ]] && DoEcho "PASS"
	[[ $1 != 0 ]] && DoEcho "FAIL"
	cd $CPWD
	exit $1
}
DoCmd()
{
	local cmd ret
	cmd="$*"
	DoEchoVer "$cmd"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		DoEcho "$cmd - returned $ret"
		cat $tmpf
		DoExit 1
	fi
	[[ $verbose == 1 ]] && cat $tmpf
	return 0
}
UserAdd()
{
	local user uid
	user=$1
	uid=$2
	DoCmd mkdir -p $HOME_MAC/$user
	DoCmd useradd -c test_mac -d $HOME_MAC/$user -G 100 -s /bin/bash -u $uid $user
	DoCmd chown -R $user:100 $HOME_MAC/$user
}
UserDel()
{
	[[ $verbose == 1 ]] && DoEcho userdel -r $1
	userdel -r $1 >$tmpf 2>&1
	[[ $verbose == 1 ]] && cat $tmpf
}
GroupDel()
{
	[[ $verbose == 1 ]] && DoEcho userdel -r $1
	groupdel $1 >$tmpf 2>&1
	[[ $verbose == 1 ]] && cat $tmpf
}
Clean()
{
	DoCmd rm -f /etc/HOSTNAME
	DoCmd rm -f tmac.cnf
	UserDel usra
	UserDel usrb
	UserDel usrc
	UserDel usrd
	UserDel usre
	UserDel usrf
	UserDel usrg
	UserDel usri
	UserDel usrj

	GroupDel usra
	GroupDel usrb
	GroupDel usrc
	GroupDel usrd
	GroupDel usre
	GroupDel usrf
	GroupDel usrg
	GroupDel usri
	GroupDel usrj
	
	UserDel macdef
	GroupDel macdef
	if [[ -d /etc/security/_mac ]]; then
		DoCmd rm -rf /etc/security/mac
		DoCmd cp -r /etc/security/_mac  /etc/security/mac
		DoCmd rm -rf /etc/security/_mac
	fi
}

trap "SigHandler" "SIGINT"

[[ ! -f /sbin/macadm ]] && DoExit 1 "cannot find /sbin/macadm"
[[ ! -x /sbin/macadm ]] && DoExit 1 "/sbin/macadm is not executable"
[[ ! -d /etc/security/mac ]] && DoExit 1 "cannot find /etc/security/mac"

if [[ $clean_flg == 1 ]]; then
	Clean
	exit 0
fi

DoCmd rm -rf $TMP_DIR
DoCmd mkdir -p -m 0777 $TMP_DIR
DoCmd mkdir -p $HOME_MAC

if [[ ! -f /etc/HOSTNAME ]]; then
	Clean
	if [[ ! -d /etc/security/_mac ]]; then 
		DoCmd cp -r /etc/security/mac  /etc/security/_mac
	fi
	DoCmd tar xf mac.tar
	DoCmd cp -r mac /etc/security/
	DoCmd rm -rf mac
	host=`uname -n`
	cat /etc/security/mac/mac_policy | sed -e "s/yerevan/`uname -n`/g" >${tmpf}
	cp ${tmpf} /etc/security/mac/mac_policy
	cat /etc/security/mac/mac_hosts | sed -e "s/yerevan/`uname -n`/g" >${tmpf}
	cp ${tmpf} /etc/security/mac/mac_hosts
	if [[ ! -f tmac.cnf ]]; then
		cp tmac.cnf.orig tmac.cnf
		echo HOST : `uname -n` >> tmac.cnf
	fi
	DoEcho "create test users"
	UserAdd usra 140
	UserAdd usrb 141
	UserAdd usrc 142
	UserAdd usrd 143
	UserAdd usre 144
	UserAdd usrf 145
	UserAdd usrg 146
	UserAdd usri 147
	UserAdd usrj 148
	UserAdd macdef 149
	uname -n > /etc/HOSTNAME
fi

[[ $verbose == 1 ]] && DoEcho lsmod
lsmod >$tmpf 2>&1
if [[ $? != 0 ]]; then
	DoEcho "cannot execute lsmod"
	cat $tmpf
	DoExit 1
else
	[[ $verbose == 1 ]] && cat $tmpf
fi
cat $tmpf | grep "^macctl" >/dev/null 2>&1
if [[ $? != 0 ]]; then
	DoCmd insmod /lib/modules/$(uname -r)/extra/macctl.ko
	[[ $verbose == 1 ]] && DoEcho lsmod
	lsmod >$tmpf 2>&1
	if [[ $? != 0 ]]; then
		DoEcho "cannot execute lsmod"
		cat $tmpf
		DoExit 1
	else
		[[ $verbose == 1 ]] && cat $tmpf
	fi
fi

DoEcho "/sbin/macadm test"
DoCmd /sbin/macadm test

grep "MAC is active" $tmpf > /dev/null 2>&1
if [[ $? == 0 ]]; then
	macadm_started=true
else
	DoEcho "/sbin/macadm start"
	DoCmd /sbin/macadm start
	macadm_start_flag=true
fi

export ROOT_E90=$CWD
[[ $verbose == 1 ]] && DoEcho ROOT_E90=$ROOT_E90

DoEcho "./tmac 1"
$CWD/tmac 1 >${TMP_DIR}/tmac_ret 2>&1
ret=$?
if [[ $ret != 0 ]]; then
	DoEcho "./tmac 1 - returned $ret"
	if [[ $verbose == 1 ]]; then
		cat ${TMP_DIR}/tmac_ret
	else
		DoEcho "RESULTS - ${TMP_DIR}/tmac_ret"
	fi
	DoExit 1
fi 

DoCmd diff ${TMP_DIR}/tmac_ret $CWD/tmac_ret_etalon

DoExit 0
