#!/bin/bash

set +xv

CPWD=$(pwd)
progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TEST_NAME=mem_tc
TMP_DIR=${TC_DIR}/tmp
CFG_DIR=${TC_DIR}/cfg
ENV_FILE=${CFG_DIR}/env


tmpf=/tmp/_elmac_tmpf

count=100
size=0x100000
mode=malloc
debug=0


CWD=$(pwd)

verbose=0
################################################################################
usage()
{
	echo " usage: $progname [-v [-s SIZE_ALLOC] [-c COUNT_ITER ] [-m TEST_METHOD]]"
	echo "        SIZE_ALLOC  - max 0x100000. Not relevant COW. "
	echo "        COUNT_ITER  - if 0, allocated all free memmory. Not relevant COW."
	echo "        TEST_METHOD - sbrk, malloc, mmap, COW"
	echo
	exit 1
}
OPTIONS="vs:c:m:"
while getopts $OPTIONS option
do
	case $option in
		v) verbose=1; debug=-d;;
		s) size=${OPTARG};;
		c) count=${OPTARG};;
		m) mode=${OPTARG};;
		*) echo "$prog: illegal option -- $option"; usage; exit 1;;
	esac
done
################################################################################
DoEcho()
{
	echo "$progname: === $*"
}
DoEchoVer()
{
	[[ $verbose == 1 ]] && DoEcho $*
}
SigHandler()
{
	DoExit 1
}
DoExit()
{
	[[ ! -z $2 ]] && DoEcho $2
	[[ $1 == 0 ]] && DoEcho "PASS"
	[[ $1 != 0 ]] && DoEcho "FAIL"
	cd $CPWD
	exit $1
}
DoCmd()
{
	local cmd ret
	cmd="$*"
	DoEchoVer "$cmd"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		DoEcho "$cmd - returned $ret"
		cat $tmpf
		DoExit 1
	fi
	[[ $verbose == 1 ]] && cat $tmpf
	return 0
}

trap "SigHandler" "SIGINT"

export ROOT_E90=$CWD
[[ $verbose == 1 ]] && DoEcho ROOT_E90=$ROOT_E90

DoCmd rm -rf $TMP_DIR
DoCmd mkdir -p -m 0777 $TMP_DIR


DoEcho "./${TEST_NAME} ${debug} -s ${size} -c ${count} -m ${mode}"
DoCmd $CWD/mem_tc ${debug} -s ${size} -c ${count} -m ${mode}


DoExit 0
