#!/bin/bash
set +x
#Test pam

# DEFENITION
###############################################################################
date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
short_date_stamp=$(date "+%y%m%d%H%M%S" 2>/dev/null)

progname=$(basename $0 2>/dev/null)

cd $(dirname $0)

cpwd=$(pwd)
#log="${cpwd}/trace.log"
log=
utils=pam

tmp_dir="${cpwd}/tmp"
tmp_file_output="${tmp_dir}/tmp_output"
tmp_file_res="${tmp_dir}/tmp_res"


arch=
user=

export su_user=su_user_test

verbose=0
debug=0
###############################################################################


# Usage
###############################################################################
usage()
{
        echo " usage: $progname [-vdh]"
        echo
        exit 1
}
################################################################################

# Echo
###############################################################################
DoEcho()
{
	if [ $verbose != 0 ]; then
		echo -e "${progname}: === $*"
	fi
	echo -e "${progname}: === $*" >> $log
}
###############################################################################

# Result checking
###############################################################################
DoCheckResult()
{
	local return_code_to_check="$1"

	if [ "$return_code_to_check" -ne 0 ]; then
		exit "$return_code_to_check"
        fi
}
###############################################################################


# Input text formating
###############################################################################
DoInputRes()
{
	DoEcho "   | Output command:"
	while read LINE; do DoEcho "   | $LINE"; done < ${tmp_file_output}
	DoEcho "   | Result command:"
	if [ "$1" -eq 0 ]; then
		DoEcho "   | PASS"
	else
		DoEcho "   | FAIL"
	fi
}
###############################################################################

# Operation
###############################################################################
DoCmd()
{
	unset do_cmd_out
	unset do_cmd_out_add
	unset do_cmd_ret_code

	sleep 0.5

	local cmd="$@"

	if [ -d ${tmp_dir} ]; then
		echo "" > ${tmp_file_output}
	fi

	let "oper = oper + 1"
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | ${cmd}"
	do_cmd_out=$(eval $cmd 2>>$log)
	do_cmd_ret_code=$?
	if [ -d ${tmp_dir} ]; then
		echo "$do_cmd_out" > ${tmp_file_output}
		DoInputRes $do_cmd_ret_code
	fi
	return "$do_cmd_ret_code"
}

###############################################################################




# Init
###############################################################################
DoInit()
{
#	oper=0
	DoEcho ""
	DoEcho "***************************************************************************"
	DoEcho " Init test"
	DoEcho "***************************************************************************"

	DoEcho "Making tmp dir"
	DoCmd mkdir -p ${tmp_dir}
	DoCheckResult "$?"

	DoEcho "Arch settings"
	DoCmd "uname -m"
	DoCheckResult "$?"
	arch=$do_cmd_out

	DoEcho "Checking user"
	DoCmd "whoami"
	DoCheckResult "$?"
	arch=$do_cmd_out

	DoEcho "Creating test users"
	userdel ${su_user} > /dev/null 2>/dev/null
	DoCmd useradd ${su_user}
	DoCheckResult "$?"



}
###############################################################################


# User creating
###############################################################################
AddUser()
{
	DoEcho "Add user: $1"
	DoCmd "useradd -u 777 $1"
	DoCheckResult "$?"
}

# User deleting
###############################################################################
DelUser()
{
	DoEcho "Deleting user: $1"
	DoCmd "userdel $1"
	DoCheckResult "0"
}

# Group deleting
###############################################################################
DelGroup()
{
	DoEcho "Deleting group: $1"
	DoCmd "groupdel $1"
	DoCheckResult "0"
}

# Clean
###############################################################################
DoCleanUp()
{
#	oper=0
	DoEcho "***************************************************************************"
	DoEcho " Cleanup test"
	DoEcho "***************************************************************************"


	DelUser ${su_user}

}
###############################################################################


trap "DoExit" EXIT
trap "exit 127" HUP INT QUIT PIPE TERM

###############################################################################
DoExit()
{
	local exit_code="$?"
	DoCleanUp
	date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
	DoEcho "Stopping PAM TEST at $date_stamp"
	if [ "$exit_code" -eq 0 ]; then
		DoEcho "PASS"
	else
		DoEcho "FAIL"
	fi
}
###############################################################################





# MAIN
###############################################################################

OPTIONS="vdh"
while getopts $OPTIONS option
do
        case $option in
                v) verbose=1;;
                d) debug=1;;
		h) echo -e "\v - verbose"
	           echo -e "\d - debug mode (set -x)"
		   echo -e "\h - help"
		   exit 1;;
                *) echo "$prog: illegal option -- $option"; usage; exit 1;;
        esac
done


log="${cpwd}/trace.${utils}.log"
if [ -f ${log} ]; then
	DoEcho "Removing old log"
	DoCmd rm  ${log}
	DoCheckResult "$?"
fi


DoEcho "Starting PAM TEST at $date_stamp"

DoInit

DoEcho ""
DoEcho "***************************************************************************"
DoEcho "1) Checking PAM "
DoEcho "***************************************************************************"

oper=0

DoEcho "Duplicate out auth log in ${tmp_file_res}"
tail -F -n 0 /var/log/auth.log > ${tmp_file_res} 2>/dev/null  &
log_pid=$!

DoEcho "Creating a session for the user ${su_user}. Searching PID session."
DoCmd "su -c 'ps -u $su_user -o ppid,pid 2> /dev/null | grep -m 1 \$$ 2> /dev/null | sed -e \"s|^ *||g\" 2> /dev/null | cut -f1 -d\" \" 2> /dev/null' -m $su_user | grep -v $su_user"
DoCheckResult "$?"
su_pid=$do_cmd_out
DoEcho "su PID ${su_pid}"

a=`file /var/log/auth.log | awk -F":" '{print $2}'`
if [ "$a" != " ASCII text" ]; then
        DoEcho "File /var/log/auth.log is of a type other than ASCII text."
fi

kill -15 ${log_pid} > /dev/null 2>/dev/null

DoEcho "Finding information about the opening and closing session of the  user ${su_user}"
DoCmd " grep -a ${su_user}  ${tmp_file_res} | grep \"session opened\""
DoCheckResult "$?"
DoCmd " grep -a ${su_user}  ${tmp_file_res} | grep \"session closed\""
DoCheckResult "$?"
