#!/bin/bash

set +xv

cd $(dirname $0)
cpwd=$(pwd)
progname=$(basename $0 2>/dev/null)

timeout=$(head -n 1 ../timeout)

cfg_file="../scen.cp"

log="${cpwd}/trace.config.log"
if [ -f ${log} ]; then
    rm ${log}
fi

l_echo()
{
	echo -e "${progname}: === $*" >> $log
}


ver=$(uname -r 2>/dev/null)
ret=$?
if [[ $ret != 0 ]]; then
	l_echo "Unknown kernel"
	exit 1
fi
l_echo "Kernel version $ver"

m_ver=$(uname -r | awk -F. '{print $1}')
ret=$?
if [[ $ret != 0 ]]; then
	l_echo "Unknown kernel major number"
	exit 1
fi
l_echo "Kernel version major number ${m_ver}"

fs_type_enable=""

if ! zcat /proc/config.gz 2>/dev/null | grep "CONFIG_EXT3_FS="; then
    l_echo "The kernel not support EXT3 fs"
else
    l_echo "The kernel support EXT3 fs"
    fs_type_enable=$(echo ${fs_type_enable} ext3)
fi

if [ "$m_ver" = "2" ]; then
    if ! zcat /proc/config.gz 2>/dev/null | grep "CONFIG_EXT2_FS="; then
        l_echo "The kernel not support EXT2 fs"
    else
        l_echo "The kernel support EXT2 fs"
        fs_type_enable=$(echo ext2 ${fs_type_enable})
    fi
else
    if ! zcat /proc/config.gz 2>/dev/null | grep "CONFIG_EXT4_USE_FOR_EXT2="; then
        l_echo "The kernel not support EXT2 over EXT4 fs"
    else
        l_echo "The kernel support EXT2 over EXT4 fs"
        fs_type_enable=$(echo ext2 ${fs_type_enable})
    fi

    if ! zcat /proc/config.gz 2>/dev/null | grep "CONFIG_EXT4_FS="; then
        l_echo "The kernel not support EXT4 fs"
    else
        l_echo "The kernel support EXT4 fs"
        fs_type_enable=$(echo ${fs_type_enable} ext4)
    fi
fi

echo ${fs_type_enable}

utils=""

if ! which srm 2>/dev/null; then
	l_echo "srm not found"
else
    l_echo "srm is present"
    utils=$(echo ${utils} srm)
fi

if ! which shred 2>/dev/null; then
	l_echo "shred not found"
else
    l_echo "shred is present"
    utils=$(echo ${utils} shred)
fi
echo "utils ${utils}"

rm ${cfg_file} 2>/dev/null

for utils_iter in  ${utils}; do
    for fs_iter in  ${fs_type_enable}; do
        echo "root:   ./bin/secdel_tc -u ${utils_iter} -t ${fs_iter} -T ${timeout} " >> ${cfg_file}
    done
done

secdel_type=""

if [ "$m_ver" = "2" ]; then
    secdel_type="secdel hard_secdel"
else
    secdel_type="secdel secdel=3 secdel_rand secdel_rand=3 secdel_shn"
fi
for fs_iter in  ${fs_type_enable}; do
    for type_iter in ${secdel_type}; do
        echo "root:   ./bin/secdel_tc -u kernel_del -t ${fs_iter} -r ${type_iter}  -T ${timeout} " >> ${cfg_file}
    done
done

if ! grep "secdel_tc" ${cfg_file} 2>/dev/null; then
	l_echo "Config incorrect"
	exit 1
fi

l_echo "Create config for $ver success"

exit 0
