#!/bin/sh

#Test srm

set +x

# DEFENITION
###############################################################################
date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
short_date_stamp=$(date "+%y%m%d%H%M%S" 2>/dev/null)

progname=$(basename $0 2>/dev/null)
working_dir="/tmp/audit_test_${date_stamp}"

cd $(dirname $0)

cpwd=$(pwd)
#log="${cpwd}/trace.log"
log=

tmp_dir="${cpwd}/tmp"
tmp_file_output="${tmp_dir}/tmp_output"
tmp_file_res="${tmp_dir}/tmp_res"

test_image="/tmp/image-$$"
test_mntpoint="/tmp/mntpoint-$$"

TYPE_FS="undef"
TYPE_DEL="undef"

arch=
user=

verbose=0
debug=0
###############################################################################


# Usage
###############################################################################
usage()
{
        echo " usage: $progname [-vdhtr]"
        echo
        exit 1
}
################################################################################

# Echo
###############################################################################
DoEcho()
{
	if [ $verbose != 0 ]; then
		echo -e "${progname}: === $*"
	fi
	echo -e "${progname}: === $*" >> $log
}
###############################################################################

# Result checking
###############################################################################
DoCheckResult()
{
	local return_code_to_check="$1"

	if [ -d ${tmp_dir} ]; then
		DoInputRes $return_code_to_check
	fi

	if [ "$return_code_to_check" -ne 0 ]; then
		exit 1
	fi
}
###############################################################################


# Input text formating
###############################################################################
DoInputRes()
{
	DoEcho "   | Output command:"
	while read LINE; do DoEcho "   | $LINE"; done < ${tmp_file_output}
	DoEcho "   | Result command:"
	if [ "$1" -eq 0 ]; then
		DoEcho "   | PASS"
	else
		DoEcho "   | FAIL"
	fi
}
###############################################################################

# Clean
###############################################################################
DoCleanUp()
{
#	oper=0
DoEcho "***************************************************************************"
	DoEcho " Cleanup test"
DoEcho "***************************************************************************"

	mount | grep ${test_mntpoint} 2>/dev/null
	res="$?"
	if [ "$res" -eq 0 ]; then
		DoCmd   "umount ${test_mntpoint}"
	fi

	DoCmd "rm -f ${test_image}"
	DoCheckResult "$?"
	DoCmd "rmdir ${test_mntpoint}"
	DoCheckResult "$?"
	if [ -d ${working_dir} ]; then
		DoCmd "rm -rf ${working_dir}"
		DoCheckResult "$?"
	fi
	if [ -d ${tmp_dir} ]; then
		DoCmd "rm -rf ${tmp_dir}"
		DoCheckResult "$?"
	fi
	DoCheckResult "$?"

}
###############################################################################


trap "DoExit" EXIT
trap "exit 127" HUP INT QUIT PIPE TERM

###############################################################################
DoExit()
{
	local exit_code="$?"
	DoCleanUp
	date_stamp=$(date "+%Y-%m-%d_%H-%M-%S" 2>/dev/null)
	DoEcho "Stopping SECURE DELETING TEST at $date_stamp"
	if [ "$exit_code" -eq 0 ]; then
		DoEcho "PASS"
	else
		DoEcho "FAIL"
	fi
}
###############################################################################

# Operation
###############################################################################
DoCmd()
{
	unset do_cmd_out
	unset do_cmd_out_add
	unset do_cmd_ret_code

	local cmd="$@"

	if [ -d ${tmp_dir} ]; then
		echo "" > ${tmp_file_output}
	fi

	#let "oper = oper + 1"
	oper=$((oper + 1))
	DoEcho "   --------------------------------Step $oper---------------------------------"
	DoEcho "   | Command:"
	DoEcho "   | ${cmd}"
	do_cmd_out=$(eval $cmd 2>>$log)
	do_cmd_ret_code=$?
	if [ -d ${tmp_dir} ]; then
		echo "$do_cmd_out" > ${tmp_file_output}
	fi
	return "$do_cmd_ret_code"
}

###############################################################################




# Init
###############################################################################
DoInit()
{
#	oper=0
	DoEcho ""
	DoEcho "***************************************************************************"
	DoEcho " Init test"
	DoEcho "***************************************************************************"

	DoEcho "Load loop module"
	DoCmd "modprobe loop"
	DoCheckResult "$?"

	DoEcho "Making tmp dir"
	DoCmd "mkdir -p $tmp_dir"
	DoCheckResult "$?"

	DoEcho "Arch settings"
	DoCmd "uname -m"
	DoCheckResult "$?"
	arch=$do_cmd_out

	DoEcho "Checking User"
	DoCmd "whoami"
	DoCheckResult "$?"
	arch=$do_cmd_out

	DoEcho "Checking Utils"
	if [ "$1" != "kernel_del" ]; then
		DoCmd "which ${utils}"
	fi
	DoCheckResult "$?"

	if [ "$1" = "srm" ]; then
		RM="srm -l"
		res=0
	elif  [ "$1" = "shred" ]; then
		RM="shred -u"
		res=0
	elif [ "$1" = "kernel_del" ]; then
		RM="rm"
		res=0
	else
		DoEcho "   | Select correct utils"
		res=1
	fi
	DoCheckResult "$res"
}
###############################################################################



# MAIN
###############################################################################

TIMEOUT=1

OPTIONS="vdhu:t:r:T:"
while getopts $OPTIONS option; do
	case $option in
		v) verbose=1;;
		d) debug=1;;
		u) utils=${OPTARG};;
		t) TYPE_FS=${OPTARG};;
		r) TYPE_DEL=",${OPTARG}";;
		T) TIMEOUT="${OPTARG}";;
		h)	echo -e "\v - verbose"
			echo -e "\d - debug mode (set -x)"
			echo -e "\h - help"
			echo -e "\t - type filesystem(ext2,ext3,ext4(only kernel >= 3.14))"
			echo -e "\r - Deletind method kernel 2.6.33:"
			echo -e "                     secdel, hard_secdel"
			echo -e "\r - Deletind method kernel >= 3.14:"
			echo -e "                     secdel, secdel={count},"
			echo -e "                     secdel_rand, secdel_rand={count},"
			echo -e "                     secdel_shn)"
			echo -e "\T - umount timeout (sec)"
			exit 1;;
		*) echo "$prog: illegal option -- $option"; usage; exit 1;;
	esac
done


log="${cpwd}/trace.${utils}.log"
if [ -f ${log} ]; then
	DoEcho "Remove old log"
	DoCmd "rm ${log}"
	DoCheckResult "$?"
fi

if [ "${TYPE_FS}" = "undef" ]; then
	DoEcho "   | Type FS not set. Use ext3"
	echo -e "=== Type FS not set. Use ext3"
	TYPE_FS="ext3"
fi

if [ "$utils" = "kernel_del" ]; then
	if [ "${TYPE_DEL}" = "undef" ]; then
		DoEcho "   | Deletind method not set"
		echo -e "=== Deletind method not set"
		TYPE_DEL=
	fi
fi


DoEcho "Starting SECURE DELETING TEST at $date_stamp"

DoInit ${utils}

DoEcho ""
DoEcho "***************************************************************************"
DoEcho "1) Checking ${utils} utils"
DoEcho "***************************************************************************"

oper=0

DoEcho "Creating image. Operation if=/dev/zero of=${test_image} bs=5M count=1"
DoCmd 	"dd if=/dev/zero of=${test_image} bs=5M count=1"
DoCheckResult "$?"

DoEcho "Creating FS. Operation mke2fs -t ${TYPE_FS} ${test_image}"
DoCmd 	"mke2fs -F -t ${TYPE_FS} ${test_image}"
#DoCmd	"echo "y" | mke2fs -t ${TYPE_FS} "${test_image}""
DoCheckResult "$?"

DoEcho "Creating mount point. Operation mkdir -p ${test_mntpoint}"
DoCmd 	"mkdir -p ${test_mntpoint}"
DoCheckResult "$?"

DoEcho "Image mounting. Operation mount -t ${TYPE_FS} -o loop,sync ${test_image} ${test_mntpoint}"
DoCmd 	"mount -t ${TYPE_FS} -o loop,sync ${test_image} ${test_mntpoint}"
DoCheckResult "$?"

DoEcho "Creating a signature testfile. Operation echo \"Hello world\" > ${test_mntpoint}/file.txt"
DoCmd 	"echo 'Hello world' > ${test_mntpoint}/file.txt"
DoCheckResult "$?"

sleep ${TIMEOUT}
DoEcho "Image umounting. Operation umount ${test_mntpoint}"
DoCmd 	"umount ${test_mntpoint}"
DoCheckResult "$?"

DoEcho "Signature presence search in the image.Operation fgrep Hello world ${test_image}"
DoCmd 	"fgrep 'Hello world' ${test_image}"
DoCheckResult "$?"

DoEcho "Image mounting. Operation mount -t ${TYPE_FS} -o loop,sync ${test_image} ${test_mntpoint}"
DoEcho "$utils"
if [ "$utils" = "kernel_del" ]; then
	DoCmd 	"mount -t ${TYPE_FS} -o loop,sync${TYPE_DEL} ${test_image} ${test_mntpoint}"
	ret="$?"
else
	DoCmd 	"mount -t ${TYPE_FS} -o loop,sync ${test_image} ${test_mntpoint}"
	ret="$?"
fi
DoCheckResult "$ret"


DoEcho "Testfile deleting. Operation $RM ${test_mntpoint}/file.txt"
DoCmd 	"$RM ${test_mntpoint}/file.txt"
DoCheckResult "$?"

sleep ${TIMEOUT}
DoEcho "Image umounting. Operation umount ${test_mntpoint}"
DoCmd 	"umount ${test_mntpoint}"
DoCheckResult "$?"

DoEcho "Signature presence search in the image. Operation fgrep \"Hello world\" ${test_image}"
DoCmd 	"fgrep 'Hello world' ${test_image}"
if [ "$?" -ne 0 ]; then
	ret=0
else
	ret=1
fi
DoCheckResult "$ret"
